@@ -53,6 +53,24 @@ their specific print lengths, if they have one.")
   (declare (ignore args))
   (setf (gethash (obj frame) (dico frame)) t))
 
+;; Remember the scrolling state between redisplays.
+(defmethod redisplay-frame-panes :around ((frame inspector) &key force-p)
+  (declare (ignore force-p))
+  ;; `Make-clim-stream-pane' creates bizarro object hierarchies, so
+  ;; getting the actual scrollable is not obvious.
+  (let* ((scrollable-pane (sheet-parent (find-pane-named frame 'app)))
+         (viewport (pane-viewport scrollable-pane)))
+    (multiple-value-bind (x-displacement y-displacement)
+        (transform-position (sheet-transformation scrollable-pane) 0 0)
+      (call-next-method)
+      (scroll-extent scrollable-pane
+                     (min (- x-displacement)
+                          (- (bounding-rectangle-width scrollable-pane)
+                             (bounding-rectangle-width viewport)))
+                     (min (- y-displacement)
+                          (- (bounding-rectangle-height scrollable-pane)
+                             (bounding-rectangle-height viewport)))))))
+
 (defmethod redisplay-frame-pane :after ((frame inspector)
 					(pane application-pane)
 					&key force-p)
@@ -67,11 +85,20 @@ their specific print lengths, if they have one.")
 		 (*print-level* 10))
 	     (run-frame-top-level
 	      (make-application-frame 'inspector :obj obj)))))
+    
+    (when (typep *application-frame* 'inspector)
+      (restart-case (error "Clouseau called from inside Clouseau, possibly infinite recursion")
+        (continue ()
+         :report "Continue by starting a new Clouseau instance")
+        (abort-clouseau ()
+         :report "Abort this call to Clouseau"
+         (return-from inspector))))
     (if new-process
 	(clim-sys:make-process #'run
-			       :name (format nil "Inspector Clouseau: ~S"
-					     obj))
-	(run))))
+         :name (format nil "Inspector Clouseau: ~S"
+                       obj))
+	(run))
+    obj))
 
 (defparameter *inspected-objects* '()
   "A list of objects which are currently being inspected with
@@ -147,7 +174,8 @@ table, possibly using INSPECTOR-TABLE-ROW."
 	   (,evaluated-object ,object))
       (with-output-as-presentation
 	  (pane ,evaluated-object
-		(presentation-type-of ,evaluated-object))
+		(presentation-type-of ,evaluated-object)
+                :single-box t)
 	(formatting-table (,evaluated-pane)
 	  (formatting-column (,evaluated-pane)
 	    (formatting-cell (,evaluated-pane)
@@ -746,7 +774,8 @@ an error if the given time is not a decodable universal time."
   (frame-exit *application-frame*))
 
 (define-inspector-command (com-inspect :name t) ()
-  (let ((obj (accept t :prompt "Select an object")))
+  (let ((obj (accept t :prompt "Select an object"))
+        (*application-frame* nil))      ; To get around security.
     (inspector obj :new-process t)))
 
 (define-inspector-command (com-toggle-show-list-cells :name t)
