@@ -532,7 +532,7 @@
 (defun direct-slot-definitions (class slot-name)
   (let ((cpl (reverse (clim-mop:class-precedence-list class)))
         (direct-slots nil))
-    (dolist (foo cpl)
+    (dolist (foo cpl)                   ; rewrite this
       (let ((dslots (clim-mop:class-direct-slots foo)))
         (dolist (slot dslots)
           (when (eq slot-name (clim-mop:slot-definition-name slot))
@@ -554,10 +554,10 @@
          (initargs (clim-mop:slot-definition-initargs slot))
          (initfunc (clim-mop:slot-definition-initfunction slot))
          (initform (clim-mop:slot-definition-initform slot))
-         (direct-slots (direct-slot-definitions class name))
-         (readers (reduce #'append (filtermap direct-slots #'clim-mop:slot-definition-readers)))
-         (writers (reduce #'append (filtermap direct-slots #'clim-mop:slot-definition-writers)))
-         (documentation (first (filtermap direct-slots (lambda (x) (documentation x t)))))
+         (direct-slots (direct-slot-definitions class name))         
+         (readers (mapcan (lambda (x) (copy-list (clim-mop:slot-definition-readers x))) direct-slots))
+         (writers (mapcan (lambda (x) (copy-list (clim-mop:slot-definition-writers x))) direct-slots))
+         (documentation (first (remove nil (mapcar (lambda (x) (documentation x t)) direct-slots))))
          (*standard-output* stream))
 
   (macrolet ((with-ink ((var) &body body)
@@ -672,25 +672,32 @@
                          class))))
 
 (define-command (com-show-class-slots :name "Show Class Slots"
-				      :command-table show-commands
+                                      :command-table show-commands
                                       :menu "Class Slots"
-				      :provide-output-destination-keyword t)
-    ((class-name 'clim:symbol :prompt "class name"))
-  (let ((class (find-class class-name nil)))
-    (if (null class)
-	(format t "~&~A is not a defined class.~%" class-name)
-      (let ((slots (clim-mop:class-slots class)))
-	(if (null slots)
-	    (note "~%This class has no slots.~%~%")
-            (progn
-            ; oddly, looks much better in courier, because of all the capital letters.
-;            (with-text-family (t :sans-serif)
-              (invoke-as-heading
-               (lambda ()
-                 (format t "~&Slots for ")
-                 (with-output-as-presentation (t (clim-mop:class-name class) 'class-name :single-box t)
-                   (princ (clim-mop:class-name class)))))
-              (present-the-slots class) ))))))
+                                      :provide-output-destination-keyword t)
+    ((class-name 'clim:symbol :prompt "class name"))  
+  (let* ((class (find-class class-name nil))
+         (finalized-p (and class
+                           (typep class 'standard-class)
+                           (progn
+                             (clim-mop:finalize-inheritance class)
+                             (clim-mop:class-finalized-p class))))
+         (slots (and finalized-p (clim-mop:class-slots class))))
+    (cond
+     ((null class)
+      (note "~A is not a defined class.~%" class-name))
+     ((not (typep class 'standard-class))
+      (note "Class ~A is not a STANDARD-CLASS.~%" class-name))
+     ((not finalized-p)
+      (note "Class ~A is not finalized." class-name))
+     ((null slots)
+      (note "~%This class has no slots.~%~%"))
+     (t (invoke-as-heading
+         (lambda ()
+           (format t "~&Slots for ")
+           (with-output-as-presentation (t (clim-mop:class-name class) 'class-name :single-box t)
+             (princ (clim-mop:class-name class)))))
+        (present-the-slots class)))))
 
 (defparameter *ignorable-internal-class-names*
   '(standard-object))
@@ -712,11 +719,10 @@
   (error "Sorry, not supported in your CL implementation. See the function X-SPECIALIZER-DIRECT-GENERIC-FUNCTION if you are interested in fixing this."))
 
 (defun class-funcs (class)
-  (let ((classes (remove-ignorable-classes (copy-list (clim-mop:class-precedence-list class))))
-        (gfs nil))
-    (dolist (x classes)
-      (setf gfs (append gfs (x-specializer-direct-generic-functions x))))
-    (remove-duplicates gfs)))
+  (remove-duplicates
+   (mapcan (lambda (class) 
+             (copy-list (x-specializer-direct-generic-functions class)))
+           (remove-ignorable-classes (clim-mop:class-precedence-list class)))))
 
 (defun slot-name-sortp (a b)
   (flet ((slot-name-symbol (x)
@@ -1117,14 +1123,14 @@
      (show-hidden  'boolean :default nil :prompt "show hidden")
      (hide-garbage 'boolean :default t   :prompt "hide garbage")
      (show-all     'boolean :default nil :prompt "show all")
-     (style '(member items list) :default 'items :prompt "listing style")
+     (style '(member :items :list) :default :items :prompt "listing style")
      (group-directories 'boolean :default t :prompt "group directories?")
      (full-names 'boolean :default nil :prompt "show full name?")
      (list-all-direct-subdirectories 'boolean :default nil :prompt "list all direct subdirectories?"))
 
-  (let* ((pathname (if (wild-pathname-p pathname) ; Forgot why I did this..
-                       (merge-pathnames pathname)
-                     pathname))
+  (let* ((pathname
+          ;; helpfully fix things if trailing slash wasn't entered
+          (directorify-pathname pathname))
          (wild-pathname (gen-wild-pathname pathname))
          (dir (if list-all-direct-subdirectories
                   (list-directory-with-all-direct-subdirectories wild-pathname)
@@ -1133,13 +1139,13 @@
     (with-text-family (t :sans-serif)      
       (invoke-as-heading
         (lambda ()
-          (format t "Directory contents of ")
+          (format t "Contents of ")
           (present (directory-namestring pathname) 'pathname)
           (when (pathname-type pathname)
             (format t " (only files of type ~a)" (pathname-type pathname)))))
     
       (when (parent-directory pathname)
-        (with-output-as-presentation (t (strip-filespec (parent-directory pathname)) 'clim:pathname :single-box t)
+        (with-output-as-presentation (t (parent-directory pathname) 'clim:pathname :single-box t)
           (draw-icon t (standard-icon "up-folder.xpm") :extra-spacing 3)
           (format t "Parent Directory~%")))
 
@@ -1147,19 +1153,14 @@
         (unless show-all
           (setf group (filter-garbage-pathnames group show-hidden hide-garbage)))
         (ecase style
-          (items (abbreviating-format-items group :row-wise nil :x-spacing "  " :y-spacing 1
-                                            :printer (lambda (x stream)
-                                                       (declare (ignore stream))
-                                                       (pretty-pretty-pathname x *standard-output*
-                                                                               :long-name full-names)))
-                 #+NIL
-                 (format-items group :row-wise nil :x-spacing "  " :y-spacing 1
-                               :printer (lambda (x stream)
-                                          (declare (ignore stream))
-                                          (pretty-pretty-pathname x *standard-output* :long-name full-names)))
-                 (goatee::reposition-stream-cursor *standard-output*)                 
-                 (vertical-gap t))
-          (list (dolist (ent group)
+          (:items
+           (abbreviating-format-items group :row-wise nil :x-spacing "  " :y-spacing 1
+                                      :printer (lambda (x stream)
+                                                 (pretty-pretty-pathname x stream
+                                                                         :long-name full-names)))           
+           (multiple-value-bind (x y) (stream-cursor-position *standard-output*)
+             (setf (stream-cursor-position *standard-output*) (values 0 y))))
+          (:list (dolist (ent group)
                   (let ((ent (merge-pathnames ent pathname))) ;; This is for CMUCL, see above. (fixme!)
                                                               ;; And breaks some things for SBCL.. (mgr) 
                     (pretty-pretty-pathname ent *standard-output* :long-name full-names)))))))))
@@ -1180,12 +1181,12 @@
                                       :menu t
                                       :command-table filesystem-commands)
   ((pathname 'pathname :prompt "pathname"))
-  (let ((pathname (merge-pathnames pathname)))
-    (cond ((not (probe-file pathname))
-           (note "~A does not exist." pathname))
-          ((not (directoryp pathname))
-           (note "~A is not a directory." pathname))
-          (t (change-directory (merge-pathnames pathname))) )))
+  (let ((pathname (merge-pathnames
+                   ;; helpfully fix things if trailing slash wasn't entered
+                   (directorify-pathname pathname))))
+    (if (not (probe-file pathname))
+        (note "~A does not exist.~%" pathname)
+        (change-directory pathname))))
 
 (define-command (com-up-directory :name "Up Directory"
                                   :menu t
@@ -1278,11 +1279,7 @@
              (mv-or
               (when mime-type (mime-type-to-command mime-type pathname))
               (when command
-                (values command doc pointer-doc))
-              (when (and mime-type (subtypep mime-type 'text))
-                (values `(com-edit-file ,pathname)
-                        "Edit File"
-                        (format nil "Edit ~A" pathname))) ))))))
+                (values command doc pointer-doc))))))))
 
 (define-presentation-translator automagic-pathname-translator
   (clim:pathname clim:command filesystem-commands
@@ -1315,15 +1312,12 @@
 (define-command (com-push-directory :name "Push Directory"
                                     :menu t
                                     :command-table directory-stack-commands)
-  ((pathname 'pathname :prompt "pathname"))
-  (let ((pathname (merge-pathnames pathname)))
-    (if (and (probe-file pathname)
-             (directoryp pathname));; FIXME: Need smart conversion to directories, here and elsewhere.
+  ((pathname 'pathname :prompt "directory"))
+  (let ((pathname (merge-pathnames (directorify-pathname pathname))))
+    (if (not (probe-file pathname))
+        (note "~A does not exist.~%" pathname)
         (progn (push *default-pathname-defaults* *directory-stack*)
-               (com-change-directory pathname))
-        (italic (t)
-           (fresh-line) (present (truename pathname))
-           (format t " does not exist or is not a directory.~%")) ))
+               (com-change-directory pathname))))
   (compute-dirstack-command-eligibility *application-frame*))
 
 (defun comment-on-dir-stack ()
@@ -1392,9 +1386,6 @@
   ((pathname 'pathname  :prompt "pathname"))
   (clim-sys:make-process (lambda () (ed pathname))))
 
-;; Leave this translator disabled for now, the automagic translator will now produce
-;; com-edit-file where there is not a more specific handler for a text mime type.
-#+IGNORE
 (define-presentation-to-command-translator edit-file
   (clim:pathname com-edit-file filesystem-commands :gesture :select
 		 :pointer-documentation ((object stream)
@@ -1403,7 +1394,9 @@
 		 :tester ((object)
 			  (and (not (wild-pathname-p object))
                                (probe-file object)
-                               (pathname-name object))))
+                               (pathname-name object)
+                               (let ((mime-type (pathname-mime-type object)))
+                                 (and mime-type (subtypep mime-type 'text))))))
   (object)
   (list object))
 
@@ -1413,6 +1406,38 @@
   ((object 'pathname :prompt "pathname"))
   (show-file object))
 
+(define-presentation-to-command-translator show-file
+  (clim:pathname com-show-file filesystem-commands :gesture :select
+		 :pointer-documentation ((object stream)
+					 (format stream "Show ~A" object))
+		 :documentation ((stream) (format stream "Show File"))
+		 :tester ((object)
+			  (and (not (wild-pathname-p object))
+                               (probe-file object)
+                               (pathname-name object)
+                               (let ((mime-type (pathname-mime-type object)))
+                                 (and mime-type (subtypep mime-type 'text))))))
+  (object)
+  (list object))
+
+(define-command (com-display-image :name t :command-table filesystem-commands
+                                           :menu t)
+    ((image-pathname 'pathname
+      :default (user-homedir-pathname) :insert-default t))
+  (if (probe-file image-pathname)
+      (let* ((type (funcall (case (readtable-case *readtable*)
+                              (:upcase #'string-upcase)
+                              (:downcase #'string-downcase)
+                              (t #'identity))
+                            (pathname-type image-pathname)))
+             (format (find-symbol type (find-package :keyword))))
+        (handler-case (let ((pattern (make-pattern-from-bitmap-file image-pathname :format format)))
+                        (with-room-for-graphics ()
+                          (draw-pattern* *standard-output* pattern 0 0)))
+          (unsupported-bitmap-format ()
+            (format t "Image format ~A not recognized" type))))
+      (format t "No such file: ~A" image-pathname)))
+
 (define-command (com-edit-definition :name "Edit Definition"
 				     :command-table lisp-commands
                                      :menu t
@@ -1429,10 +1454,8 @@
   (list object))
 		   
 
-;; CLIM:OPEN-WINDOW-STREAM seems to be broken.
-;; Less broken since I hacked on it, but still bad..
 (defun show-file (pathname)
-  (let ( #+ignore(*standard-output* (open-window-stream :scroll-bars :both)) )
+  (let ((*standard-output* (open-window-stream :scroll-bars :both)) )
     (with-open-file (in pathname)
       (loop for line = (read-line in nil)
 	while line
