@@ -37,6 +37,10 @@
 (defclass clx-medium (basic-medium)
   ((gc :initform nil)
    (picture :initform nil)
+   (last-medium-device-region :initform nil)
+   (clipping-region-tmp :initform (vector 0 0 0 0)
+     :documentation "This object is reused to avoid consing in the
+ most common case when configuring the clipping region.")
    (buffer :initform nil :accessor medium-buffer)))
 
 #+CLX-EXT-RENDER
@@ -100,26 +104,36 @@
 		  ((t nil) 3)
 		  (otherwise (line-style-dashes line-style)))))))))
 
-(defmethod (setf medium-clipping-region) :after (region (medium clx-medium))
-  (declare (ignore region))
-  (with-slots (gc) medium
-    (when gc
-      (let ((clipping-region (medium-device-region medium)))
-        (if (region-equal clipping-region +nowhere+)
-	    (setf (xlib:gcontext-clip-mask gc) #())
-	    (let ((rect-seq (clipping-region->rect-seq clipping-region)))
-	      (when rect-seq
-		#+nil
-		;; ok, what McCLIM is generating is not :yx-banded...
-		;; (currently at least)
-		(setf (xlib:gcontext-clip-mask gc :yx-banded) rect-seq)
-		#-nil
-		;; the region code doesn't support yx-banding...
-		;; or does it? what does y-banding mean in this implementation?
-		;; well, apparantly it doesn't mean what y-sorted means
-		;; to clx :] we stick with :unsorted until that can be sorted out
-		(setf (xlib:gcontext-clip-mask gc :unsorted) rect-seq))))))))
-  
+(defun %set-gc-clipping-region (medium gc)
+  (declare (type clx-medium medium))
+  (let ((clipping-region (medium-device-region medium))
+        (tmp (slot-value medium 'clipping-region-tmp))
+        (port (port medium)))
+    (cond
+      ((region-equal clipping-region +nowhere+)
+       (setf (xlib:gcontext-clip-mask gc) #()))
+      ((typep clipping-region 'standard-rectangle)
+       (multiple-value-bind (x1 y1 width height)
+           (region->clipping-values clipping-region)
+         (setf (aref tmp 0) x1
+               (aref tmp 1) y1
+               (aref tmp 2) width
+               (aref tmp 3) height
+               (xlib:gcontext-clip-mask gc :unsorted) tmp)))
+      (t
+        (let ((rect-seq (clipping-region->rect-seq clipping-region)))
+          (when rect-seq
+            #+nil
+            ;; ok, what McCLIM is generating is not :yx-banded...
+            ;; (currently at least)
+            (setf (xlib:gcontext-clip-mask gc :yx-banded) rect-seq)
+            #-nil
+            ;; the region code doesn't support yx-banding...
+            ;; or does it? what does y-banding mean in this implementation?
+            ;; well, apparantly it doesn't mean what y-sorted means
+            ;; to clx :] we stick with :unsorted until that can be sorted out
+            (setf (xlib:gcontext-clip-mask gc :unsorted) rect-seq)))))))
+
 
 (defgeneric medium-gcontext (medium ink))
 
@@ -133,10 +147,11 @@
              (setf (xlib:gcontext-fill-style gc) :solid))))))
 
 (defmethod medium-gcontext ((medium clx-medium) (ink color))
+  (declare (optimize (debug 3)))
   (let* ((port (port medium))
 	 (mirror (port-lookup-mirror port (medium-sheet medium)))
 	 (line-style (medium-line-style medium)))
-    (with-slots (gc) medium
+    (with-slots (gc last-medium-device-region) medium
       (unless gc
 	(setq gc (xlib:create-gcontext :drawable mirror))
 	;; this is kind of false, since the :unit should be taken
@@ -151,26 +166,14 @@
 		  (xlib:gcontext-dashes gc) (if (eq dashes t) 3
 						dashes)))))
       (setf (xlib:gcontext-function gc) boole-1)
-      (setf (xlib:gcontext-font gc) (text-style-to-X-font port (medium-text-style medium)))
       (setf (xlib:gcontext-foreground gc) (X-pixel port ink)
 	    (xlib:gcontext-background gc) (X-pixel port (medium-background medium)))
-      ;; Here is a bug with regard to clipping ... ;-( --GB )
-      #-nil ; being fixed at the moment, a bit twitchy though -- BTS
-      (let ((clipping-region (medium-device-region medium)))
-        (if (region-equal clipping-region +nowhere+)
-	    (setf (xlib:gcontext-clip-mask gc) #())
-	    (let ((rect-seq (clipping-region->rect-seq clipping-region)))
-	      (when rect-seq
-		#+nil
-		;; ok, what McCLIM is generating is not :yx-banded...
-		;; (currently at least)
-		(setf (xlib:gcontext-clip-mask gc :yx-banded) rect-seq)
-		#-nil
-		;; the region code doesn't support yx-banding...
-		;; or does it? what does y-banding mean in this implementation?
-		;; well, apparantly it doesn't mean what y-sorted means
-		;; to clx :] we stick with :unsorted until that can be sorted out
-		(setf (xlib:gcontext-clip-mask gc :unsorted) rect-seq)))))
+      (let ((fn (text-style-to-X-font port (medium-text-style medium))))
+        (when (typep fn 'xlib:font)
+          (setf (xlib:gcontext-font gc) fn)))
+      (unless (eq last-medium-device-region (medium-device-region medium))
+        (setf last-medium-device-region (medium-device-region medium))
+        (%set-gc-clipping-region medium gc))
       gc)))
 
 (defmethod medium-gcontext ((medium clx-medium) (ink (eql +transparent-ink+)))
@@ -316,7 +319,7 @@
                               (:msbfirst #'identity))
                             (loop for i from 0 below num-bytes collect i)))))
 
-(defun generate-indexed-converter-expr (rgb-masks num-bytes byte-order)
+(defun generate-indexed-converter-expr (rgb-masks byte-order num-bytes)
   `(lambda (image-array converted-data mask-data width height inks)  
     (declare (optimize (speed 3)
                        (safety 0)
@@ -404,13 +407,27 @@
 
 (defparameter *pixel-converter-cache* (make-hash-table :test 'equal))
 
-(defun get-indexed-converter (visual-info byte-order bytes-per-pixel)
-  (let* ((rgb-masks (list (xlib:visual-info-red-mask visual-info)
-                          (xlib:visual-info-green-mask visual-info)
-                          (xlib:visual-info-blue-mask visual-info)))
-         (key (list rgb-masks byte-order bytes-per-pixel)))
+(defun ensure-indexed-converter (rgb-masks byte-order bytes-per-pixel)
+  (let ((key (list rgb-masks byte-order bytes-per-pixel)))
     (symbol-macrolet ((fn (gethash key *pixel-converter-cache*)))
-        (or fn (setf fn (compile nil (generate-indexed-converter-expr rgb-masks bytes-per-pixel byte-order)))))))
+        (or fn (setf fn (compile nil (generate-indexed-converter-expr rgb-masks byte-order bytes-per-pixel)))))))
+
+(defun visual-get-indexed-converter (visual-info byte-order bytes-per-pixel)
+  (let ((rgb-masks (list (xlib:visual-info-red-mask visual-info)
+                         (xlib:visual-info-green-mask visual-info)
+                         (xlib:visual-info-blue-mask visual-info))))
+    (ensure-indexed-converter rgb-masks byte-order bytes-per-pixel)))
+
+(defparameter *typical-pixel-formats* 
+  '(((#xFF0000 #xFF00 #xFF) :LSBFIRST 4)
+    ((#xFF0000 #xFF00 #xFF) :MSBFIRST 4))
+  "This is a table of the most likely pixel formats. Converters for
+these should be compiled in advance. Compiling the indexed->rgba 
+converter in advance will eliminate the pause observable the first
+time an indexed pattern is drawn.")
+
+(dolist (format *typical-pixel-formats*)  
+  (apply 'ensure-indexed-converter format))
 
 (defun fill-pixmap-indexed (visual-info depth byte-order array pm pm-gc mask mask-gc w h inks)
   (assert (= (array-total-size array) (* w h)))
@@ -427,7 +444,8 @@
          
     (if (and bytes-per-pixel
              (member byte-order '(:lsbfirst :msbfirst))
-             (setf pixel-converter (get-indexed-converter visual-info byte-order bytes-per-pixel)))
+             (setf pixel-converter (visual-get-indexed-converter 
+                                    visual-info byte-order bytes-per-pixel)))
         ;; Fast path - Image upload
         (let ((converted-data (make-array (* bytes-per-pixel (array-total-size array)) :element-type 'xlib:card8)))
           ;; Fill the pixel arrays
@@ -569,22 +587,28 @@
                        (round (rectangle-width rectangle))
                        (round (rectangle-height rectangle)))))
 
+(defun region->clipping-values (region)
+  (with-bounding-rectangle* (min-x min-y max-x max-y) region
+    (let ((clip-x (round-coordinate min-x))
+          (clip-y (round-coordinate min-y)))
+      (values clip-x
+              clip-y
+              (- (round-coordinate max-x) clip-x)
+              (- (round-coordinate max-y) clip-y)))))
+
 ; this seems to work, but find out why all of these +nowhere+s are coming from
 ; and kill them at the source...
 #-nil
 (defun clipping-region->rect-seq (clipping-region)
-  (loop
-     for region in (nreverse (mapcan
-			      (lambda (v) (unless (eq v +nowhere+) (list v)))
-			      (region-set-regions clipping-region
-						  :normalize :y-banding)))
-     as rectangle = (bounding-rectangle region)
-     for clip-x = (round-coordinate (rectangle-min-x rectangle))
-     for clip-y = (round-coordinate (rectangle-min-y rectangle))
-     nconcing (list clip-x
-		    clip-y
-		    (- (round-coordinate (rectangle-max-x rectangle)) clip-x)
-		    (- (round-coordinate (rectangle-max-y rectangle)) clip-y))))
+  (typecase clipping-region 
+    (area (multiple-value-list (region->clipping-values clipping-region)))
+    (t (loop 
+          for region in (nreverse (mapcan
+                                   (lambda (v) (unless (eq v +nowhere+) (list v)))
+                                   (region-set-regions clipping-region
+                                                       :normalize :y-banding)))
+          nconcing (multiple-value-list (region->clipping-values region))))))
+    
 
 (defmacro with-clx-graphics ((medium) &body body)
   `(let* ((port (port ,medium))
@@ -594,7 +618,7 @@
 	     (ink        (medium-ink ,medium))
 	     (gc         (medium-gcontext ,medium ink)))
 	line-style ink
-	(unwind-protect             
+	(unwind-protect
 	     (unless (eql ink +transparent-ink+)
                (progn ,@body))
 	  #+ignore(xlib:free-gcontext gc))))))
@@ -1187,12 +1211,12 @@
 	      (xlib:with-gcontext (gcontext 
 				   :clip-mask mask
 				   :clip-x x
-				   :clip-y (- y height))
+				   :clip-y y)
 		(xlib:copy-area pixmap gcontext 0 0 width height
-				da x (- y height))))
+				da x y)))
 	    (t
 	      (xlib:copy-area pixmap gcontext 0 0 width height
-			      da x (- y height)))))))))
+			      da x y))))))))
 
 (defmethod climi::medium-free-image-design
     ((medium clx-medium) (design climi::rgb-image-design))
