@@ -111,6 +111,8 @@
     :initarg  :ascent
     :reader   fontset-width)))
 
+(defvar *fontset* nil)
+
 (defmethod print-object ((object fontset) stream)
   (format stream "#<fontset ~A>" (fontset-name object)))
 
@@ -167,35 +169,52 @@
    (selection-timestamp :initform nil :accessor selection-timestamp)
    (font-families :accessor font-families)))
 
+(defun automagic-clx-server-path ()  
+  (let ((name (get-environment-variable "DISPLAY")))
+    (assert name (name)
+            "Environment variable DISPLAY is not set")
+    (let* (; this code courtesy telent-clx.
+           (slash-i (or (position #\/ name) -1))
+           (colon-i (position #\: name :start (1+ slash-i)))
+           (decnet-colon-p (eql (elt name (1+ colon-i)) #\:))
+           (host (subseq name (1+ slash-i) colon-i))
+           (dot-i (and colon-i (position #\. name :start colon-i)))
+           (display (and colon-i
+                      (parse-integer name
+                                     :start (if decnet-colon-p
+                                                (+ colon-i 2)
+                                                (1+ colon-i))
+                                     :end dot-i)))
+           (screen (and dot-i
+                     (parse-integer name :start (1+ dot-i))))
+           (protocol
+            (cond ((or (string= host "") (string-equal host "unix")) :local)
+                  (decnet-colon-p :decnet)
+                  ((> slash-i -1) (intern
+                                   (string-upcase (subseq name 0 slash-i))
+                                   :keyword))
+                  (t :internet))))
+      (list :clx
+	    :host host
+	    :display-id (or display 0)
+	    :screen-id (or screen 0)
+	    :protocol protocol))))
+
+(defun helpfully-automagic-clx-server-path ()
+  (restart-case (automagic-clx-server-path)
+    (use-localhost ()
+      :report "Use local unix display"
+      (parse-clx-server-path '(:clx :host "" :protocol :unix)))))
+
 (defun parse-clx-server-path (path)
   (pop path)
-  (let* ((name (get-environment-variable "DISPLAY"))
-	 ;; this code courtesy telent-clx.
-         (slash-i (or (position #\/ name) -1))
-         (colon-i (position #\: name :start (1+ slash-i)))
-         (decnet-colon-p (eql (elt name (1+ colon-i)) #\:))
-         (host (subseq name (1+ slash-i) colon-i))
-         (dot-i (and colon-i (position #\. name :start colon-i)))
-         (display (when colon-i
-                    (parse-integer name
-                                   :start (if decnet-colon-p
-                                              (+ colon-i 2)
-					      (1+ colon-i))
-                                   :end dot-i)))
-         (screen (when dot-i
-                   (parse-integer name :start (1+ dot-i))))
-         (protocol
-          (cond ((or (string= host "") (string-equal host "unix")) :local)
-                (decnet-colon-p :decnet)
-                ((> slash-i -1) (intern
-                                 (string-upcase (subseq name 0 slash-i))
-                                 :keyword))
-                (t :internet))))
-    (list :clx
-	  :host (getf path :host host)
-	  :display-id (getf path :display-id (or display 0))
-	  :screen-id (getf path :screen-id (or screen 0))
-	  :protocol protocol)))
+  (if path
+      (list :clx
+	    :host       (getf path :host "localhost")
+	    :display-id (getf path :display-id 0)
+	    :screen-id  (getf path :screen-id 0)
+	    :protocol   (getf path :protocol :internet))
+      (helpfully-automagic-clx-server-path)))
 
 (setf (get :x11 :port-type) 'clx-port)
 (setf (get :x11 :server-path-parser) 'parse-clx-server-path)
@@ -277,7 +296,7 @@
         #'clx-error-handler)
 
       #+nil ;; Uncomment this when debugging CLX backend if asynchronous errors become troublesome..
-      (setf (xlib:display-after-function (clx-port-display port)) #'xlib:display-force-output))
+      (setf (xlib:display-after-function (clx-port-display port)) #'xlib:display-finish-output))
       
     
     (setf (clx-port-screen port) (nth (getf options :screen-id)
@@ -664,6 +683,8 @@
 ;; pointer-event-buttons slot to pointer events. -- moore
 ;; 
 
+(defvar *clx-port*)
+
 (defun event-handler (&key display window event-key code state mode time
 		      type width height x y root-x root-y
 		      data override-redirect-p send-event-p hint-p
@@ -671,7 +692,6 @@
                       request first-keycode count
                       &allow-other-keys)
   (declare (ignore display request first-keycode count))
-  (declare (special *clx-port*))
   (let ((sheet (and window (port-lookup-sheet *clx-port* window))))
     (when sheet
       (case event-key
@@ -861,7 +881,6 @@
   (declare (ignore wait-function))
   (let* ((*clx-port* port)
          (display    (clx-port-display port)))
-    (declare (special *clx-port*))
     (unless (xlib:event-listen display)
       (xlib:display-finish-output (clx-port-display port)))
     ; temporary solution
@@ -942,8 +961,6 @@
 	(xlib:open-font display (first fonts))
         (xlib:open-font display "fixed"))))
 
-(defvar *fontset* nil)
-
 (defmethod text-style-mapping ((port clx-port) text-style
                                &optional character-set)
   (declare (ignore character-set))
