@@ -35,19 +35,13 @@ buffer-protocol related tests." :in drei-tests)
 (buffer-test buffer-make-instance
   "Test that the various types of buffers can be created, and
 that they are initialized properly."
-  (let* ((buffer (make-instance %%buffer))
-	 (low (low-mark buffer))
-	 (high (low-mark buffer)))
-    (is (= (offset low) 0))
-    (is (= (offset high) 0))
-    (is (null (modified-p buffer)))
-    (is (eq (buffer low) (buffer high)))
+  (let* ((buffer (make-instance %%buffer)))
     (signals motion-after-end
       (make-instance %%left-sticky-mark :buffer buffer :offset 1))
     (handler-case
         (progn (make-instance %%right-sticky-mark
-                              :buffer buffer
-                              :offset 1)
+                :buffer buffer
+                :offset 1)
                (fail))
       (motion-after-end (c)
         (is (= (drei-buffer::condition-offset c) 1))))))
@@ -55,8 +49,8 @@ that they are initialized properly."
 (buffer-test clone-mark
   "Test that marks really are cloned."
   (let* ((buffer (make-instance %%buffer))
-         (low (low-mark buffer))
-         (high (high-mark buffer))
+         (low (make-buffer-mark buffer))
+         (high (make-buffer-mark buffer))
          (low2 (clone-mark low))
          (high2 (clone-mark high))
          (low3 (clone-mark high :left))
@@ -78,22 +72,13 @@ that they are initialized properly."
 order, and that the buffer is updated appropriately."
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-object buffer 0 #\a)
-    ;; The lower bound of the change.
-    (is (= (offset (low-mark buffer)) 0))
-    ;; The upper bound of the change.
-    (is (= (offset (high-mark buffer)) 1))
-    ;; It should have recorded the modification.
-    (is-true (modified-p buffer))
-    ;; The buffer should have increased in size the accomodate the
+    ;; The buffer should have increased in size to accomodate the
     ;; object.
     (is (= (size buffer) 1))
     ;; The contents should be what we put in there.
     (is (string= (buffer-substring buffer 0 1) "a"))
     (insert-buffer-object buffer 0 #\b)
     (insert-buffer-object buffer 0 #\a)
-    (is (= (offset (low-mark buffer)) 0))
-    (is (= (offset (high-mark buffer)) 3))
-    (is-true (modified-p buffer))
     (is (= (size buffer) 3))
     (is (equal (buffer-substring buffer 0 3) "aba"))
     ;; We should be able to insert an object in the middle of the
@@ -106,7 +91,7 @@ order, and that the buffer is updated appropriately."
         (progn (insert-buffer-object buffer 5 #\a)
                (fail "Failed to signal during insertion of object
 after buffer end"))
-      (motion-after-end (c)
+      (offset-after-end (c)
         (is (= (condition-offset c) 5))))
     ;; Attempting to insert an object before buffer start should cause
     ;; an error.
@@ -114,7 +99,7 @@ after buffer end"))
         (progn (insert-buffer-object buffer -1 #\a)
                (fail "Failed to signal during insertion of object
 before buffer start"))
-      (motion-before-beginning (c)
+      (offset-before-beginning (c)
         (= (condition-offset c) -1)))))
 
 (buffer-test insert-buffer-sequence
@@ -137,13 +122,13 @@ size and contents are updated appropriately."
         (progn (insert-buffer-sequence buffer 37 "climacs")
                (fail "Failed to signal during insertion of
 sequence after buffer end"))
-      (motion-after-end (c)
+      (offset-after-end (c)
         (= (condition-offset c) 37)))
     (handler-case
         (progn (insert-buffer-sequence buffer -1 "climacs")
                (fail "Failed to signal during insertion of
 sequence before buffer start"))
-      (motion-before-beginning (c)
+      (offset-before-beginning (c)
         (is (= (condition-offset c) -1))))))
 
 (buffer-test delete-buffer-range
@@ -152,65 +137,55 @@ works."
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
     (delete-buffer-range buffer 0 7)
-    (is (= (offset (low-mark buffer)) 0))
-    (is (= (offset (high-mark buffer)) 0))
-    (is-true (modified-p buffer))
-    (is (= (size buffer) 0))
+    (is (= 0 (size buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
     (delete-buffer-range buffer 0 3)
-    (is (= (offset (low-mark buffer)) 0))
-    (is (= (offset (high-mark buffer)) 4))
-    (is-true (modified-p buffer))
-    (is (= (size buffer) 4))
-    (is (string= (buffer-substring buffer 0 4) "macs"))
+    (is (= 4 (size buffer)))
+    (is (string= "macs" (buffer-substring buffer 0 4)))
     (insert-buffer-sequence buffer 0 "climacs")
     (delete-buffer-range buffer 3 4)
-    (is (= (size buffer) 7))
-    (is (string= (buffer-substring buffer 0 3) "cli"))
+    (is (= 7 (size buffer)))
+    (is (string= "cli" (buffer-substring buffer 0 3)))
     ;; Deleting 0 characters should work (and leave the buffer
     ;; untouched.)
     (insert-buffer-sequence buffer 0 "climacs")
     (delete-buffer-range buffer 3 0)
-    (is (= (size buffer) 14))
-    (is (string= (buffer-substring buffer 0 7) "climacs"))
+    (is (= 14 (size buffer)))
+    (is (string= "climacs" (buffer-substring buffer 0 7)))
     (handler-case
         (progn
           (delete-buffer-range buffer -1 0)
           (fail "Failed to signal during deletion of range with
 negative offset"))
-      (motion-before-beginning (c)
-        (= (condition-offset c) -1)))
+      (offset-before-beginning (c)
+        (is (= -1 (condition-offset c)))))
     (handler-case
         (progn
           (delete-buffer-range buffer 12 3)
           (fail "Failed to signal during deletion of range that
 exceeded buffer end."))
-      (motion-after-end (c)
-        (= (condition-offset c) 15)))))
+      (offset-after-end (c)
+        (is (= 15 (condition-offset c)))))))
 
 (buffer-test insert-object
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :left)))
-      (setf (offset m) 3)
+    (let ((m (make-buffer-mark buffer 3 :left)))
       (insert-object m #\X)
-      (is (= (size buffer) 8))
-      (is (= (offset m) 3))
-      (is (string= (buffer-substring buffer 0 8) "cliXmacs")))
-    (let ((m (clone-mark (low-mark buffer) :right)))
-      (setf (offset m) 2)
+      (is (= 8 (size buffer)))
+      (is (= 3 (offset m)))
+      (is (string= "cliXmacs" (buffer-substring buffer 0 8))))
+    (let ((m (make-buffer-mark buffer 2 :right)))
       (insert-object m #\X)
-      (is (= (size buffer) 9))
-      (is (= (offset m) 3))
-      (is (string= (buffer-substring buffer 0 9) "clXiXmacs")))))
+      (is (= 9 (size buffer)))
+      (is (= 3 (offset m)))
+      (is (string= "clXiXmacs" (buffer-substring buffer 0 9))))))
 
 (buffer-test insert-sequence
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :left))
-	  (m2 (clone-mark (low-mark buffer) :left)))
-      (setf (offset m) 3
-	    (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :left))
+	  (m2 (make-buffer-mark buffer 5 :left)))
       (insert-sequence m "ClimacS")
       (is (= (size buffer) 14))
       (is (eq (buffer m) (buffer m2)))
@@ -219,10 +194,8 @@ exceeded buffer end."))
       (is (string= (buffer-substring buffer 0 14) "cliClimacSmacs"))))
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :right))
-          (m2 (clone-mark (low-mark buffer) :right)))
-      (setf (offset m) 3
-            (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :right))
+          (m2 (make-buffer-mark buffer 5 :right)))
       (insert-sequence m "ClimacS")
       (is (= (size buffer) 14))
       (is (eq (buffer m) (buffer m2)))
@@ -233,10 +206,8 @@ exceeded buffer end."))
 (buffer-test delete-range
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :left))
-	  (m2 (clone-mark (low-mark buffer) :left)))
-      (setf (offset m) 3
-            (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :left))
+	  (m2 (make-buffer-mark buffer 5 :left)))
       (delete-range m 2)
       (is (= (size buffer) 5))
       (is (eq (buffer m) (buffer m2)))
@@ -245,10 +216,8 @@ exceeded buffer end."))
       (is (string= (buffer-substring buffer 0 5) "clics"))))
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :right))
-	  (m2 (clone-mark (low-mark buffer) :right)))
-      (setf (offset m) 3
-	    (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :right))
+	  (m2 (make-buffer-mark buffer 5 :right)))
       (delete-range m -2)
       (is (= (size buffer) 5))
       (is (eq (buffer m) (buffer m2)))
@@ -259,75 +228,63 @@ exceeded buffer end."))
 (buffer-test delete-region
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :left))
-	  (m2 (clone-mark (low-mark buffer) :left)))
-      (setf (offset m) 3
-	    (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :left))
+	  (m2 (make-buffer-mark buffer 5 :left)))
       (delete-region m m2)
-      (is (= (size buffer) 5))
+      (is (= 5 (size buffer)))
       (is (eq (buffer m) (buffer m2)))
-      (is (= (offset m) 3))
-      (is (= (offset m2) 3))
+      (is (= 3 (offset m)))
+      (is (= 3 (offset m2)))
       (is (string= (buffer-substring buffer 0 5) "clics"))))
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :right))
-	  (m2 (clone-mark (low-mark buffer) :right)))
-      (setf (offset m) 3
-	    (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :right))
+	  (m2 (make-buffer-mark buffer 5 :right)))
       (delete-region m m2)
-      (is (= (size buffer) 5))
+      (is (= 5 (size buffer)))
       (is (eq (buffer m) (buffer m2)))
-      (is (= (offset m) 3))
-      (is (= (offset m2) 3))
+      (is (= 3 (offset m)))
+      (is (= 3 (offset m2)))
       (is (string= (buffer-substring buffer 0 5) "clics"))))
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :left))
-          (m2 (clone-mark (low-mark buffer) :left)))
-      (setf (offset m) 3
-            (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :left))
+          (m2 (make-buffer-mark buffer 5 :left)))
       (delete-region m2 m)
-      (is (= (size buffer) 5))
+      (is (= 5 (size buffer)))
       (is (eq (buffer m) (buffer m2)))
-      (is (= (offset m) 3))
-      (is (= (offset m2) 3))
+      (is (= 3 (offset m)))
+      (is (= 3 (offset m2)))
       (is (string= (buffer-substring buffer 0 5) "clics"))))
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :right))
-	  (m2 (clone-mark (low-mark buffer) :right)))
-      (setf (offset m) 3
-	    (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :right))
+	  (m2 (make-buffer-mark buffer 5 :right)))
       (delete-region m2 m)
-      (is (= (size buffer) 5))
+      (is (= 5 (size buffer)))
       (is (eq (buffer m) (buffer m2)))
-      (is (= (offset m) 3))
-      (is (= (offset m2) 3))
+      (is (= 3 (offset m)))
+      (is (= 3 (offset m2)))
       (is (string= (buffer-substring buffer 0 5) "clics"))))
   (signals error
     (let ((buffer (make-instance %%buffer))
           (buffer2 (make-instance %%buffer)))
       (insert-buffer-sequence buffer 0 "climacs")
       (insert-buffer-sequence buffer2 0 "climacs")
-      (let ((m (clone-mark (low-mark buffer) :right))
-            (m2 (clone-mark (low-mark buffer2) :right)))
-        (setf (offset m) 3
-              (offset m2) 5)
+      (let ((m (make-buffer-mark buffer 3 :right))
+            (m2 (make-buffer-mark buffer2 5 :right)))
         (delete-region m2 m))))
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m (clone-mark (low-mark buffer) :left))
-	  (m2 (clone-mark (low-mark buffer) :left)))
-      (setf (offset m) 3
-	    (offset m2) 5)
+    (let ((m (make-buffer-mark buffer 3 :left))
+	  (m2 (make-buffer-mark buffer 5 :left)))
       (delete-region m 5)
       (delete-region 1 m2)
-      (is (= (size buffer) 3))
+      (is (= 3 (size buffer)))
       (is (eq (buffer m) (buffer m2)))
-      (is (= (offset m) 1))
-      (is (= (offset m2) 1))
-      (is (string= (buffer-substring buffer 0 3) "ccs")))))
+      (is (= 1 (offset m)))
+      (is (= 1 (offset m2)))
+      (is (string= "ccs" (buffer-substring buffer 0 3))))))
 
 (buffer-test number-of-lines
   (let ((buffer (make-instance %%buffer)))
@@ -340,18 +297,12 @@ climacs
 (buffer-test mark-relations
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let ((m0 (clone-mark (low-mark buffer) :right))
-          (m1 (clone-mark (low-mark buffer) :left))
-          (m1a (clone-mark (low-mark buffer) :right))
-          (m2 (clone-mark (low-mark buffer) :right))
-          (m2a (clone-mark (low-mark buffer) :left))
-          (m3 (clone-mark (low-mark buffer) :left)))
-      (setf (offset m0) 0
-            (offset m1) 3
-            (offset m1a) 3
-            (offset m2) 5
-            (offset m2a) 5
-            (offset m3) 7)
+    (let ((m0 (make-buffer-mark buffer 0 :right))
+          (m1 (make-buffer-mark buffer 3 :left))
+          (m1a (make-buffer-mark buffer 3 :right))
+          (m2 (make-buffer-mark buffer 5 :right))
+          (m2a (make-buffer-mark buffer 5 :left))
+          (m3 (make-buffer-mark buffer 7 :left)))
       (is-true (mark< m0 m1)) (not (mark> m0 m1)) (not (mark>= m0 m1))
       (is-true (mark< m0 m2)) (not (mark> m0 m2)) (not (mark>= m0 m2))
       (is-true (mark< m0 m3)) (not (mark> m0 m3)) (not (mark>= m0 m3))
@@ -380,38 +331,32 @@ climacs
   (handler-case
       (let ((buffer (make-instance %%buffer)))
 	(insert-buffer-sequence buffer 0 "climacs")
-	(let ((m (clone-mark (low-mark buffer) :left)))
-	  (setf (offset m) -1)
-          (fail "Failed to signal when setting offset of mark to
-negative value")))
+	(make-buffer-mark buffer -1 :left)
+        (fail "Failed to signal when setting offset of mark to
+negative value"))
     (motion-before-beginning (c)
       (is (= (condition-offset c) -1))))
   (handler-case
       (let ((buffer (make-instance %%buffer)))
 	(insert-buffer-sequence buffer 0 "climacs")
-	(let ((m (clone-mark (low-mark buffer) :left)))
-	  (setf (offset m) 8)
-          (fail "Failed to signal when setting offset of mark to
-a value too large for the buffer")))
+        (make-buffer-mark buffer 8 :left)
+        (fail "Failed to signal when setting offset of mark to
+a value too large for the buffer"))
     (motion-after-end (c)
       (is (= (condition-offset c) 8)))))
 
 (buffer-test backward-object
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let* ((m1 (clone-mark (low-mark buffer) :left))
+    (let* ((m1 (make-buffer-mark buffer 4 :left))
 	   (m2 (clone-mark m1)))
-      (setf (offset m1) 4
-	    (offset m2) 4)
       (backward-object m1 2)
       (is (string= (region-to-string m1 m2) "im"))))
   (handler-case
       (let ((buffer (make-instance %%buffer)))
 	(insert-buffer-sequence buffer 0 "climacs")
-	(let* ((m1 (clone-mark (low-mark buffer) :right))
+	(let* ((m1 (make-buffer-mark buffer 2 :right))
 	       (m2 (clone-mark m1)))
-	  (setf (offset m1) 2
-		(offset m2) 2)
 	  (backward-object m1 3)
 	  (region-to-sequence m1 m2)
           (fail "Failed to signal when moving mark to before
@@ -422,16 +367,14 @@ buffer start.")))
 (buffer-test forward-object
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (let* ((m1 (clone-mark (low-mark buffer) :left))
+    (let* ((m1 (make-buffer-mark buffer 4 :left))
 	   (m2 (clone-mark m1)))
-      (setf (offset m1) 4
-	    (offset m2) 4)
       (forward-object m1 2)
       (is (string= (region-to-string m1 m2) "ac"))))
   (handler-case
       (let ((buffer (make-instance %%buffer)))
 	(insert-buffer-sequence buffer 0 "climacs")
-	(let* ((m1 (clone-mark (low-mark buffer) :right))
+	(let* ((m1 (make-buffer-mark buffer 0 :right))
 	       (m2 (clone-mark m1)))
 	  (setf (offset m1) 6
 		(offset m2) 6)
@@ -459,7 +402,7 @@ offset 0"))
 	(setf (buffer-object buffer -1) #\a)
         (fail "Failed to signal when setting buffer object at
 negative offset"))
-    (motion-before-beginning (c)
+    (offset-before-beginning (c)
       (is (= (condition-offset c) -1)))))
 
 (buffer-test mark<
@@ -468,8 +411,8 @@ negative offset"))
           (buffer2 (make-instance %%buffer)))
       (insert-buffer-sequence buffer 0 "climacs")
       (insert-buffer-sequence buffer2 0 "climacs")
-      (let ((m1 (clone-mark (low-mark buffer)))
-            (m2 (clone-mark (low-mark buffer2))))
+      (let ((m1 (make-buffer-mark buffer))
+            (m2 (make-buffer-mark buffer2)))
         (mark< m1 m2)))))
 
 (buffer-test mark>
@@ -478,8 +421,8 @@ negative offset"))
           (buffer2 (make-instance %%buffer)))
       (insert-buffer-sequence buffer 0 "climacs")
       (insert-buffer-sequence buffer2 0 "climacs")
-      (let ((m1 (clone-mark (low-mark buffer)))
-            (m2 (clone-mark (low-mark buffer2))))
+      (let ((m1 (make-buffer-mark buffer))
+            (m2 (make-buffer-mark buffer2)))
         (mark> m1 m2)))))
 
 (buffer-test mark<=
@@ -488,8 +431,8 @@ negative offset"))
           (buffer2 (make-instance %%buffer)))
       (insert-buffer-sequence buffer 0 "climacs")
       (insert-buffer-sequence buffer2 0 "climacs")
-      (let ((m1 (clone-mark (low-mark buffer)))
-            (m2 (clone-mark (low-mark buffer2))))
+      (let ((m1 (make-buffer-mark buffer))
+            (m2 (make-buffer-mark buffer2)))
         (mark<= m1 m2)))))
 
 (buffer-test mark>=
@@ -498,8 +441,8 @@ negative offset"))
           (buffer2 (make-instance %%buffer)))
       (insert-buffer-sequence buffer 0 "climacs")
       (insert-buffer-sequence buffer2 0 "climacs")
-      (let ((m1 (clone-mark (low-mark buffer)))
-            (m2 (clone-mark (low-mark buffer2))))
+      (let ((m1 (make-buffer-mark buffer))
+            (m2 (make-buffer-mark buffer2)))
         (mark>= m1 m2)))))
 
 (buffer-test mark=
@@ -508,16 +451,16 @@ negative offset"))
           (buffer2 (make-instance %%buffer)))
       (insert-buffer-sequence buffer 0 "climacs")
       (insert-buffer-sequence buffer2 0 "climacs")
-      (let ((m1 (clone-mark (low-mark buffer)))
-            (m2 (clone-mark (low-mark buffer2))))
+      (let ((m1 (make-buffer-mark buffer))
+            (m2 (make-buffer-mark buffer2)))
         (mark= m1 m2)))))
 
 (buffer-test line-number
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 climacs")
-    (let ((m1 (clone-mark (low-mark buffer) :left))
-	  (m2 (clone-mark (low-mark buffer) :right)))
+    (let ((m1 (make-buffer-mark buffer 0 :left))
+	  (m2 (make-buffer-mark buffer 0 :right)))
       (setf (offset m1) 3
 	    (offset m2) 11)
       (is (= 0 (line-number m1) (1- (line-number m2)))))))
@@ -538,8 +481,8 @@ climacs")
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 climacs")
-    (let ((m1 (clone-mark (low-mark buffer) :left))
-	  (m2 (clone-mark (low-mark buffer) :right)))
+    (let ((m1 (make-buffer-mark buffer 0 :left))
+	  (m2 (make-buffer-mark buffer 0 :right)))
       (setf (offset m1) 3
 	    (offset m2) 11)
       (is (= 3 (column-number m1) (column-number m2)))))t)
@@ -548,7 +491,7 @@ climacs")
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 climacs")
-    (let ((m (clone-mark (low-mark buffer) :left)))
+    (let ((m (make-buffer-mark buffer 0 :left)))
       (setf (offset m) 11)
       (is-true (not (beginning-of-line-p m)))
       (beginning-of-line m)
@@ -558,7 +501,7 @@ climacs")
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 climacs")
-    (let ((m (clone-mark (low-mark buffer) :left)))
+    (let ((m (make-buffer-mark buffer 0 :left)))
       (setf (offset m) 11)
       (is-true (not (end-of-line-p m)))
       (end-of-line m)
@@ -566,7 +509,7 @@ climacs")
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 ")
-    (let ((m (clone-mark (low-mark buffer) :left)))
+    (let ((m (make-buffer-mark buffer 0 :left)))
       (setf (offset m) 1)
       (is-true (not (end-of-line-p m)))
       (end-of-line m)
@@ -577,7 +520,7 @@ climacs")
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 climacs")
-    (let ((m (clone-mark (low-mark buffer) :left)))
+    (let ((m (make-buffer-mark buffer 0 :left)))
       (setf (offset m) 11)
       (is-true (not (beginning-of-buffer-p m)))
       (beginning-of-buffer m)
@@ -587,7 +530,7 @@ climacs")
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs
 climacs")
-    (let ((m (clone-mark (low-mark buffer) :left)))
+    (let ((m (make-buffer-mark buffer 0 :left)))
       (setf (offset m) 11)
       (is-true (not (end-of-buffer-p m)))
       (end-of-buffer m)
@@ -636,11 +579,11 @@ too big for buffer"))
 (buffer-test object-before
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (is (char= (object-before (high-mark buffer)) #\s)))
+    (is (char= (object-before (make-buffer-mark buffer 7)) #\s)))
   (handler-case
       (let ((buffer (make-instance %%buffer)))
 	(insert-buffer-sequence buffer 0 "climacs")
-	(object-before (low-mark buffer))
+	(object-before (make-buffer-mark buffer))
         (fail "Failed to signal when requesting object before
 buffer start"))
     (no-such-offset (c)
@@ -649,11 +592,11 @@ buffer start"))
 (buffer-test object-after
   (let ((buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 "climacs")
-    (is (char= (object-after (low-mark buffer)) #\c)))
+    (is (char= (object-after (make-buffer-mark buffer)) #\c)))
   (handler-case
       (let ((buffer (make-instance %%buffer)))
 	(insert-buffer-sequence buffer 0 "climacs")
-	(object-after (high-mark buffer))
+	(object-after (make-buffer-mark buffer 7))
         (fail "Failed to signal when requesting object past
 buffer end"))
     (no-such-offset (c)
@@ -663,42 +606,42 @@ buffer end"))
   (let ((seq "climacs")
 	(buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 seq)
-    (let ((seq2 (region-to-string (low-mark buffer) (high-mark buffer))))
+    (let ((seq2 (region-to-string (make-buffer-mark buffer) (make-buffer-mark buffer 7))))
       (is-true (not (eq seq seq2)))
       (is (string= seq2 "climacs"))))
   (let ((seq "climacs")
 	(buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 seq)
-    (let ((seq2 (region-to-string 0 (high-mark buffer))))
+    (let ((seq2 (region-to-string 0 (make-buffer-mark buffer 7))))
       (is-true (not (eq seq seq2)))
       (is (string= seq2 "climacs"))))
   (let ((seq "climacs")
 	(buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 seq)
-    (let ((seq2 (region-to-string (high-mark buffer) 0)))
+    (let ((seq2 (region-to-string (make-buffer-mark buffer 7) 0)))
       (is-true (not (eq seq seq2)))
       (is (string= seq2 "climacs"))))
   (let ((seq "climacs")
 	(buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 seq)
-    (let ((seq2 (region-to-string (low-mark buffer) 7)))
+    (let ((seq2 (region-to-string (make-buffer-mark buffer) 7)))
       (is-true (not (eq seq seq2)))
       (is (string= seq2 "climacs"))))
   (let ((seq "climacs")
 	(buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 seq)
-    (let ((seq2 (region-to-string 7 (low-mark buffer))))
+    (let ((seq2 (region-to-string 7 (make-buffer-mark buffer))))
       (is-true (not (eq seq seq2)))
       (is (string= seq2 "climacs"))))
   (let ((seq "climacs")
 	(buffer (make-instance %%buffer)))
     (insert-buffer-sequence buffer 0 seq)
-    (is (string= (region-to-string (high-mark buffer) (low-mark buffer))
+    (is (string= (region-to-string (make-buffer-mark buffer 7) (make-buffer-mark buffer))
                  "climacs")))
   (signals error
     (let ((buffer1 (make-instance %%buffer))
           (buffer2 (make-instance %%buffer)))
-      (region-to-string (low-mark buffer1) (high-mark buffer2)))))
+      (region-to-string (make-buffer-mark buffer1) (make-buffer-mark buffer2)))))
 
 ;;;; performance tests
 
@@ -827,7 +770,7 @@ buffer-protocol implementation performance tests.")
   (is-false (time
              (let ((b (make-instance %%buffer)))
                (insert-buffer-sequence b 0 (make-array '(100000) :initial-element #\a))
-               (let ((m (clone-mark (low-mark b))))
+               (let ((m (clone-mark (make-buffer-mark b))))
                  (loop
                     for i from 0 below 1000
                     for f = t then (not b)
@@ -841,7 +784,7 @@ buffer-protocol implementation performance tests.")
                (insert-buffer-object b 0 #\Newline)
                (insert-buffer-sequence b 0 (make-array '(100000) :initial-element #\a))
                (insert-buffer-object b 0 #\Newline)
-               (let ((m (clone-mark (low-mark b))))
+               (let ((m (clone-mark (make-buffer-mark b))))
                  (loop
                     for i from 0 below 1000
                     for f = t then (not b)
@@ -855,7 +798,7 @@ buffer-protocol implementation performance tests.")
                (insert-buffer-object b 0 #\Newline)
                (insert-buffer-sequence b 0 (make-array '(100000) :initial-element #\a))
                (insert-buffer-object b 0 #\Newline)
-               (let ((m (clone-mark (low-mark b))))
+               (let ((m (clone-mark (make-buffer-mark b))))
                  (incf (offset m))
                  (loop
                     for i from 0 below 1000
@@ -870,7 +813,7 @@ buffer-protocol implementation performance tests.")
                 (insert-buffer-object b 0 #\Newline)
                 (insert-buffer-sequence b 0 (make-array '(100000) :initial-element #\a))
                 (insert-buffer-object b 0 #\Newline)
-                (let ((m (clone-mark (low-mark b))))
+                (let ((m (clone-mark (make-buffer-mark b))))
                   (setf (offset m) (floor (size b) 2))
                   (loop
                      for i from 0 below 10
@@ -883,7 +826,7 @@ buffer-protocol implementation performance tests.")
           (let ((b (make-instance %%buffer)))
             (insert-buffer-sequence
              b 0 (make-array '(100000) :initial-element #\Newline))
-            (let ((m (clone-mark (low-mark b))))
+            (let ((m (clone-mark (make-buffer-mark b))))
               (loop
                  for i from 0 below 1000
                  for f = t then (not b)
