@@ -85,64 +85,88 @@ true (the default)."
       `(PROGN
          (DEFINE-COMMAND (,com-forward :NAME T
                                        :COMMAND-TABLE ,command-table)
-             ((COUNT 'INTEGER :PROMPT ,(concat "Number of " plural)))
+             ((COUNT 'INTEGER :PROMPT ,(concat "Number of " plural) :default 1))
            ,(concat "Move point forward by one " noun ".
 With a numeric argument N, move point forward by N " plural ".
 With a negative argument -N, move point backward by N " plural ".")
            (handling-motion-limit-errors (,plural)
-             (,forward *current-point*
-                       (SYNTAX *current-buffer*)
+             (,forward (point)
+                       (current-syntax)
                        COUNT)))
-         (DEFINE-COMMAND (,com-backward
-                          :NAME T
+         (DEFINE-COMMAND (,com-backward :NAME T
                           :COMMAND-TABLE ,command-table)
-             ((COUNT 'INTEGER :PROMPT ,(concat "Number of " plural)))
+             ((COUNT 'INTEGER :PROMPT ,(concat "Number of " plural) :default 1))
            ,(concat "Move point backward by one " noun ".
 With a numeric argument N, move point backward by N " plural ".
 With a negative argument -N, move point forward by N " plural ".")
            (handling-motion-limit-errors (,plural)
-             (,backward *current-point*
-                        (SYNTAX *current-buffer*)
+             (,backward (point)
+                        (current-syntax)
                         COUNT)))))))
 
 ;;; Manually define some commands
 
 (define-command (com-beginning-of-line :name t :command-table movement-table) ()
   "Move point to the beginning of the current line."
-  (beginning-of-line *current-point*))
+  (beginning-of-line (point)))
 
 (define-command (com-end-of-line :name t :command-table movement-table) ()
   "Move point to the end of the current line."
-  (end-of-line *current-point*))
+  (end-of-line (point)))
 
 ;; Object movement comands - defined specially because FORWARD-OBJECT
 ;; and BACKWARD-OBJECT is part of the buffer protocol, not the
 ;; high-level motion abstraction.
 (define-command (com-forward-object :name t :command-table movement-table)
-    ((count 'integer :prompt "Number of objects"))
+    ((count 'integer :prompt "Number of objects" :default 1))
   "Move point forward by one object.
 With a numeric argument N, move point forward by N objects.
 With a negative argument -N, move point backward by M objects."
   (handling-motion-limit-errors ("objects")
-    (forward-object *current-point*
+    (forward-object (point)
                     count)))
 
 (define-command (com-backward-object :name t :command-table movement-table)
-    ((count 'integer :prompt "number of objects"))
+    ((count 'integer :prompt "number of objects" :default 1))
   "Move point backward by one object.
 With a numeric argument N, move point backward by N objects.
 With a negative argument -N, move point forward by N objects."
   (handling-motion-limit-errors ("objects")
-    (backward-object *current-point*
+    (backward-object (point)
                      count)))
 
 ;;; Autogenerate commands
 (define-motion-commands word movement-table)
-(define-motion-commands line movement-table)
 (define-motion-commands page movement-table)
 (define-motion-commands paragraph movement-table)
 (define-motion-commands sentence movement-table)
 
+;;; Lines have goal-columns, so we have to define the commands
+;;; manually.
+(define-command (com-forward-line :name t :command-table movement-table)
+    ((count 'integer :prompt "number of lines" :default 1))
+  "move point forward by one line.
+with a numeric argument n, move point forward by n lines.
+with a negative argument -n, move point backward by n lines."
+  (handling-motion-limit-errors ("lines")
+    (unless (member (unlisted (previous-command (drei-instance)))
+                    '(com-forward-line com-backward-line))
+      (setf (goal-column (current-view)) (column-number (point))))
+    (forward-line (point) (current-syntax) count)
+    (setf (column-number (point)) (goal-column (current-view)))))
+
+(define-command (com-backward-line :name t :command-table movement-table)
+    ((count 'integer :prompt "number of lines" :default 1))
+  "move point backward by one line.
+with a numeric argument n, move point backward by n lines.
+with a negative argument -n, move point forward by n lines."
+  (handling-motion-limit-errors ("lines")
+    (unless (member (unlisted (previous-command (drei-instance)))
+                    '(com-forward-line com-backward-line))
+      (setf (goal-column (current-view)) (column-number (point))))
+    (backward-line (point) (current-syntax) count)
+    (setf (column-number (point)) (goal-column (current-view)))))
+
 ;;; Bind gestures to commands
 (set-key `(com-forward-object ,*numeric-argument-marker*)
 	 'movement-table
@@ -278,15 +302,15 @@ With a negative argument -N, move point forward by N objects."
            ;; Kill Unit
            (define-command (,com-kill :name t
                                       :command-table ,command-table)
-               ((count 'integer :prompt ,(concat "Number of " plural)))
+               ((count 'integer :prompt ,(concat "Number of " plural) :default 1))
              ,(concat "Kill " plural " up to the next " noun " end.
 With a numeric argument, kill forward (backward if negative) 
 that many " plural ".
 
 Successive kills append to the kill ring.")
              (handling-motion-limit-errors (,plural)
-               (,forward-kill *current-point*
-                              (syntax *current-buffer*)
+               (,forward-kill (point)
+                              (current-syntax)
                               count
                               (eq (command-name *previous-command*) ',com-kill))))
 
@@ -294,31 +318,33 @@ Successive kills append to the kill ring.")
            (define-command (,com-backward-kill
                             :name t
                             :command-table ,command-table)
-               ((count 'integer :prompt ,(concat "Number of " plural)))
+               ((count 'integer :prompt ,(concat "Number of " plural) :default 1))
              ,(concat "Kill from point until the previous " noun " beginning.
 With a numeric argument, kill backward (forward, if negative) 
 that many " plural ".
 
 Successive kills append to the kill ring.")
              (handling-motion-limit-errors (,plural)
-               (,backward-kill *current-point*
-                               (syntax *current-buffer*)
+               (,backward-kill (point)
+                               (current-syntax)
                                count
                                (eq (command-name *previous-command*) ',com-backward-kill))))
 
            ;; Delete Unit
            (define-command (,com-delete :name t :command-table ,command-table)
-               ((count 'integer :prompt ,(concat "Number of " plural)))
+               ((count 'integer :prompt ,(concat "Number of " plural) :default 1))
              ,(concat "Delete from point until the next " noun " end.
 With a positive numeric argument, delete that many " plural " forward.")
-             (,backward-delete *current-point* (syntax *current-buffer*) count))
+             (handling-motion-limit-errors (,plural)
+              (,backward-delete (point) (current-syntax) count)))
 
            ;; Backward Delete Unit
            (define-command (,com-backward-delete :name t :command-table ,command-table)
-               ((count 'integer :prompt ,(concat "Number of " plural)))
+               ((count 'integer :prompt ,(concat "Number of " plural) :default 1))
              ,(concat "Delete from point until the previous " noun " beginning.
 With a positive numeric argument, delete that many " plural " backward.")
-             (,backward-delete *current-point* (syntax *current-buffer*) count)))))))
+             (handling-motion-limit-errors (,plural)
+               (,backward-delete (point) (current-syntax) count))))))))
 
 (defmacro define-editing-commands (unit command-table &key
                                    noun
@@ -350,8 +376,7 @@ transpose that " noun " with the next one. With point
 before the first " noun " of the buffer, transpose the
 first two " plural " of the buffer.")
            (handling-motion-limit-errors (,plural)
-             (,transpose *current-point*
-                         (syntax *current-buffer*))))))))
+             (,transpose (point) (current-syntax))))))))
 
 ;;; Some manually defined commands
 
@@ -361,29 +386,29 @@ At the end of a line transpose the previous two objects without
 advancing point. At the beginning of the buffer do nothing.  At
 the beginning of any line other than the first effectively move
 the first object of that line to the end of the previous line."
-  (transpose-objects *current-point*))
+  (transpose-objects (point)))
 
 (define-command (com-delete-object :name t :command-table deletion-table)
-    ((count 'integer :prompt "Number of Objects")
-     (killp 'boolean :prompt "Kill?"))
+    ((count 'integer :prompt "Number of Objects" :default 1)
+     (killp 'boolean :prompt "Kill?" :default nil))
   "Delete the object after point.
 With a numeric argument, kill that many objects 
 after (or before, if negative) point."
   (handling-motion-limit-errors ("objects")
     (if killp
-        (forward-kill-object *current-point* count)
-        (forward-delete-object *current-point* count))))
+        (forward-kill-object (point) count)
+        (forward-delete-object (point) count))))
 
 (define-command (com-backward-delete-object :name t :command-table deletion-table)
-    ((count 'integer :prompt "Number of Objects")
-     (killp 'boolean :prompt "Kill?"))
+    ((count 'integer :prompt "Number of Objects" :default 1)
+     (killp 'boolean :prompt "Kill?" :default nil))
   "Delete the object before point.
 With a numeric argument, kills that many objects 
 before (or after, if negative) point."
   (handling-motion-limit-errors ("objects")
     (if killp
-        (backward-kill-object *current-point* count #'error-limit-action)
-        (backward-delete-object *current-point* count #'error-limit-action))))
+        (backward-kill-object (point) count #'error-limit-action)
+        (backward-delete-object (point) count #'error-limit-action))))
 
 ;; We require somewhat special behavior from Kill Line, so define a
 ;; new function and use that to implement the Kill Line command.
@@ -416,8 +441,8 @@ before (or after, if negative) point."
       (delete-region start mark))))
 
 (define-command (com-kill-line :name t :command-table deletion-table)
-    ((numarg 'integer :prompt "Kill how many lines?")
-     (numargp 'boolean :prompt "Kill entire lines?"))
+    ((numarg 'integer :prompt "Kill how many lines?" :default 1)
+     (numargp 'boolean :prompt "Kill entire lines?" :default nil))
   "Kill the objects on the current line after point.
 When at the end of a line, kill the #\\Newline. 
 With a numeric argument of 0, kill the objects on the current line before point.
@@ -426,7 +451,7 @@ if negative) from point.
 
 Successive kills append to the kill ring."
   (let* ((concatenate-p (eq (command-name *previous-command*) 'com-kill-line)))
-    (user-kill-line *current-point* numarg numargp concatenate-p)))
+    (user-kill-line (point) numarg numargp concatenate-p)))
 
 ;;; Autogenerate commands
 
@@ -457,17 +482,17 @@ Successive kills append to the kill ring."
 	 '((#\x :control) (#\t :control)))
 
 (set-key `(com-delete-object ,*numeric-argument-marker*
-			     ,*numeric-argument-p*)
+			     ,*numeric-argument-marker*)
 	 'deletion-table
 	 '(#\Rubout))
 
 (set-key `(com-delete-object ,*numeric-argument-marker*
-			     ,*numeric-argument-p*)
+			     ,*numeric-argument-marker*)
 	 'deletion-table
 	 '((#\d :control)))
 
 (set-key `(com-backward-delete-object ,*numeric-argument-marker*
-				      ,*numeric-argument-p*)
+				      ,*numeric-argument-marker*)
 	 'deletion-table
 	 '(#\Backspace))
 
@@ -475,7 +500,7 @@ Successive kills append to the kill ring."
 	 'editing-table
 	 '((#\t :control)))
 
-(set-key `(com-kill-line ,*numeric-argument-marker* ,*numeric-argument-p*)
+(set-key `(com-kill-line ,*numeric-argument-marker* ,*numeric-argument-marker*)
 	 'deletion-table
 	 '((#\k :control)))
 
@@ -486,14 +511,20 @@ Successive kills append to the kill ring."
 ;;; These are what do the basic keypress->character inserted in buffer
 ;;; mapping.
 
-(define-command com-self-insert ((count 'integer))
-  (loop repeat count do (insert-character *current-gesture*)))
+(define-command com-self-insert
+    ((count 'integer :default 1))
+  "Insert the gesture used to invoke this command into the
+current buffer `count' times. `Count' should get its value from
+the numeric arguments."
+  (loop repeat count
+        do (insert-character *current-gesture*)))
 
-(loop for code from (char-code #\Space) to (char-code #\~)
-      do (set-key `(com-self-insert ,*numeric-argument-marker*)
-	     'self-insert-table
-	     (list (list (code-char code)))))
+(defmethod command-for-unbound-gestures ((view textual-drei-syntax-view) gestures)
+  (when (and (= (length gestures) 1)
+             (characterp (first gestures))
+             (graphic-char-p (first gestures)))
+    `(com-self-insert ,*numeric-argument-marker*)))
 
 (set-key `(com-self-insert ,*numeric-argument-marker*)
-	 'self-insert-table
-	 '((#\Newline)))
\ No newline at end of file
+         'self-insert-table
+	 '((#\Newline)))
