@@ -53,22 +53,27 @@
 
 ;;; NOTE: Despite the name, this does not have anything to do with
 ;;; CLIM cursors, though perhaps this facility should be built on top
-;;; of what CLIM already provides. That seemed a bit hairy, though.
+;;; of what CLIM already provides. That seemed a bit (=a lot) hairy,
+;;; though.
+
+;;; Cursors are output records. After a cursor is created, The owning
+;;; Drei instance instnace should add it to the output stream. The
+;;; owner of the cursor (a Drei instance) is responsible for removing
+;;; the cursor once it is done with it. Cursors can be active/inactive
+;;; and enabled/disabled and have the same activity-status as their
+;;; associated view.
 (defclass drei-cursor (standard-sequence-output-record)
-  ((%drei-instance :reader drei-instance
-                   :initarg :drei-instance
-                   :initform (error "A Drei cursor must be associated with a Drei instance"))
-   (%mark :reader mark
+  ((%view :reader view
+          :initarg :view
+          :initform (error "A Drei cursor must be associated with a Drei view")
+          :type drei-view)
+   (%output-stream :reader output-stream
+                   :initarg :output-stream
+                   :initform (error "A Drei cursor must be associated with an output stream")
+                   :type extended-output-stream)
+   (%mark :reader mark-of
           :initarg :mark
           :initform (error "A Drei cursor must be associated with a mark."))
-   (%active :accessor active
-            :initarg :active
-            :initform t
-            :type boolean
-            :documentation "Whether the cursor is active or
-not. An active cursor is drawn using the active ink, and an
-inactive is drawn using the inactive ink. Typically, a cursor
-will be active when the associated Drei instance has focus.")
    (%enabled :accessor enabled
              :initarg :enabled
              :initform t
@@ -78,8 +83,8 @@ drawn when its associated Drei instance is drawn. When it is not
 enabled, it will simply be ignored during redisplay.")
    (%active-ink :accessor active-ink
                 :initarg :active-ink
-                :initform +red+
-                :type color
+                :initform +flipping-ink+
+                :type design
                 :documentation "The ink used to draw the cursor
 when it is active.")
    (%inactive-ink :accessor inactive-ink
@@ -92,6 +97,14 @@ when it is inactive."))
 Drei buffer. The most important role for instances of subclasses
 of this class is to visually represent the position of point."))
 
+(defgeneric active (cursor)
+  (:documentation "Whether the cursor is active or
+not. An active cursor is drawn using the active ink, and an
+inactive is drawn using the inactive ink. Typically, a cursor
+will be active when the associated Drei view has focus.")
+  (:method ((cursor drei-cursor))
+    (active (view cursor))))
+
 (defgeneric ink (cursor)
   (:documentation "Return the ink object that should be used for
   displaying the given cursor."))
@@ -101,19 +114,19 @@ of this class is to visually represent the position of point."))
       (active-ink cursor)
       (inactive-ink cursor)))
 
-(defmethod (setf enabled) ((new-value null) (cursor drei-cursor))
-  (erase-output-record cursor (editor-pane (drei-instance cursor)) nil))
+(defmethod (setf enabled) :after ((new-val null) (cursor drei-cursor))
+  (clear-output-record cursor))
 
 (defclass point-cursor (drei-cursor)
   ()
   (:default-initargs
    :mark nil
-    :active t)
+   :active t)
   (:documentation "A class that should be used for the visual
 representation of the point of a Drei instance."))
 
-(defmethod mark ((cursor point-cursor))
-  (point (drei-instance cursor)))
+(defmethod mark-of ((cursor point-cursor))
+  (point (view cursor)))
 
 (defclass mark-cursor (drei-cursor)
   ()
@@ -125,40 +138,50 @@ representation of the point of a Drei instance."))
   (:documentation "A class that should be used for the visual
 representation of the mark of a Drei instance."))
 
-(defmethod mark ((cursor mark-cursor))
-  (mark (drei-instance cursor)))
+(defmethod mark-of ((cursor mark-cursor))
+  (mark (view cursor)))
 
 (defmethod enabled ((cursor mark-cursor))
-  *show-mark*)
+  (and (call-next-method) *show-mark*))
 
-(defgeneric visible (cursor drei)
-  (:documentation "Is `cursor', associated with `drei', visible?
+(defgeneric visible-1 (cursor view)
+  (:documentation "Is `cursor', associated with `view', visible?
 If this function returns true, it is assumed that it is safe to
 display `cursor' to the editor stream. If just one of the
 applicable methods returns false, the entire function returns
 false.")
   (:method-combination and)
-  (:method and (cursor drei)
+  (:method and (cursor view)
     (enabled cursor)))
 
+(defun visible-p (cursor)
+  "Return true if `cursor' is visible. This is a trampoline
+function that calls `visible-1' with `cursor' and the view of
+`cursor'."
+  (visible-1 cursor (view cursor)))
+
 ;;; Drei instances.
 
 (defclass drei-pane (drei application-pane)
   ()
   (:default-initargs
-   :incremental-redisplay t
-    :end-of-line-action :scroll
-    :background *background-color*
-    :foreground *foreground-color*
-    :display-function 'display-drei-pane
-    :default-view +drei-textual-view+
-    :width 900
-    :active nil)
+   :incremental-redisplay nil
+   :end-of-line-action :scroll
+   :background *background-color*
+   :foreground *foreground-color*
+   :display-function 'display-drei-pane
+   :width 900
+   :active nil)
+  (:metaclass modual-class)
   (:documentation "An actual, instantiable Drei pane that
 permits (and requires) the host application to control the
 command loop completely."))
 
-(defmethod display-drei ((drei drei-pane))
+(defmethod stream-default-view ((stream drei-pane))
+  (view stream))
+
+(defmethod display-drei ((drei drei-pane) &rest args)
+  (declare (ignore args))
   (redisplay-frame-pane (pane-frame drei) drei))
 
 (defmethod editor-pane ((drei drei-pane))
@@ -166,28 +189,28 @@ command loop completely."))
   ;; display surface.
   drei)
 
-(defmethod visible and (cursor (drei drei-pane))
+(defmethod visible-1 and (cursor (view drei-buffer-view))
   ;; We should only redisplay when the cursor is on display, or
-  ;; `offset-to-screen-position' will return a non-number.
-  (<= (offset (top drei))
-      (offset (mark cursor))
-      (offset (bot drei))))
+  ;; `offset-to-screen-position' will return a non-number. Also don't
+  ;; display if the view hasn't been displayed yet.
+  (and (<= (offset (top view))
+           (offset (mark cursor))
+           (offset (bot view)))
+       (plusp (displayed-lines-count view))))
 
-(defmethod tab-width ((pane drei-pane))
-  (tab-width (stream-default-view pane)))
+(defmethod (setf view) :after (new-val (drei drei-pane))
+  (window-clear drei))
 
-(defmethod space-width ((pane drei-pane))
-  (space-width (stream-default-view pane)))
+(defmethod (setf cursors) :around (new-cursors (drei drei-pane))
+  (let ((old-cursors (cursors drei)))
+    (call-next-method)
+    (dolist (old-cursor old-cursors)
+      (erase-output-record old-cursor drei nil))
+    (dolist (new-cursor new-cursors)
+      (stream-add-output-record drei new-cursor))))
 
-(defmethod note-sheet-grafted :around ((pane drei-pane))
-  (call-next-method)
-  (setf (stream-default-view pane) (view pane))
-  (with-slots (space-width tab-width) (stream-default-view pane)
-    (with-sheet-medium (medium pane)
-      (setf (medium-text-style medium) (pane-text-style pane))
-      (let ((style (medium-text-style medium)))
-        (setf space-width (text-size medium " " :text-style style)
-              tab-width (* 8 space-width))))))
+(defmethod note-sheet-grafted :after ((pane drei-pane))
+  (setf (stream-default-view pane) (view pane)))
 
 ;;; The fun is that in the gadget version of Drei, we do not control
 ;;; the application command loop, and in fact, need to operate
@@ -212,7 +235,9 @@ command loop completely."))
    (%previous-focus :accessor previous-focus :initform nil
                     :documentation "The pane that previously had
 keyboard focus"))
-  (:default-initargs :command-executor 'execute-drei-command)
+  (:metaclass modual-class)
+  (:default-initargs
+   :command-executor 'execute-drei-command)
   (:documentation "An actual, instantiable Drei gadget with
  event-based command processing."))
 
@@ -229,8 +254,8 @@ keyboard focus"))
   ;; literal objects. We return a string if we can, an array
   ;; otherwise. This is a bit slow, as we cons up the array and then
   ;; probably a new one for the string, most of the time.
-  (let ((contents (buffer-sequence (buffer gadget)
-                                   0 (size (buffer gadget)))))
+  (let ((contents (buffer-sequence (buffer (view gadget))
+                                   0 (size (buffer (view gadget))))))
     (if (every #'characterp contents)
         (coerce contents 'string)
         contents)))
@@ -239,10 +264,10 @@ keyboard focus"))
                                 &key (invoke-callback t))
   ;; I think we're supposed to permit this, even if the buffer is
   ;; non-editable.
-  (letf (((read-only-p (buffer gadget)) nil))
+  (letf (((read-only-p (buffer (view gadget))) nil))
     (performing-drei-operations (gadget :with-undo nil :redisplay nil)
-      (delete-buffer-range (buffer gadget) 0 (size (buffer gadget)))
-      (insert-buffer-sequence (buffer gadget) 0 new-value)))
+      (delete-buffer-range (buffer (view gadget)) 0 (size (buffer (view gadget))))
+      (insert-buffer-sequence (buffer (view gadget)) 0 new-value)))
   (when invoke-callback
     (value-changed-callback gadget
                             (gadget-client gadget)
@@ -264,40 +289,26 @@ keyboard focus"))
 Drei gadget variant has determined that a keyboard event
 corresponds to a useful gesture that should be handled. A useful
 gesture is, for example, one that is not simply a click on a
-modifier key. When this function is called, the Drei special
-variables (`*current-window*', `*current-buffer*', etc) are
-properly bound."))
+modifier key."))
 
 (defmethod handle-gesture ((drei drei-gadget-pane) gesture)
   (let ((*command-processor* drei)
-        (*abort-gestures* *esa-abort-gestures*))
-    ;; It is important that the minibuffer of the Drei object is
-    ;; actually the minibuffer that will be used for output, or it
-    ;; will not be properly redisplayed by `display-drei'.
+        (*abort-gestures* *esa-abort-gestures*)
+        (*standard-input* drei))
     (accepting-from-user (drei)
-      (letf (((minibuffer drei) (or (minibuffer drei) *minibuffer*
-                                    (unless (eq drei *standard-input*)
-                                      *standard-input*))))
-        (handler-case (process-gesture drei gesture)
-          (unbound-gesture-sequence (c)
-            (display-message "~A is unbound" (gesture-name (gestures c))))
-          (abort-gesture ()
-            (display-message "Aborted")))
-        (display-drei drei)
-        (when (modified-p (buffer drei))                   
-          (clear-modify (buffer drei))
-          (when (gadget-value-changed-callback drei)
-            (value-changed-callback drei
-                                    (gadget-client drei)
-                                    (gadget-id drei)
-                                    (gadget-value drei))))))))
-
-(defmethod execute-drei-command :after ((drei drei-gadget-pane) command)
-  (with-accessors ((buffer buffer)) drei
-    (when (syntax buffer)
-      (update-syntax buffer (syntax buffer)))
-    (when (modified-p buffer)
-      (setf (needs-saving buffer) t))))
+      (handler-case (process-gesture drei gesture)
+        (unbound-gesture-sequence (c)
+          (display-message "~A is unbound" (gesture-name (gestures c))))
+        (abort-gesture ()
+          (display-message "Aborted")))
+      (display-drei drei :redisplay-minibuffer t)
+      (when (modified-p (view drei))
+        (when (gadget-value-changed-callback drei)
+          (value-changed-callback drei
+                                  (gadget-client drei)
+                                  (gadget-id drei)
+                                  (gadget-value drei)))
+        (setf (modified-p (view drei)) nil)))))
 
 ;;; This is the method that functions as the entry point for all Drei
 ;;; gadget logic.
@@ -307,8 +318,7 @@ properly bound."))
       (let ((gesture (convert-to-gesture event)))
         (when (proper-gesture-p gesture)
           (with-bound-drei-special-variables (gadget :prompt (format nil "~A " (gesture-name gesture)))
-            (let ((*standard-input* (or *minibuffer* *standard-input*)))
-              (handle-gesture gadget gesture))))))))
+            (handle-gesture gadget gesture)))))))
 
 (defmethod handle-event :before 
     ((gadget drei-gadget-pane) (event pointer-button-press-event))
@@ -327,9 +337,10 @@ properly bound."))
 
 (defmethod additional-command-tables append ((drei drei-gadget-pane)
                                              (table drei-command-table))
-  `(exclusive-gadget-table))
+  `(exclusive-gadget-table
+    ,(frame-command-table *application-frame*)))
 
-(defclass drei-area (drei standard-sequence-output-record
+(defclass drei-area (drei displayed-output-record region
                           command-processor
                           instant-macro-execution-mixin)
   ((%background-ink :initarg :background-ink
@@ -342,25 +353,155 @@ properly bound."))
 editable area. Should be an integer >= 0 or T, meaning that it
 will extend to the end of the viewport, if the Drei area is in a
 scrolling arrangement.")
-   (%drei-position :accessor input-editor-position
-                   :initarg :input-editor-position
-                   :documentation "The position of the Drei
+   (%position :accessor area-position
+              :initarg :area-position
+              :documentation "The position of the Drei
 editing area in the coordinate system of the encapsulated
 stream. An (X,Y) list, not necessarily the same as the position
-of the associated output record."))
-  (:default-initargs :command-executor 'execute-drei-command)
+of the associated output record.")
+   (%parent-output-record :accessor output-record-parent
+                          :initarg :parent
+                          :initform nil
+                          :documentation "The parent output
+record of the Drei area instance."))
+  (:metaclass modual-class)
+  (:default-initargs
+   :command-executor 'execute-drei-command)
   (:documentation "A Drei editable area implemented as an output
 record."))
 
 (defmethod initialize-instance :after ((area drei-area)
-				       &key)
-  (setf (input-editor-position area)
-        (multiple-value-list (output-record-position area)))
-  (tree-recompute-extent area))
+				       &key x-position y-position)
+  (check-type x-position number)
+  (check-type y-position number)
+  (setf (area-position area) (list x-position y-position)
+        (extend-pane-bottom (view area)) t))
 
-(defmethod display-drei ((drei drei-area))
+(defmethod (setf view) :after ((new-view drei-view) (drei drei-area))
+  (setf (extend-pane-bottom new-view) t))
+
+(defmethod (setf cursors) :after (new-cursors (drei drei-area))
+  (dolist (new-cursor (cursors drei))
+    (setf (output-record-parent new-cursor) drei)))
+
+(defmethod esa-current-window ((drei drei-area))
+  (editor-pane drei))
+
+(defmethod display-drei ((drei drei-area) &rest args)
+  (declare (ignore args))
   (display-drei-area drei))
 
+;;; Implementation of the displayed-output-record and region protocol
+;;; for Drei areas. The redisplay-related stuff is in
+;;; drei-redisplay.lisp.
+
+(defmethod output-record-position ((record drei-area))
+  (values-list (area-position record)))
+
+(defmethod* (setf output-record-position) ((new-x number) (new-y number)
+                                           (record drei-area))
+  (multiple-value-bind (old-x old-y) (output-record-position record)
+    (setf (area-position record) (list new-x new-y))
+    (dolist (cursor (cursors record))
+      (multiple-value-bind (cursor-x cursor-y) (output-record-position cursor)
+        (setf (output-record-position cursor)
+              (values (+ (- cursor-x old-x) new-x)
+                      (+ (- cursor-y old-y) new-y)))))))
+
+(defmethod output-record-start-cursor-position ((record drei-area))
+  (output-record-position record))
+
+(defmethod* (setf output-record-start-cursor-position) ((new-x number) (new-y number)
+                                                       (record drei-area))
+  (setf (output-record-position record) (values new-x new-y)))
+
+(defmethod output-record-hit-detection-rectangle* ((record drei-area))
+  (bounding-rectangle* record))
+
+(defmethod output-record-refined-position-test ((record drei-area) x y)
+  t)
+
+(defmethod displayed-output-record-ink ((record drei-area))
+  +foreground-ink+)
+
+(defmethod output-record-children ((record drei-area))
+  (cursors record))
+
+(defmethod output-record-count ((record drei-area))
+  (length (cursors record)))
+
+(defmethod map-over-output-records-containing-position
+    (function (record drei-area) x y
+     &optional (x-offset 0) (y-offset 0)
+     &rest function-args)
+  (declare (ignore x-offset y-offset))
+  (dolist (cursor (cursors record))
+    (when (region-contains-position-p cursor x y)
+      (apply function cursor function-args))))
+
+(defmethod map-over-output-records-overlapping-region
+    (function (record drei-area) region
+     &optional (x-offset 0) (y-offset 0)
+     &rest function-args)
+  (declare (ignore x-offset y-offset))
+  (dolist (cursor (cursors record))
+    (when (region-intersects-region-p cursor region)
+      (apply function cursor function-args))))
+
+(defmethod bounding-rectangle* ((drei drei-area))
+  (with-accessors ((pane editor-pane)
+                   (min-width min-width)) drei
+    (let* ((style (medium-text-style pane))
+           (style-width (text-style-width style pane))
+           (height (text-style-height style pane)))
+      (multiple-value-bind (x1 y1 x2 y2)
+          (drei-bounding-rectangle* drei)
+        (when (= x1 y1 x2 y2 0)
+          ;; It hasn't been displayed yet, so stuff the position into
+          ;; it...
+          (setf x1 (first (area-position drei))
+                y1 (second (area-position drei))))
+        (values x1 y1
+                (max x2 (+ x1 style-width)
+                     (cond ((numberp min-width)
+                            (+ x1 min-width))
+                           ;; Must be T, then.
+                           ((pane-viewport pane)
+                            (+ x1 (bounding-rectangle-width (pane-viewport-region pane))))
+                           (t 0)))
+                (max y2 (+ y1 height)))))))
+
+(defmethod replay-output-record :after ((drei drei-area) (stream extended-output-stream)
+                                        &optional (x-offset 0) (y-offset 0) (region +everywhere+))
+  (declare (ignore x-offset y-offset region))
+  (dolist (cursor (cursors drei))
+    (replay cursor stream)))
+
+(defmethod recompute-extent-for-changed-child  ((drei drei-area) (child output-record)
+                                                old-min-x old-min-y old-max-x old-max-y)
+  nil)
+
+(defmethod rectangle-edges* ((rectangle drei-area))
+  (bounding-rectangle* rectangle))
+
+(defmethod region-union ((region1 drei-area) region2)
+  (region-union (bounding-rectangle region1) region2))
+
+(defmethod region-union (region1 (region2 drei-area))
+  (region-union region1 (bounding-rectangle region2)))
+
+(defmethod region-intersection ((region1 drei-area) region2)
+  (region-intersection (bounding-rectangle region1) region2))
+
+(defmethod region-intersection (region1 (region2 drei-area))
+  (region-intersection region1 (bounding-rectangle region2)))
+
+(defmethod region-difference ((region1 drei-area) region2)
+  (region-difference (bounding-rectangle region1) region2))
+
+(defmethod region-difference (region1 (region2 drei-area))
+  (region-difference region1 (bounding-rectangle region2)))
+
 ;; For areas, we need to switch to ESA abort gestures after we have
 ;; left the CLIM gesture reading machinery, but before we start doing
 ;; ESA gesture processing.
@@ -390,8 +531,8 @@ record."))
   (:documentation "A constellation of a Drei gadget instance and
   a minibuffer."))
 
-(defmethod display-drei :after ((drei drei))
-  (when (and *minibuffer* (not (eq *minibuffer* (editor-pane drei))))
+(defmethod display-drei :after ((drei drei) &key redisplay-minibuffer)
+  (when (and *minibuffer* redisplay-minibuffer)
     ;; We need to use :force-p t to remove any existing output from
     ;; the pane.
     (redisplay-frame-pane (pane-frame *minibuffer*) *minibuffer* :force-p t)))
@@ -412,12 +553,13 @@ record."))
                                    (syntax nil) (initial-contents "")
                                    (minibuffer t) (border-width 1)
                                    (scroll-bars :horizontal)
-                                   (drei-class 'drei-gadget-pane))
+                                   (drei-class 'drei-gadget-pane)
+                                   (view 'textual-drei-syntax-view))
   (check-type initial-contents array)
   (check-type border-width integer)
   (check-type scroll-bars (member t :both :vertical :horizontal nil))
   (with-keywords-removed (args (:minibuffer :scroll-bars :border-width
-                                            :syntax :drei-class))
+                                                         :syntax :drei-class :view))
     (let* ((borderp (and border-width (plusp border-width)))
            (minibuffer-pane (cond ((eq minibuffer t)
                                    (make-pane 'drei-minibuffer-pane))
@@ -428,26 +570,28 @@ record."))
                                   (t (error "Provided minibuffer
 is not T, NIL or a `minibuffer-pane'."))))
            (drei-pane (apply #'make-pane-1 fm frame drei-class
-                             :minibuffer minibuffer-pane args))
-           (pane drei-pane))
-      (letf (((read-only-p (buffer drei-pane)) nil))
-        (insert-sequence (point drei-pane) initial-contents))
+                       :minibuffer minibuffer-pane
+                       :view (make-instance view)
+                       args))
+           (pane drei-pane)
+           (view (view drei-pane)))
+      (letf (((read-only-p (buffer view)) nil))
+        (insert-buffer-sequence (buffer view) 0 initial-contents))
       (if syntax
-          (setf (syntax (buffer drei-pane))
+          (setf (syntax view)
                 (make-instance (or (when (syntaxp syntax)
                                      syntax)
                                    (syntax-from-name (string syntax))
                                    (error "Syntax ~A not found" (string syntax)))
-                               :buffer (buffer drei-pane)))
-          (update-syntax (buffer drei-pane) (syntax (buffer drei-pane))))
+                 :buffer (buffer view))))
       (when scroll-bars
         (setf pane (scrolling (:scroll-bar scroll-bars)
                      pane)))
       (when minibuffer
         (setf pane (make-pane 'drei-constellation
-                              :drei drei-pane
-                              :minibuffer minibuffer-pane
-                              :contents (list pane minibuffer-pane))))
+                    :drei drei-pane
+                    :minibuffer minibuffer-pane
+                    :contents (list pane minibuffer-pane))))
       (when borderp
         (setf pane (#+(or mcclim building-mcclim)
                       climi::bordering
