@@ -33,7 +33,7 @@
 ;; `drei-input-editing-mixin' class does not have a scan pointer. We
 ;; assume that the subclass defines a scan pointer.
 (defclass drei-input-editing-mixin ()
-  ((%drei-instance :accessor drei-instance
+  ((%drei-instance :accessor drei-instance-of
                    :initarg :drei-instance)
    (%input-position :accessor input-position
                     :initform 0)
@@ -53,7 +53,7 @@ instantiated."))
 
 (defmethod initialize-instance :after ((obj drei-input-editing-mixin)
 				       &rest args
-				       &key stream (initial-contents "")
+				       &key stream
 				       (cursor-visibility t)
                                        (min-width 0))
   (check-type min-width (or (integer 0) (eql t)))
@@ -66,38 +66,36 @@ instantiated."))
 	      (apply #'make-instance
 		     'drei-area
 		     :editor-pane stream
-		     :buffer (make-instance 'drei-buffer
-                                            :name "Input-editor buffer"
-                                            :initial-contents initial-contents)
 		     :x-position cx
 		     :y-position cy
 		     :active cursor-visibility
 		     :max-width max-width
-                     :minibuffer (or *minibuffer*
-                                     *pointer-documentation-output*)
                      :allow-other-keys t
 		     args)))
-      (update-syntax (buffer (drei-instance obj))
-                     (syntax (buffer (drei-instance obj))))
       ;; XXX Really add it here?
-      (stream-add-output-record stream (drei-instance obj))
-      (display-drei (drei-instance obj)))))
+      (stream-add-output-record stream (drei-instance obj)))))
+
+(defmethod stream-default-view ((stream drei-input-editing-mixin))
+  (view (drei-instance stream)))
 
 (defmethod stream-insertion-pointer
     ((stream drei-input-editing-mixin))
-  (offset (point (drei-instance stream))))
+  (offset (point (view (drei-instance stream)))))
 
 (defmethod (setf stream-insertion-pointer)
     ((new-value integer) (stream drei-input-editing-mixin))
-  (let* ((drei (drei-instance stream)))
-    (setf (offset (point drei)) new-value)))
+  (setf (offset (point (view (drei-instance stream)))) new-value))
 
 (defmethod cursor-visibility ((stream drei-input-editing-mixin))
-  (active (point-cursor (drei-instance stream))))
+  (if (point-cursor (drei-instance stream))
+      (active (point-cursor (drei-instance stream)))
+      ;; Uh... no I guess?
+      nil))
 
 (defmethod (setf cursor-visibility)
     (visibility (stream drei-input-editing-mixin))
-  (setf (active (drei-instance stream)) visibility))
+  (setf (active (drei-instance stream)) visibility
+        (cursors-visible (drei-instance stream)) visibility))
 
 (defclass drei-unselectable-presentation (presentation)
   ()
@@ -125,7 +123,7 @@ by the input-editor gesture reader. They should not be used
 outside the input editor."))
 
 (define-presentation-method present ((object noise-string) (type noise-string)
-                                     stream (view drei-textual-view) &key &allow-other-keys)
+                                     stream (view textual-view) &key &allow-other-keys)
   (with-text-style (stream (text-style object))
     (princ (noisy-string object) stream)))
 
@@ -142,7 +140,7 @@ while in an input context for the input-editor. They should not
 be used outside the input-editor."))
 
 (define-presentation-method present (object (type accept-result) stream
-                                            (view drei-textual-view) &rest args
+                                            (view textual-view) &rest args
                                             &key)
   (apply #'present (object object) (result-type object) :stream stream :view view args))
 
@@ -155,25 +153,30 @@ be used outside the input-editor."))
   ;; already at the input position or if we are rescanning. This is so
   ;; we can support fancy accept methods such as the one for
   ;; `command-or-form'
-  (unless (or (stream-rescanning-p stream)
-              (= (stream-scan-pointer stream) (input-position stream)))
-    (call-next-method)
-    ;; We skip ahead of any noise strings to put us past the
-    ;; prompt. This is safe, because the noise strings are to be
-    ;; ignored anyway, but we need to be ahead to set the input
-    ;; position properly (ie. after the prompt).
-    (loop
-       with buffer = (buffer (drei-instance stream))
-       until (>= (stream-scan-pointer stream) (size buffer))
-       while (or (typep #1=(buffer-object buffer (stream-scan-pointer stream)) 'noise-string)
-                 (delimiter-gesture-p #1#))
-       do (incf (stream-scan-pointer stream)))
-    (setf (input-position stream) (stream-scan-pointer stream))))
+  (unless (stream-rescanning-p stream)
+    ;; Put the prompt in the proper place, but be super careful not to
+    ;; mess with the insertion pointer.
+    (let ((ip-clone (clone-mark (point (view (drei-instance stream))))))
+      (unwind-protect (progn (setf (stream-insertion-pointer stream)
+                                   (stream-scan-pointer stream))
+                             (call-next-method))
+        (setf (stream-insertion-pointer stream) (offset ip-clone)))
+      (redraw-input-buffer stream)))
+  ;; We skip ahead of any noise strings to put us past the
+  ;; prompt. This is safe, because the noise strings are to be
+  ;; ignored anyway, but we need to be ahead to set the input
+  ;; position properly (ie. after the prompt).
+  (loop with buffer = (buffer (view (drei-instance stream)))
+        until (>= (stream-scan-pointer stream) (size buffer))
+        while (or (typep #1=(buffer-object buffer (stream-scan-pointer stream)) 'noise-string)
+                  (delimiter-gesture-p #1#))
+        do (incf (stream-scan-pointer stream)))
+  (setf (input-position stream) (stream-scan-pointer stream)))
 
 (defmethod stream-accept :after ((stream drei-input-editing-mixin) type &key &allow-other-keys)
   ;; If we end up asking for more input using the stream, we do not
   ;; want to permit the user to undo input for this context.
-  (clear-undo-history (buffer (drei-instance stream))))
+  (clear-undo-history (buffer (view (drei-instance stream)))))
 
 (defun buffer-array-mismatch (sequence1 sequence2 
                               &key (from-end nil)
@@ -219,7 +222,7 @@ contents of the array to the Drei buffer. This will set the
 contents of the buffer to the contents of the array up to the
 fill pointer."
   (with-accessors ((array input-buffer-array)) stream
-    (let ((buffer (buffer (drei-instance stream))))
+    (let ((buffer (buffer (view (drei-instance stream)))))
       (when array
         ;; Attempt to minimise the changes to the buffer, so the
         ;; position of marks will not be changed too much. Find the
@@ -253,8 +256,6 @@ fill pointer."
               ;; Insert from the mismatch to end mismatch from the
               ;; array into the buffer.
               (insert-buffer-sequence buffer index (subseq array index array-end))
-              ;; We also need to update the syntax.
-              (update-syntax buffer (syntax buffer))
               ;; Finally, see if it is possible to maintain the old
               ;; position of the insertion pointer.
               (setf (stream-insertion-pointer stream)
@@ -265,7 +266,7 @@ fill pointer."
 contents of the Drei buffer to the array. The fill pointer of the
 array will point to after the last element."
   (with-accessors ((array input-buffer-array)) stream
-    (let ((buffer (buffer (drei-instance stream))))
+    (let ((buffer (buffer (view (drei-instance stream)))))
       (when array
         (let ((new-array (buffer-sequence buffer 0 (size buffer))))
           (setf array
@@ -282,7 +283,7 @@ the contents of the buffer to the contents of the array up to the
 fill pointer. Changes to the buffer will be recordes as
 undoable. When this function returns, the `input-buffer-array' of
 `stream' will be NIL. Also, the syntax will be up-to-date."
-  (with-undo ((list (buffer (drei-instance stream))))
+  (with-undo ((list (buffer (view (drei-instance stream)))))
     (synchronize-drei-buffer stream))
   (setf (input-buffer-array stream) nil))
 
@@ -322,19 +323,19 @@ undoable. When this function returns, the `input-buffer-array' of
   ;; buffer before we do anything.
   (synchronize-drei-buffer stream)
   (let* ((drei (drei-instance stream))
+         (view (view drei))
          (new-contents (subseq new-input start end))
-         (old-contents (buffer-sequence (buffer drei)
+         (old-contents (buffer-sequence (buffer view)
                                         buffer-start
                                         (stream-scan-pointer stream)))
          (equal (and (= (length new-contents)
                         (length old-contents))
                      (every #'equal new-contents old-contents))))
-    (let ((begin-mark (clone-mark (point drei))))
+    (let ((begin-mark (clone-mark (point view))))
       (unless equal
         (setf (offset begin-mark) buffer-start)
         (delete-region begin-mark (stream-scan-pointer stream))
         (insert-sequence begin-mark new-contents)
-        (update-syntax (buffer drei) (syntax (buffer drei)))
         ;; Make the buffer reflect the changes in the array.
         (synchronize-input-buffer-array stream))
       (display-drei drei)
@@ -348,7 +349,7 @@ undoable. When this function returns, the `input-buffer-array' of
                                                 (length old-contents))))
       ;; We have to return "the position in the input buffer". We
       ;; return the insertion position.
-      buffer-start)))
+      (stream-insertion-pointer stream))))
 
 (defun present-acceptably-to-string (object type view for-context-type)
   "Return two values - a string containing the printed
@@ -480,60 +481,59 @@ input-editing-stream. Bound when executing a command.")
     (with-accessors ((insertion-pointer stream-insertion-pointer)
                      (scan-pointer stream-scan-pointer)
                      (activation-gesture activation-gesture)) stream
-      (let ((buffer (buffer (drei-instance stream))))
-        (loop
-           (loop
-              while (< scan-pointer insertion-pointer)
-              while (< scan-pointer (size buffer))
-              do (let ((gesture (buffer-object buffer scan-pointer)))
-                   ;; Skip noise strings.
-                   (cond ((typep gesture 'noise-string)
-                          (incf scan-pointer))
-                         ((and (not peek-p)
-                               (typep gesture 'accept-result))
-                          (incf scan-pointer)
-                          #+(or mcclim building-mcclim)
-                          (climi::throw-object-ptype (object gesture)
-                                                     (result-type gesture)))
-                         ;; Note that this implies that
-                         ;; `stream-read-gesture' may return accept
-                         ;; results, which might as well be arbitrary
-                         ;; objects to the code calling
-                         ;; `stream-read-gesture', since it can't really
-                         ;; do anything with them except for asserting
-                         ;; that they exist. According to the spec,
-                         ;; "accept results are treated as a single
-                         ;; gesture", and this kind of behavior is
-                         ;; necessary to make sure `stream-read-gesture'
-                         ;; doesn't simply claim that there are no more
-                         ;; gestures in the input-buffer when the
-                         ;; remaining gesture(s) is an accept result.
-                         ((typep gesture 'accept-result)
-                          (return-from stream-read-gesture gesture))
-                         (t
-                          (unless peek-p
-                            (incf scan-pointer))
-                          (return-from stream-read-gesture gesture))
-                         (t (incf scan-pointer)))))
-           (setf (stream-rescanning stream) nil)
-           (when activation-gesture
-             (return-from stream-read-gesture
-               (prog1 activation-gesture
-                 (unless peek-p
-                   (setf activation-gesture nil)))))
-           ;; In McCLIM, stream-process-gesture is responsible for
-           ;; inserting characters into the buffer, changing the
-           ;; insertion pointer and possibly setting up the
-           ;; activation-gesture slot.
-           (loop
-              with gesture and type
-              do (setf (values gesture type)
-                       (apply #'stream-read-gesture
-                              (encapsulating-stream-stream stream) rest-args))
-              when (null gesture)
-              do (return-from stream-read-gesture (values gesture type))
-              when (stream-process-gesture stream gesture type)
-              do (loop-finish)))))))
+      (let ((buffer (buffer (view (drei-instance stream))))
+            (last-was-noisy nil)) ; T if last passed gesture is noise-string
+        (loop (loop while (< scan-pointer insertion-pointer)
+                    while (< scan-pointer (size buffer))
+                    do (let ((gesture (buffer-object buffer scan-pointer)))
+                         ;; Skip noise strings.
+                         (cond ((typep gesture 'noise-string)
+                                (incf scan-pointer)
+                                (setf last-was-noisy t))
+                               ((and (not peek-p)
+                                     (typep gesture 'accept-result))
+                                (incf scan-pointer)
+                                #+(or mcclim building-mcclim)
+                                (climi::throw-object-ptype (object gesture)
+                                                           (result-type gesture)))
+                               ;; Note that this implies that
+                               ;; `stream-read-gesture' may return accept
+                               ;; results, which might as well be arbitrary
+                               ;; objects to the code calling
+                               ;; `stream-read-gesture', since it can't really
+                               ;; do anything with them except for asserting
+                               ;; that they exist. According to the spec,
+                               ;; "accept results are treated as a single
+                               ;; gesture", and this kind of behavior is
+                               ;; necessary to make sure `stream-read-gesture'
+                               ;; doesn't simply claim that there are no more
+                               ;; gestures in the input-buffer when the
+                               ;; remaining gesture(s) is an accept result.
+                               ((typep gesture 'accept-result)
+                                (return-from stream-read-gesture gesture))
+                               (t
+                                (unless peek-p
+                                  (incf scan-pointer))
+                                (return-from stream-read-gesture gesture)))))
+         (unless last-was-noisy      ; This prevents double-prompting.
+           (setf (stream-rescanning stream) nil))
+         (when activation-gesture
+           (return-from stream-read-gesture
+             (prog1 activation-gesture
+               (unless peek-p
+                 (setf activation-gesture nil)))))
+         ;; In McCLIM, stream-process-gesture is responsible for
+         ;; inserting characters into the buffer, changing the
+         ;; insertion pointer and possibly setting up the
+         ;; activation-gesture slot.
+         (loop with gesture and type
+               do (setf (values gesture type)
+                        (apply #'stream-read-gesture
+                               (encapsulating-stream-stream stream) rest-args))
+               when (null gesture)
+               do (return-from stream-read-gesture (values gesture type))
+               when (stream-process-gesture stream gesture type)
+               do (loop-finish)))))))
 
 (defmethod stream-unread-gesture ((stream drei-input-editing-mixin)
 				  gesture)
@@ -556,47 +556,43 @@ the gesture that will be read in the first call to
 `stream-read-gesture' for the stream encapsulated by
 `stream'. The second return value of this function will be `type'
 if stuff is inserted after the insertion pointer."
+  (assert (<= (input-position stream) (stream-scan-pointer stream)))
   (let* ((drei (drei-instance stream))
-         (buffer (buffer drei))
+         (buffer (buffer (view drei)))
          (*command-processor* drei)
          (was-directly-processing (directly-processing-p drei))
-         (minibuffer (or (minibuffer drei) *minibuffer*))
-         (*drei-input-editing-stream* stream))
-    (with-bound-drei-special-variables (drei
-                                        ;; If the minibuffer is the
-                                        ;; stream we are encapsulating
-                                        ;; for the
-                                        ;; input-editing-stream, we
-                                        ;; don't want to use it as a
-                                        ;; minibuffer.
-                                        :minibuffer (if (eq minibuffer *standard-input*)
-                                                        *pointer-documentation-output*
-                                                        minibuffer)
-                                        :prompt "M-x ")
+         (*drei-input-editing-stream* stream)
+         (old-buffer-contents (buffer-sequence buffer 0 (size buffer))))
+    (with-bound-drei-special-variables (drei :prompt "M-x ")
       (update-drei-buffer stream)
       ;; Commands are permitted to signal immediate rescans, but
       ;; we may need to do some stuff first.
       (unwind-protect
            (accepting-from-user (drei)
-             ;; We narrow the buffer to the input position, so the user won't
-             ;; be able to erase the original command (when entering command
-             ;; arguments) or stuff like argument prompts.
-             (drei-core:with-narrowed-buffer (drei (input-position stream) t t)
+             ;; We narrow the buffer to the last object before
+             ;; input-position, so the user will not be able to
+             ;; delete arguments prompts or other things.
+             (drei-core:with-narrowed-buffer (drei
+                                              (loop for index from
+                                                    (1- (input-position stream)) above 0
+                                                    when (typep (buffer-object buffer index)
+                                                                'noise-string)
+                                                    return (1+ index)
+                                                    finally (return 0))
+                                              t t)
                (handler-case (process-gestures-or-command drei)
                  (unbound-gesture-sequence (c)
                    (display-message "~A is unbound" (gesture-name (gestures c))))
                  (abort-gesture (c)
                    (if (member (abort-gesture-event c)
                                *abort-gestures*
-                               :test #'event-matches-gesture-name-p)
+                        :test #'event-matches-gesture-name-p)
                        (signal 'abort-gesture :event (abort-gesture-event c))
                        (when was-directly-processing
                          (display-message "Aborted")))))))
         (update-drei-buffer stream))
-      ;; Will also take care of redisplaying minibuffer.
-      (display-drei drei)
-      (let ((first-mismatch (offset (high-mark buffer))))
-        (clear-modify buffer)
+      (let ((first-mismatch (buffer-array-mismatch buffer old-buffer-contents)))
+        (display-drei drei :redisplay-minibuffer t)
         (cond ((null first-mismatch)
                ;; No change actually took place, even though IP may
                ;; have moved.
@@ -625,27 +621,22 @@ if stuff is inserted after the insertion pointer."
                  (climi::gesture-match gesture *help-gestures*)
                  (climi::gesture-match gesture *possibilities-gestures*))
              (directly-processing-p (drei-instance stream)))
-    (end-of-buffer (point (drei-instance stream)))
+    (end-of-buffer (point (view (drei-instance stream))))
     (unless (= (stream-scan-pointer stream)
-               (size (buffer (drei-instance stream))))
+               (size (buffer (view (drei-instance stream)))))
       (queue-rescan stream))
     (setf (activation-gesture stream) gesture)
     (rescan-if-necessary stream)
     (return-from stream-process-gesture gesture))
-  ;; XXX: The problem is that `*original-stream*' is a subclass of
-  ;; DREI-INPUT-EDITING-MIXIN (`stream', actually) at this point,
-  ;; which has an array as input buffer (as demanded by the spec),
-  ;; while the `stream-unread-gesture' method expects an event queue.
-  (let ((*original-stream* nil)
-        (*standard-input* (encapsulating-stream-stream stream)))
-    (when (proper-gesture-p gesture)
-      (unread-gesture gesture))
-    (read-gestures-and-act stream gesture type)))
+  (when (proper-gesture-p gesture)
+    (unread-gesture gesture :stream (encapsulating-stream-stream stream)))
+  (read-gestures-and-act stream gesture type))
 
 (defmethod reset-scan-pointer ((stream drei-input-editing-mixin)
 			       &optional (scan-pointer 0))
-  (setf (stream-scan-pointer stream) scan-pointer)
-  (setf (stream-rescanning stream) t))
+  (setf (stream-scan-pointer stream) scan-pointer
+        (stream-rescanning stream) t
+        (input-position stream) (min scan-pointer (input-position stream))))
 
 ;; This has been cribbed from SPLIT-SEQUENCE and lightly modified.
 (defun split-sequence (delimiter seq &key (count nil) (remove-empty-subseqs nil) (start 0) (end nil) (test nil test-supplied) (test-not nil test-not-supplied) (key nil key-supplied))
@@ -684,6 +675,9 @@ CL:SUBSEQ into the sequence indicating where processing stopped."
 (defmethod input-editor-format ((stream drei-input-editing-mixin)
 				format-string
 				&rest format-args)
+  "Insert a noise string at the insertion-pointer of `stream'."
+  ;; Since everything inserted with this method is noise strings, we
+  ;; do not bother to modify the scan pointer or queue rescans.
   (let* ((drei (drei-instance stream))
          (output (apply #'format nil format-string format-args)))
     (when (or (stream-rescanning-p stream)
@@ -693,15 +687,12 @@ CL:SUBSEQ into the sequence indicating where processing stopped."
     ;; malfunction. Of course, the newlines inserted this way aren't
     ;; actually noise-strings. FIXME.
     (loop for (seq . rest) on (split-sequence #\Newline output)
-       when (plusp (length seq))
-       do (insert-object (point drei) (make-instance 'noise-string
-                                                     :string seq))
-       unless (null rest)
-       do (insert-object (point drei) #\Newline))
-    ;; Since everything inserted with this method is noise strings, we
-    ;; do not bother to modify the scan pointer or queue rescans.
-    (update-syntax (buffer drei) (syntax (buffer drei)))
-    (display-drei drei)))
+          when (plusp (length seq))
+          do (insert-object (point (view drei))
+                            (make-instance 'noise-string
+                             :string seq))
+          unless (null rest)
+          do (insert-object (point (view drei)) #\Newline))))
 
 (defmethod redraw-input-buffer ((stream drei-input-editing-mixin)
                                 &optional (start-position 0))
@@ -737,7 +728,10 @@ CL:SUBSEQ into the sequence indicating where processing stopped."
 
 (defun add-input-editor-command (gestures function)
   "Set up Drei so performing `gestures' will result in the
-invocation of `function' "
+invocation of `function'. Only works for Drei-based input-editing
+streams. `Function' will be called with four arguments: the
+input-editing stream, the input buffer, the gesture used to
+invoke the command, and the accumulated numeric argument."
   (set-key `(,(lambda (numeric-argument)
                       (funcall function *drei-input-editing-stream*
                                (stream-input-buffer *drei-input-editing-stream*)
@@ -746,49 +740,32 @@ invocation of `function' "
            'exclusive-input-editor-table
            gestures))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; 
-;;; `With-input-editor-typeout'
-;;;
-;;; The CLIM spec is very vague about what this function is supposed
-;;; to do, but the Franz users guide suggests that it is to be used to
-;;; display information to the user while editing, sort of what we
-;;; normally use a minibuffer for. Perhaps the output should be put in
-;;; an output record above or below the editing area, but for now, we
-;;; just put it in the minibuffer. That also means the `:erase'
-;;; keyword argument is meaningless. We do add some extra limitations,
-;;; though (check the docstring)
-
-(defgeneric invoke-with-input-editor-typeout (stream continuation &key erase)
-  (:documentation "Call `continuation' with a single argument, a
-stream to do input-editor-typeout on."))
-
-(defmethod invoke-with-input-editor-typeout ((stream drei-input-editing-mixin)
-                                             (continuation function) &key erase)
-  (declare (ignore erase))
-  (let ((drei (drei-instance stream)))
-    (when (minibuffer drei)
-      (funcall continuation (minibuffer drei)))))
-
-(defmacro with-input-editor-typeout ((&optional (stream t) &rest args
-                                                &key erase)
-                                     &body body)
-  "`Stream' is not evaluated and must be a symbol. If T (the
-default), `*standard-input*' will be used. `Stream' will be bound
-to an `extended-output-stream' while `body' is being evaluated."
-  (declare (ignore erase))
-  (check-type stream symbol)
-  (let ((stream (if (eq stream t) *standard-input* stream)))
-    `(apply #'invoke-with-input-editor-typeout
-            ,stream
-            #'(lambda (,stream)
-                ,@body)
-            ,args)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
 ;;; Presentation type specialization.
 
+;;; When starting out with reading `command-or-form', we use Lisp
+;;; syntax, so things like Structedit works. If it turns out to be a
+;;; command, switch back to Fundamental.
+
+(define-presentation-method accept :around
+  ((type command-or-form)
+   (stream drei-input-editing-mixin)
+   view &key)
+  (with-drei-options ((drei-instance stream)
+                      :syntax "Lisp"
+                      :keep-syntax t)
+    (call-next-method)))
+
+(define-presentation-method accept :around
+  ((type command)
+   (stream drei-input-editing-mixin)
+   view &key)
+  (with-drei-options ((drei-instance stream)
+                      :syntax "Fundamental"
+                      :keep-syntax nil)
+    (call-next-method)))
+
 (define-presentation-method accept :around
   ((type expression)
    (stream drei-input-editing-mixin)
@@ -809,8 +786,9 @@ to an `extended-output-stream' while `body' is being evaluated."
     (with-delimiter-gestures (nil :override t)
       (loop
          named control-loop
+         with start-scan-pointer = (stream-scan-pointer stream)
          with drei = (drei-instance stream)
-         with syntax = (syntax (buffer drei))
+         with syntax = (syntax (view drei))
          ;; The input context permits the user to mouse-select displayed
          ;; Lisp objects and put them into the input buffer as literal
          ;; objects.
@@ -831,7 +809,7 @@ to an `extended-output-stream' while `body' is being evaluated."
          ;; just retrieved from the buffer during a rescan.
          for freshly-inserted = (and (plusp (stream-scan-pointer stream))
                                      (not (equal (buffer-object
-                                                  (buffer drei)
+                                                  (buffer (view drei))
                                                   (1- (stream-scan-pointer stream)))
                                                  gesture)))
          for form = (drei-lisp-syntax::form-after syntax (input-position stream))
@@ -840,8 +818,18 @@ to an `extended-output-stream' while `body' is being evaluated."
          ;; #\Newline characters in the input will not cause premature
          ;; activation.
          until (and (activation-gesture-p gesture)
-                    freshly-inserted
-                    (drei-lisp-syntax::form-complete-p form))
+                    (or (and freshly-inserted
+                             (drei-lisp-syntax::form-complete-p form))))
+         when (and (activation-gesture-p gesture)
+                   (null form))
+         do ;; We have to remove the buffer contents (whitespace,
+            ;; comments or error states, if this happens) or code
+            ;; above us will not believe us when we tell them that the
+            ;; input is empty
+           (delete-buffer-range (buffer (view drei)) start-scan-pointer
+                                (stream-scan-pointer stream))
+           (setf (stream-scan-pointer stream) start-scan-pointer)
+           (simple-parse-error "Empty input")
          ;; We only want to process the gesture if it is fresh, because
          ;; if it isn't, it has already been processed at some point in
          ;; the past.
@@ -850,19 +838,19 @@ to an `extended-output-stream' while `body' is being evaluated."
          do (with-activation-gestures (nil :override t)
               (stream-process-gesture stream gesture nil))
          finally (unread-gesture gesture :stream stream)
-           (let* ((object (handler-case
-                              (drei-lisp-syntax:form-to-object syntax form
-                                                               :read t
-                                                               :package *package*)
-                            (drei-lisp-syntax:form-conversion-error (e)
-                              ;; Move point to the problematic form
-                              ;; and signal a rescan.
-                              (setf (activation-gesture stream) nil)
-                              (handle-drei-condition drei e)
-                              (display-drei drei)
-                              (immediate-rescan stream))))
-                  (ptype (presentation-type-of object)))
-             (return-from control-loop
-               (values object
-                       (if (presentation-subtypep ptype 'expression)
-                           ptype 'expression))))))))
+         (let* ((object (handler-case
+                            (drei-lisp-syntax:form-to-object syntax form
+                             :read t
+                             :package *package*)
+                          (drei-lisp-syntax:form-conversion-error (e)
+                            ;; Move point to the problematic form
+                            ;; and signal a rescan.
+                            (setf (activation-gesture stream) nil)
+                            (handle-drei-condition drei e)
+                            (display-drei drei :redisplay-minibuffer t)
+                            (immediate-rescan stream))))
+                (ptype (presentation-type-of object)))
+           (return-from control-loop
+             (values object
+                     (if (presentation-subtypep ptype 'expression)
+                         ptype 'expression))))))))
