@@ -65,18 +65,10 @@
 ;;;
 ;;; Code interrogation/form analysis
 
-(defparameter +cl-lambda-list-keywords+
-  lambda-list-keywords)
-
-(defparameter +cl-garbage-keywords+
-  '(&whole &environment))
-
-(defun lambda-list-keyword-p (arg)
-  "Return true if `arg' is a lambda list keyword, false otherwise."
-  (member arg +cl-lambda-list-keywords+))
-
 (defgeneric parameter-match-p (parameter arg-indices &key &allow-other-keys)
   (:method ((parameter parameter) (arg-indices list) &key)
+    nil)
+  (:method :around ((parameter parameter) (arg-indices null) &key)
     nil))
 
 (defmethod parameter-match-p ((parameter named-parameter) (arg-indices list) &key)
@@ -574,6 +566,7 @@ not within a form, everything will be bound to nil. The values
 provided are, in order: the form, the forms operator, the indices
 to the operand at `offset', or the indices to an operand entered
 at that position if none is there, and the operands in the form."
+  (update-parse syntax)
   (let* ((form
           ;; Find a form with a valid (fboundp) operator.
           (let ((immediate-form
@@ -591,12 +584,12 @@ at that position if none is there, and the operands in the form."
          ;; If we cannot find a form, there's no point in looking
          ;; up any of this stuff.
          (operator (when (and form (form-list-p form))
-                     (form-to-object syntax (form-operator syntax form))))
+                     (form-to-object syntax (form-operator form))))
          (operands (when (and form (form-list-p form))
                      (mapcar #'(lambda (operand)
                                  (when operand
                                    (form-to-object syntax operand)))
-                             (form-operands syntax form))))
+                             (form-operands form))))
          (current-operand-indices (when form
                                     (find-operand-info syntax offset form))))
     (funcall continuation form operator current-operand-indices operands)))
@@ -717,7 +710,7 @@ to nil."
                 (format stream "&KEY ")
                 (show-parameters (keyword-parameters lambda-list))))))
     (format stream "(~A" operator)
-    (when (required-parameters lambda-list)
+    (when (all-parameters lambda-list)
       (princ #\Space stream))
     (show-lambda-list lambda-list)
     (format stream ")")))
@@ -875,15 +868,20 @@ function will return NIL."
                t)
               (t (replace-symbol-at-mark
                   syntax mark
-                  (or (menu-choose (mapcar
-                                    #'(lambda (completion)
-                                        (if (listp completion)
-                                            (cons completion
-                                                  (first completion))
-                                            completion))
-                                    completions)
-                                   :label "Possible completions"
-                                   :scroll-bars :vertical)
+                  (or (when (or useful-token
+                                (accept 'boolean
+                                 :prompt "You are asking for a list of all exported symbols, proceed?"))
+                        (frame-manager-menu-choose
+                         (find-frame-manager)
+                         (mapcar
+                          #'(lambda (completion)
+                              (if (listp completion)
+                                  (cons completion
+                                        (first completion))
+                                  completion))
+                          completions)
+                         :label "Possible completions"
+                         :scroll-bars :vertical))
                       longest))
                  t))))))
 
@@ -929,3 +927,275 @@ results."
              (result (apply #'format nil "~{{~:[No values~;~:*~{~S~^,~}~]}~}"
                             values)))
         (esa:display-message result)))))
+
+(defclass undefiner ()
+  ()
+  (:documentation "A base class for classes that contain logic
+for undefining Lisp constructs. Subclasses of `undefiner' must
+implement the undefiner protocol. An instance of `undefiner'
+works on a specific kind of definition (a `defun', `defclass',
+`defgeneric', etc)."))
+
+(defgeneric undefiner-type (undefiner)
+  (:documentation "Return the kind of definition undefined by
+`undefiner'. The return value is a string - a textual,
+user-oriented description."))
+
+(defgeneric definition-name (undefiner syntax definition-form)
+  (:documentation "Return the name of the definition described by
+`definition-form', as per the kind of definition `undefiner'
+handles. `Syntax' is the Lisp syntax object that has
+`definition-form'. The name returned is an actual Lisp
+object. `Form-conversion-error' is signalled if the form
+describing the name cannot be converted to an object, or if the
+form is otherwise inappropriate."))
+
+(defgeneric undefine (undefiner syntax definition-form)
+  (:documentation "Undefine whatever `definition-form' defines,
+provided `definition-form' is the kind of definition handled by
+`undefiner'. If it isn't, the results are undefined. `Syntax' is
+the Lisp syntax object that has `definition-form'."))
+
+(defvar *undefiners* (make-hash-table)
+  "A hash table mapping operators to undefiners. The undefiners
+are instances of `undefiner'.")
+
+(defun get-undefiner (definition-type)
+  "Return the undefiner for `definition-type', which must be a
+symbol. Returns NIL if there is no undefiner of the given type."
+  (values (gethash definition-type *undefiners*)))
+
+(defun invalid-form-for-type (syntax form type-name)
+  "Signal a `form-conversion-error' describing the fact that
+`form' cannot define a `type-name'."
+  (form-conversion-error syntax form "Form cannot define a ~A." type-name))
+
+(defun invalid-form (undefiner syntax form)
+  "Signal a `form-conversion-error' describing the fact that
+`form' cannot define whatever kind of definition `undefiner'
+handles."
+  (invalid-form-for-type syntax form (undefiner-type undefiner)))
+
+(defclass simple-undefiner (undefiner)
+  ((%undefiner-type :reader undefiner-type
+                    :initform (error "A description must be provided.")
+                    :type string
+                    :documentation "A textual, user-oriented name
+for the type of definition handled by this
+undefiner."
+                    :initarg :undefiner-type)
+   (%undefiner-function :reader undefiner-function
+                        :initform (error "An undefiner function must be provided.")
+                        :documentation "A function of three
+arguments: the syntax object, the name of the definition to be
+undefined and the form to be undefined."
+                        :initarg :undefiner-function)))
+
+(defmethod definition-name ((undefiner simple-undefiner) (syntax lisp-syntax) (form form))
+  (invalid-form undefiner syntax form))
+
+(defmethod definition-name ((undefiner simple-undefiner) (syntax lisp-syntax) (form list-form))
+  (if (>= (length (form-children form)) 2)
+      (form-to-object syntax (second-form (children form)))
+      (call-next-method)))
+
+(defmethod undefine ((undefiner simple-undefiner) (syntax lisp-syntax) (form form))
+  (funcall (undefiner-function undefiner) syntax
+           (definition-name undefiner syntax form)
+           form))
+
+(defmacro define-simple-undefiner (definition-spec (syntax-sym name-sym form-sym) &body body)
+  "Define a way to undefine some definition. `Definition-spec' is
+the operator (like `defun', `defclass', etc), and `syntax-sym',
+`name-sym' and `form-sym' will be bound to the Lisp syntax
+instance, the name of the definition to be undefined and the
+entire form of the definition, when the undefinition is invoked
+by the user. Syntactical problems (such as an incomplete or
+invalid `form') should be signalled via `form-conversion-error'."
+  (check-type definition-spec (or list symbol))
+  (let* ((definition-type (unlisted definition-spec))
+         (undefiner-name (if (listp definition-spec)
+                             (second definition-spec)
+                             (string-downcase definition-type))))
+    (check-type definition-type symbol)
+    `(setf (gethash ',definition-type *undefiners*)
+           (make-instance 'simple-undefiner
+            :undefiner-type ,undefiner-name
+            :undefiner-function #'(lambda (,syntax-sym ,name-sym ,form-sym)
+                                    (declare (ignorable ,syntax-sym ,name-sym ,form-sym))
+                                    ,@body)))))
+
+(defclass generic-undefiner (undefiner)
+  ((%undefiner-type :reader undefiner-type
+                    :initform (error "A description must be provided.")
+                    :type string
+                    :documentation "A textual, user-oriented name
+for the type of definition handled by this
+undefiner."
+                    :initarg :undefiner-type)
+   (%name-function :reader name-function
+                   :initform (error "A name retrieval function must be provided.")
+                   :documentation "A function of three arguments:
+the syntax object and the form to retrieve a name from. Should
+return the name as a Lisp object (probably a symbol). Should
+signal a `form-conversion-error' if the form cannot define
+whatever type this undefiner handles."
+                   :initarg :name-function)
+   (%undefiner-function :reader undefiner-function
+                        :initform (error "An undefiner function must be provided.")
+                        :documentation "A function of three
+arguments: the syntax object, the name of the definition to be
+undefined and the form to be undefined."
+                        :initarg :undefiner-function)))
+
+(defmethod definition-name ((undefiner generic-undefiner) (syntax lisp-syntax) (form form))
+  (funcall (name-function undefiner) syntax form))
+
+(defmethod undefine ((undefiner generic-undefiner) (syntax lisp-syntax) (form form))
+  (funcall (undefiner-function undefiner) syntax
+           (definition-name undefiner syntax form)
+           form))
+
+(defmacro define-undefiner (definition-spec
+                            ((name-syntax-sym name-form-sym) &body name-body)
+                            ((undef-syntax-sym undef-name-sym undef-form-sym)
+                             &body undefiner-body))
+  "Define a way to undefine definitions. `Definition-spec' is the
+operator (like `defun', `defclass', etc) and may optionally be a
+list, in which case the first element is the operator, and the
+second a user-oriented name for the kind of thing defined by the
+operator. `Name-body' and `Undefiner-body' will be evaluated to
+retrieve the name and perform the undefinition, respectively.
+
+`Name-syntax-sym' and `name-form-sym' will be bound to the Lisp
+syntax instance and the entire form of the definition during
+evaluation of `name-body'. Syntactical problems (such as an
+incomplete or invalid form) should be signalled by an
+invocation `(invalid)'
+
+`undef-syntax-sym', `undef-name-sym' and `undef-form-sym' will be
+bound to the Lisp syntax instance, the name of the definition to
+be undefined and the entire form of the definition when
+`undefiner-body' is evaluated. Syntactical problems (such as an
+incomplete or invalid form) should be signalled by an
+invocation `(invalid)'."
+  (check-type definition-spec (or list symbol))
+  (let* ((definition-type (unlisted definition-spec))
+         (undefiner-name (if (listp definition-spec)
+                             (second definition-spec)
+                             (string-downcase definition-type))))
+    (check-type definition-type symbol)
+    `(setf (gethash ',definition-type *undefiners*)
+           (make-instance 'generic-undefiner
+            :undefiner-type ,undefiner-name
+            :name-function #'(lambda (,name-syntax-sym ,name-form-sym)
+                               (declare (ignorable ,name-syntax-sym ,name-form-sym))
+                               (flet ((invalid ()
+                                        (invalid-form-for-type ,name-syntax-sym ,name-form-sym ,undefiner-name)))
+                                 (declare (ignorable #'invalid))
+                                 ,@name-body))
+            :undefiner-function #'(lambda (,undef-syntax-sym ,undef-name-sym ,undef-form-sym)
+                                    (declare (ignorable ,undef-syntax-sym ,undef-name-sym ,undef-form-sym))
+                                    (flet ((invalid ()
+                                             (invalid-form-for-type ,undef-syntax-sym ,undef-form-sym ,undef-name-sym)))
+                                      (declare (ignorable #'invalid))
+                                      ,@undefiner-body))))))
+
+(define-simple-undefiner (defun "function") (syntax name form)
+  (fmakunbound name))
+
+(define-simple-undefiner (defgeneric "generic function") (syntax name form)
+  (fmakunbound name))
+
+(define-simple-undefiner (defmacro "macro") (syntax name form)
+  (fmakunbound name))
+
+(define-simple-undefiner (cl:defclass "class") (syntax name form)
+  (setf (find-class name nil) nil))
+
+(define-simple-undefiner (clim-lisp:defclass "class") (syntax name form)
+  (setf (find-class name nil) nil))
+
+(define-simple-undefiner (defmethod "method") (syntax name form)
+  (let ((function (fdefinition name)))
+    (labels ((get-qualifiers (maybe-qualifiers)
+               (unless (or (null maybe-qualifiers)
+                           (form-list-p (first maybe-qualifiers)))
+                 (cons (form-to-object syntax (first maybe-qualifiers))
+                       (get-qualifiers (rest maybe-qualifiers)))))
+             (get-specializers (maybe-specializers)
+               (cond ((null maybe-specializers)
+                      (form-conversion-error syntax form "~A form invalid." 'defmethod))
+                     ;; Map across the elements in the lambda list.
+                     ((form-list-p (first maybe-specializers))
+                      (mapcar #'(lambda (ll-form)
+                                  (if (and (form-list-p ll-form)
+                                           (second-form (children ll-form)))
+                                      (form-to-object syntax (second-form (children ll-form)))
+                                      t))
+                              (form-children (first maybe-specializers))))
+                     ;; Skip the qualifiers to get the lambda-list.
+                     (t (get-specializers (rest maybe-specializers))))))
+      (remove-method function (find-method function
+                                           (get-qualifiers (cddr (form-children form)))
+                                           (get-specializers (cddr (form-children form)))
+                                           nil)))))
+
+(define-simple-undefiner (defvar "special variable") (syntax name form)
+  (makunbound name))
+
+(define-simple-undefiner (defparameter "special variable") (syntax name form)
+  (makunbound name))
+
+(define-simple-undefiner (defconstant "constant") (syntax name form)
+  (makunbound name))
+
+(define-simple-undefiner (defpackage "package") (syntax name form)
+  (delete-package name))
+
+(defun get-listed-name (syntax form)
+  "Retrieve the name of `form' under the assumption that the name
+is the second element of `form', and if this is a list, the first
+element of that list. The secondary value will be true if a name
+can be found, false otherwise."
+  (if (and (form-list-p form)
+           (>= (length (form-children form)) 2))
+      (let ((name-form (second (form-children form))))
+        (cond ((and (form-list-p name-form)
+                    (form-children name-form))
+               (values (form-to-object syntax (first (form-children name-form))) t))
+              ((form-token-p name-form)
+               (values (form-to-object syntax name-form) t))
+              (t (values nil nil))))
+      (values nil nil)))
+
+;; Cannot recognize the common define-FOO-command macros.
+(define-undefiner (define-command "command")
+  ((syntax form)
+   (multiple-value-bind (name success) (get-listed-name syntax form)
+     (if success name (invalid))))
+  ((syntax name form)
+   ;; Pick out the command table from the define-command form. The
+   ;; command may also be in other command tables, but we can't find
+   ;; those.
+   (let ((name-form (listed (form-to-object syntax (second (form-children form))))))
+     (destructuring-bind (ignore &key command-table keystroke &allow-other-keys) name-form
+       (declare (ignore ignore))
+       (when command-table
+         (remove-command-from-command-table name command-table :errorp nil)
+         (remove-keystroke-from-command-table command-table keystroke :errorp nil))))
+   (fmakunbound name)))
+
+(define-undefiner (define-undefiner "undefiner")
+  ((syntax form)
+   (multiple-value-bind (name success) (get-listed-name syntax form)
+     (if success name (invalid))))
+  ((syntax name form)
+   (remhash name *undefiners*)))
+
+(define-undefiner (define-simple-undefiner "simple undefiner")
+  ((syntax form)
+   (multiple-value-bind (name success) (get-listed-name syntax form)
+     (if success name (invalid))))
+  ((syntax name form)
+   (remhash name *undefiners*)))
