@@ -267,24 +267,24 @@ being called until either `motor' succeeds or `fiddler' fails."
 Return T if successful, or NIL if the buffer limit was reached."))
 
 (defmethod forward-one-word (mark syntax)
-  (forward-to-word-boundary mark syntax)
-  (and (not (end-of-buffer-p mark))
-       (loop until (end-of-buffer-p mark)
-          while (word-constituentp syntax (object-after mark))
-          do (forward-object mark)
-          finally (return t))))
+  (unless (end-of-buffer-p mark)
+    (forward-to-word-boundary mark syntax)
+    (loop until (end-of-buffer-p mark)
+       while (word-constituentp syntax (object-after mark))
+       do (forward-object mark)
+       finally (return t))))
 
 (defgeneric backward-one-word (mark syntax)
   (:documentation "Move MARK backward over the previous word.
 Return T if successful, or NIL if the buffer limit was reached."))
 
 (defmethod backward-one-word (mark syntax)
-  (backward-to-word-boundary mark syntax)
-  (and (not (beginning-of-buffer-p mark))
-       (loop until (beginning-of-buffer-p mark)
-          while (word-constituentp syntax (object-before mark))
-          do (backward-object mark)
-          finally (return t))))
+  (unless (beginning-of-buffer-p mark)
+    (backward-to-word-boundary mark syntax)
+    (loop until (beginning-of-buffer-p mark)
+       while (word-constituentp syntax (object-before mark))
+       do (backward-object mark)
+       finally (return t))))
 
 (define-motion-fns word)
 
@@ -476,10 +476,9 @@ Return T if successful, or NIL if the buffer limit was reached."))
   (unless (beginning-of-buffer-p mark)
     (backward-object mark 1)
     (if (search-backward mark (coerce (paragraph-delimiter syntax) 'vector))
-        (progn (forward-object mark (length (paragraph-delimiter syntax)))
-               t)
-        (progn (beginning-of-buffer mark)
-               nil))))
+        (forward-object mark (length (paragraph-delimiter syntax)))
+        (beginning-of-buffer mark))
+    t))
 
 (defgeneric forward-one-paragraph (mark syntax)
   (:documentation
@@ -490,10 +489,9 @@ Return T if successful, or NIL if the buffer limit was reached."))
   (unless (end-of-buffer-p mark)
     (forward-object mark 1)
     (if (search-forward mark (coerce (paragraph-delimiter syntax) 'vector))
-        (progn (backward-object mark (length (paragraph-delimiter syntax)))
-               t)
-        (progn (end-of-buffer mark)
-               nil))))
+        (backward-object mark (length (paragraph-delimiter syntax)))
+        (end-of-buffer mark))
+    t))
 
 (define-motion-fns paragraph)
 
