@@ -27,11 +27,13 @@
 (in-package :cl-user)
 
 (defpackage :drei-buffer
-  (:use :clim-lisp :flexichain :binseq)
+  (:use :clim-lisp :flexichain :binseq :esa-utils)
+  ;; Kludge to remove symbol conflicts.
+  (:import-from :esa-io :buffer)
   (:export #:buffer #:standard-buffer
            #:mark #:left-sticky-mark #:right-sticky-mark
            #:standard-left-sticky-mark #:standard-right-sticky-mark
-           #:clone-mark
+           #:make-buffer-mark #:clone-mark
            #:condition-offset
            #:no-such-offset #:offset-before-beginning #:offset-after-end
            #:invalid-motion #:motion-before-beginning #:motion-after-end
@@ -54,12 +56,13 @@
            #:delete-region
            #:buffer-object #:buffer-sequence
            #:object-before #:object-after #:region-to-sequence #:region-to-string
-           #:low-mark #:high-mark #:modified-p #:clear-modify
+           #:observable-buffer-mixin
            #:binseq-buffer #:obinseq-buffer #:binseq2-buffer
            #:persistent-left-sticky-mark #:persistent-right-sticky-mark
            #:persistent-left-sticky-line-mark #:persistent-right-sticky-line-mark
            #:p-line-mark-mixin #:buffer-line-offset
-           #:delegating-buffer #:implementation)
+           #:delegating-buffer #:implementation
+           #:delegating-left-sticky-mark #:delegating-right-sticky-mark)
   (:documentation "An implementation of the buffer protocol. This
 package is quite low-level, not syntax-aware, not CLIM-aware and
 not user-oriented at all."))
@@ -103,9 +106,16 @@ not user-oriented at all."))
            #:constituentp
            #:just-n-spaces
            #:move-to-column
+           #:kill-region
+           #:in-place-buffer-substring
+           #:fill-string-from-buffer
+           #:buffer-find-nonchar
+           #:offset-beginning-of-line-p
+           #:offset-end-of-line-p
+           #:end-of-line-offset
+           #:extract-region
            #:buffer-whitespacep
            #:buffer-region-case
-           #:name-mixin #:name
            #:buffer-looking-at #:looking-at
            #:buffer-search-forward #:buffer-search-backward
            #:buffer-re-search-forward #:buffer-re-search-backward
@@ -137,14 +147,15 @@ characters."))
 
 (defpackage :drei-syntax
   (:use :clim-lisp :clim :drei-buffer :drei-base :flexichain :esa-utils)
-  (:export #:syntax #:syntaxp #:define-syntax #:*default-syntax* #:cursor-positions
-           #:syntax-command-table #:use-editor-commands-p #:additional-command-tables #:define-syntax-command-table
+  (:export #:syntax #:syntax-command-tables #:updater-fns #:update-parse #:syntaxp
+           #:define-syntax #:*default-syntax*
+           #:syntax-command-table #:additional-command-tables #:define-syntax-command-table
            #:eval-option
            #:define-option-for-syntax
            #:current-attributes-for-syntax
            #:make-attribute-line
            #:syntax-from-name
-           #:update-syntax #:update-syntax-for-display
+           #:update-syntax
            #:grammar #:grammar-rule #:add-rule
            #:parser #:initial-state
            #:advance-parse
@@ -156,13 +167,11 @@ characters."))
            #:parse-stack-top #:target-parse-tree #:parse-state-empty-p
            #:parse-stack-next #:parse-stack-symbol
            #:parse-stack-parse-trees #:map-over-parse-trees
-           #:no-such-operation #:no-expression
+           #:no-such-operation
            #:name-for-info-pane
            #:display-syntax-name
            #:syntax-line-indentation
            #:eval-defun
-           #:record-line-vertical-offset
-           #:line-vertical-offset
            #:syntax-line-comment-string
            #:line-comment-region #:comment-region
            #:line-uncomment-region #:uncomment-region
@@ -175,7 +184,7 @@ characters."))
   contents."))
 
 (defpackage :drei
-  (:use :clim-lisp :clim :drei-buffer :drei-base :drei-abbrev
+  (:use :clim-lisp :clim-sys :clim :drei-buffer :drei-base :drei-abbrev
         :drei-syntax :flexichain :drei-undo :esa-buffer :esa-io :esa
         :esa-utils :drei-kill-ring)
   (:export #:drei-buffer #:needs-saving
@@ -185,10 +194,9 @@ characters."))
            #:display-drei #:display-drei-pane #:display-drei-area #:full-redisplay
            #:offset-to-screen-position
            #:page-down #:page-up
-           #:indent-tabs-mode
            #:isearch-state #:search-string #:search-mark
            #:search-forward-p #:search-success-p
-           #:query-replace-state #:string1 #:string2 #:buffers #:mark #:occurrences
+           #:query-replace-state #:string1 #:string2 #:targets #:occurrences
 
            ;; Undo.
            #:undo-mixin #:undo-tree #:undo-accumulate #:performing-undo
@@ -197,30 +205,103 @@ characters."))
            #:insert-record
            #:delete-record
            #:compound-record
+           #:with-undo #:clear-undo-history
            
-           #:with-undo
            #:drei-buffer
-           #:drei-textual-view #:+drei-textual-view+
 
            ;; Signals and conditions.
            #:user-condition-mixin
            #:buffer-read-only
            #:buffer-single-line
+           #:no-available-minibuffer
 
+           ;; Views and their facilities.
+           #:drei-view #:modified-p #:no-cursors
+           
+           #:drei-buffer-view #:buffer #:top #:bot #:buffer-view-p
+           #:lines
+           #:buffer-line #:start-mark #:end-mark #:line-length #:chunks
+           #:line-containing-offset #:offset-in-line-p
+           #:buffer-view-pump-state-for-offset
+           #:buffer-view-stroke-pump
+           
+           #:drei-syntax-view #:syntax #:syntax-view-p
+           #:pump-state-for-offset-with-syntax
+           #:stroke-pump-with-syntax
+           
+           #:point-mark-view #:point-mark-view-p
+           
+           #:textual-drei-syntax-view
+           #:tab-space-count #:space-width #:tab-width #:use-tabs
+           #:auto-fill-mode #:auto-fill-column
+           #:isearch-mode #:isearch-states #:isearch-previous-string
+           #:query-replace-mode
+           #:region-visible-p
+           #:dabbrev-expansion-mark
+           #:original-prefix
+           #:prefix-start-offset
+           #:overwrite-mode
+           #:goal-column
+
+           #:invalidate-strokes
+
+           #:view-command-tables
+           #:use-editor-commands-p
+           #:synchronize-view
+           #:create-view-cursors
+           #:clear-redisplay-information
+           #:clone-view
+           #:make-syntax-for-view
+           
            ;; DREI command tables.
            #:comment-table #:deletion-table #:editing-table
            #:fill-table #:indent-table #:marking-table #:case-table
            #:movement-table #:search-table #:info-table #:self-insert-table
+           #:view-table
            #:editor-table #:exclusive-gadget-table #:exclusive-input-editor-table
 
-           ;; DREI interface stuff.
-           #:drei #:drei-pane #:drei-gadget-pane #:drei-area
+           #:minibuffer
+
+           #:drei #:editor-pane
+           #:drei-pane #:drei-gadget-pane #:drei-area
            #:handling-drei-conditions #:handle-drei-condition
-           #:execute-drei-command #:display-drei-contents #:display-drei-cursor
+           #:execute-drei-command
+
+           ;; Redisplay engine.
+           #:display-drei-view-contents #:display-drei-view-cursor
+           #:handle-redisplay
+           #:face #:make-face #:face-ink #:face-style
+           #:drawing-options #:make-drawing-options
+           #:drawing-options-face #:drawing-options-function
+           #:drawing-options-equal #:+default-drawing-options+
+           #:stroke-start-offset #:stroke-end-offset
+           #:stroke-drawing-options
+           
+           #:pump-state-for-offset #:stroke-pump
+           #:object-drawer #:*maximum-chunk-size*
+
+           #:+roman-face+ #:+roman-face-drawing-options+
+           #:+italic-face+ #:+italic-face-drawing-options+
+           #:+bold-face+ #:+bold-face-drawing-options+
+           #:+bold-italic-face+ #:+bold-italic-drawing-options+
+
+           #:*keyword-drawing-options*
+           #:*special-operator-drawing-options*
+           #:*special-variable-drawing-options*
+           #:*string-drawing-options*
+           #:*comment-drawing-options*
+           #:*error-drawing-options*
+
+           #:*highlight-strokes*
+           #:*stroke-boundary-ink*
+           #:*stroke-baseline-ink*
+
+           ;; DREI program interface stuff.
            #:with-drei-options
            #:performing-drei-operations #:invoke-performing-drei-operations
            #:with-bound-drei-special-variables
            #:accepting-from-user #:invoke-accepting-from-user
+           #:require-minibuffer
 
            ;; Gadget interface stuff.
            #:handle-gesture
@@ -234,38 +315,26 @@ characters."))
            #:mark-cursor #:active #:mark
            #:active-ink #:inactive-ink #:ink
 
-           ;; Accessors of DREI instances.
-           #:buffer
-           #:point
-           #:top #:bot
-           #:tab-space-count #:space-width #:tab-width
-           #:auto-fill-mode #:auto-fill-column
-           #:isearch-mode #:isearch-states #:isearch-previous-string
-           #:query-replace-mode
-           #:region-visible-p
-           #:minibuffer
            
-           #:dabbrev-expansion-mark
-           #:original-prefix
-           #:prefix-start-offset
-           #:overwrite-mode
-           #:goal-column
            #:in-focus-p
 
            ;; Info functions.
-           #:current-point
-           #:current-mark
-
-           ;; Info variables.
-           #:*current-point*
-           #:*current-mark*
-           #:*current-syntax*
+           #:point #:point-of
+           #:mark #:mark-of
+           #:current-syntax
+           #:current-view
+           #:drei-instance #:drei-instance-of
 
            ;; Configuration.
            #:*foreground-color*
            #:*background-color*
            #:*show-mark*
-           #:*use-tabs-for-indentation*))
+           #:*use-tabs-for-indentation*
+
+           #:view-mode #:syntax-mode
+           #:applicable-modes
+           #:define-mode #:define-view-mode #:define-syntax-mode
+           #:define-mode-toggle-commands))
 
 (defpackage :drei-motion
   (:use :clim-lisp :drei-base :drei-buffer :drei-syntax)
@@ -404,7 +473,8 @@ used to implement the editing commands."))
         :drei-syntax :drei-motion :drei :drei-kill-ring
         :drei-editing :clim :drei-abbrev :esa :esa-buffer :esa-io
         :esa-utils :drei-undo)
-  (:export #:goto-position
+  (:export #:proper-line-indentation
+           #:goto-position
            #:goto-line
            #:replace-one-string
            #:possibly-fill-line
@@ -413,6 +483,7 @@ used to implement the editing commands."))
            #:delete-horizontal-space
            #:indent-current-line
            #:insert-pair
+           #:move-past-close-and-reindent
            #:downcase-word #:upcase-word #:capitalize-word
            #:indent-region
            #:fill-line #:fill-region
@@ -431,45 +502,104 @@ used to implement the editing commands."))
            #:delete-rectangle-line-whitespace
            #:with-narrowed-buffer
 
-           #:start-mark
-           #:end-mark
-           #:make-buffer-stream)
+           #:start-mark #:end-mark
+           #:make-buffer-stream
+
+           #:target-specification
+           #:activate-target-specification
+           #:deactivate-target-specification
+           #:subsequent-targets-p #:preceding-targets-p
+           #:next-target #:previous-target
+           #:previous-target
+           #:no-more-targets
+           #:*default-target-creator*
+           #:view-list-target-specification #:views)
   (:documentation "Implementation of much syntax-aware, yet no
 syntax-specific, core functionality of Drei."))
 
 (defpackage :drei-fundamental-syntax
   (:use :clim-lisp :clim :drei-buffer :drei-base 
-        :drei-syntax :flexichain :drei)
-  (:export #:fundamental-syntax #:scan)
+        :drei-syntax :flexichain :drei :drei-core :esa-utils)
+  (:export #:fundamental-syntax)
   (:documentation "Implementation of the basic syntax module for
 editing plain text."))
 
 (defpackage :drei-lr-syntax
   (:use :clim-lisp :clim :clim-extensions :drei-buffer :drei-base
-	:drei-syntax :drei-fundamental-syntax)
+        :drei-syntax :drei :drei-core :drei-fundamental-syntax
+        :esa-utils)
   (:export #:lr-syntax-mixin #:stack-top #:initial-state
 	   #:skip-inter #:lex #:define-lexer-state
 	   #:lexer-toplevel-state #:lexer-error-state
 	   #:parser-symbol #:parent #:children
 	   #:start-offset #:end-offset #:parser-state
 	   #:preceding-parse-tree
+           #:literal-object-mixin
 	   #:define-parser-state
 	   #:lexeme #:nonterminal
 	   #:action #:new-state #:done
 	   #:reduce-fixed-number #:reduce-until-type #:reduce-all 
-	   #:error-state #:error-reduce-state)
+	   #:error-state #:error-reduce-state
+           #:do-parse-symbols-forward
+           #:parser-symbol-containing-offset
+           #:define-syntax-highlighting-rules
+           #:syntax-highlighting-rules)
   (:documentation "Underlying LR parsing functionality."))
 
 (defpackage :drei-lisp-syntax
   (:use :clim-lisp :clim :clim-extensions :drei-buffer :drei-base 
         :drei-syntax :drei-fundamental-syntax :flexichain :drei
         :drei-motion :drei-editing :esa-utils :esa :drei-core :esa-io
-	:drei-lr-syntax)
-  (:export #:lisp-syntax
+	:drei-lr-syntax :drei-kill-ring)
+  (:export #:lisp-syntax #:lisp-table
            #:lisp-string
            #:edit-definition
            #:form
-           #:form-to-object
+           #:form-to-object #:form-equal
+
+           ;; Selecting forms based on mark
+           #:form-around #:form-before #:form-after
+           #:find-list-parent
+           #:expression-at-mark
+           #:definition-at-mark
+           #:list-at-mark
+           #:symbol-at-mark
+           #:fully-quoted-form
+           #:fully-unquoted-form
+           #:this-form
+
+           ;; Querying forms
+           #:formp #:form-list-p
+           #:form-incomplete-p #:form-complete-p
+           #:form-token-p #:form-string-p
+           #:form-quoted-p
+           #:form-comma-p #:form-comma-at-p #:form-comma-dot-p
+           #:form-character-p
+           #:form-simple-vector-p
+           #:comment-p #:line-comment-p #:long-comment-p
+           #:form-at-top-level-p
+
+           ;; Querying form data
+           #:form-children
+           #:form-operator #:form-operands
+           #:form-toplevel
+           #:form-operator-p
+
+           ;; Querying about state at mark
+           #:in-string-p
+           #:in-comment-p
+           #:in-line-comment-p
+           #:in-long-comment-p
+           #:in-character-p
+           #:location-at-beginning-of-form
+           #:location-at-end-of-form
+           #:at-beginning-of-list-p
+           #:at-end-of-list-p
+           #:at-beginning-of-string-p
+           #:at-end-of-string-p
+           #:at-beginning-of-children-p
+           #:at-end-of-children-p
+           #:comment-at-mark
 
            ;; Lambda list classes.
            #:lambda-list
@@ -501,7 +631,7 @@ editing plain text."))
            #:destructuring-parameter
            #:required-parameter #:destructuring-required-parameter #:named-required-parameter
            #:optional-parameter #:destructuring-optional-parameter #:named-optional-parameter
-           #:keyword-parameter #:destructuring-keyword-parameter #:named-keyword-parameter
+           #:keyword-parameter #:destructuring-keyword-parameter
            #:rest-parameter
            #:body-parameter
 
@@ -515,7 +645,12 @@ editing plain text."))
 
            ;; Conditions.
            #:form-conversion-error
-           #:invalid-arglist)
+           #:invalid-lambda-list
+
+           ;; Configuration
+           #:*syntax-highlighting-rules*
+           #:emacs-style-highlighting
+           #:retro-highlighting)
   (:shadow clim:form)
   (:documentation "Implementation of the syntax module used for
 editing Common Lisp code."))
