@@ -8,6 +8,8 @@
 ;;;           Matthieu Villeneuve (matthieu.villeneuve@free.fr)
 ;;;  (c) copyright 2005 by
 ;;;           Aleksandar Bakic (a_bakic@yahoo.com)
+;;;  (c) copyright 2007 by
+;;;           Troels Henriksen (athas@sigkill.dk)
 
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Library General Public
@@ -28,6 +30,29 @@
 
 (in-package :drei-commands)
 
+(defun simple-search (drei-instance search-function
+                      targets more-targets-predicate more-targets-fn)
+  (let ((old-buffer (buffer drei-instance))
+        (old-offset (offset (point drei-instance))))
+    (activate-target-specification targets)
+    (or (loop until (funcall search-function (point drei-instance))
+           if (funcall more-targets-predicate targets)
+           do (funcall more-targets-fn targets)
+           else return nil
+           finally (return t))
+        (setf (buffer drei-instance) old-buffer
+              (offset (point drei-instance)) old-offset))))
+
+(defun simple-search-forward (drei-instance search-function &optional
+                             (targets (funcall *default-target-creator* drei-instance)))
+  (simple-search drei-instance search-function targets
+                 #'subsequent-targets-p #'next-target))
+
+(defun simple-search-backward (drei-instance search-function &optional
+                               (targets (funcall *default-target-creator* drei-instance)))
+  (simple-search drei-instance search-function targets
+                 #'preceding-targets-p #'previous-target))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
 ;;; String search
@@ -36,13 +61,19 @@
     ((string 'string :prompt "String Search"))
   "Prompt for a string and search forward for it.
 If found, leaves point after string. If not, leaves point where it is."
-  (search-forward *current-point* string :test (case-relevant-test string)))
+  (simple-search-forward (drei-instance)
+                        #'(lambda (mark)
+                            (search-forward mark string
+                             :test (case-relevant-test string)))))
 
 (define-command (com-reverse-string-search :name t :command-table search-table)
     ((string 'string :prompt "Reverse String Search"))
   "Prompt for a string and search backward for it.
 If found, leaves point before string. If not, leaves point where it is."
-  (search-backward *current-point* string :test (case-relevant-test string)))
+  (simple-search-backward (drei-instance)
+                        #'(lambda (mark)
+                            (search-backward mark string
+                             :test (case-relevant-test string)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
@@ -52,13 +83,17 @@ If found, leaves point before string. If not, leaves point where it is."
     ((word 'string :prompt "Search word"))
   "Prompt for a whitespace delimited word and search forward for it.
 If found, leaves point after the word. If not, leaves point where it is."
-  (search-word-forward *current-point* word))
+  (simple-search-forward (drei-instance)
+                        #'(lambda (mark)
+                            (search-word-forward mark word))))
 
 (define-command (com-reverse-word-search :name t :command-table search-table)
     ((word 'string :prompt "Search word"))
   "Prompt for a whitespace delimited word and search backward for it.
 If found, leaves point before the word. If not, leaves point where it is."
-  (search-word-backward *current-point* word))
+  (simple-search-backward (drei-instance)
+                        #'(lambda (mark)
+                            (search-backward mark word))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
@@ -66,55 +101,72 @@ If found, leaves point before the word. If not, leaves point where it is."
 
 (make-command-table 'isearch-drei-table :errorp nil)
 
-(defun isearch-command-loop (pane forwardp)
-  (let* ((point (point pane))
-         (orig-offset (offset point)))
-    (unless (endp (isearch-states pane))
-      (setf (isearch-previous-string pane)
-            (search-string (first (isearch-states pane)))))
-    (setf (isearch-mode pane) t)
-    (setf (isearch-states pane)
+(defun isearch-command-loop (drei-instance forwardp)
+  (let ((point (point (view drei-instance))))
+    (unless (endp (isearch-states drei-instance))
+      (setf (isearch-previous-string drei-instance)
+            (search-string (first (isearch-states drei-instance)))))
+    (setf (isearch-mode drei-instance) t)
+    (setf (isearch-states drei-instance)
           (list (make-instance 'isearch-state
-                               :search-string ""
-                               :search-mark (clone-mark point)
-                               :search-forward-p forwardp
-                               :search-success-p t)))
+                 :search-string ""
+                 :search-mark (clone-mark point)
+                 :search-forward-p forwardp
+                 :search-success-p t
+                 :targets (funcall *default-target-creator* drei-instance))))
+    (activate-target-specification (targets (first (isearch-states drei-instance))))
     (simple-command-loop 'isearch-drei-table
-                         (isearch-mode pane)
+                         (isearch-mode drei-instance)
                          ((display-message "Mark saved where search started")
-                          (setf (offset (mark pane)) orig-offset)
-                          (setf (isearch-mode pane) nil))
+                          (deactivate-target-specification (targets (first (isearch-states drei-instance))))
+                          (setf (isearch-mode drei-instance) nil))
                          ((display-message "Returned point to original location")
-                          (setf (offset (point pane)) orig-offset)
-                          (setf (isearch-mode pane) nil)
+                          (deactivate-target-specification (targets (first (isearch-states drei-instance))))
+                          (setf (isearch-mode drei-instance) nil)
                           (signal 'abort-gesture :event *current-gesture*)))))
 
-(defun isearch-from-mark (pane mark string forwardp)
-  (let* ((point (point pane))
-	 (mark2 (clone-mark mark))
+(defun isearch-from-mark (drei-instance mark string forwardp)
+  (let* ((mark2 (clone-mark mark))
 	 (success (funcall (if forwardp #'search-forward #'search-backward)
 			   mark2
 			   string
-			   :test (case-relevant-test string))))
-    (when success
-      (setf (offset point) (offset mark2)
-	    (offset mark) (if forwardp
-			      (- (offset mark2) (length string))
-			      (+ (offset mark2) (length string)))))
+			   :test (case-relevant-test string)))
+         (state (first (isearch-states drei-instance))))
+    (if success
+        (setf (offset (point)) (offset mark2)
+              (offset mark) (if forwardp
+                                (- (offset mark2) (length string))
+                                (+ (offset mark2) (length string))))
+        (when (funcall (if forwardp
+                           #'subsequent-targets-p
+                           #'preceding-targets-p)
+                       (targets state))
+          (funcall (if forwardp #'next-target #'previous-target)
+                   (targets state))
+          (if (isearch-from-mark drei-instance (clone-mark (point))
+                                 string forwardp)
+              (return-from isearch-from-mark t)
+              (progn (pop (isearch-states drei-instance))
+                     (funcall (if forwardp #'previous-target #'next-target)
+                              (targets state))
+                     (setf (offset (point)) (offset (search-mark state)))
+                     nil))))
     (display-message "~:[Failing ~;~]Isearch~:[ backward~;~]: ~A"
 		     success forwardp (display-string string))
     (push (make-instance 'isearch-state
-	     :search-string string
-	     :search-mark mark
-	     :search-forward-p forwardp
-	     :search-success-p success)
-	  (isearch-states pane))
+           :search-string string
+           :search-mark mark
+           :search-forward-p forwardp
+           :search-success-p success
+           :targets (targets state))
+	  (isearch-states drei-instance))
     (unless success
-      (beep))))
+      (beep))
+    success))
 
 (define-command (com-isearch-forward :name t :command-table search-table) ()
   (display-message "Isearch: ")
-  (isearch-command-loop *current-window* t))
+  (isearch-command-loop (drei-instance) t))
 
 (set-key 'com-isearch-forward
 	 'search-table
@@ -122,14 +174,14 @@ If found, leaves point before the word. If not, leaves point where it is."
 
 (define-command (com-isearch-backward :name t :command-table search-table) ()
   (display-message "Isearch backward: ")
-  (isearch-command-loop *current-window* nil))
+  (isearch-command-loop (drei-instance) nil))
 
 (set-key 'com-isearch-backward
 	 'search-table
 	 '((#\r :control)))
 
 (defun isearch-append-char (char)
-  (let* ((states (isearch-states *current-window*))
+  (let* ((states (isearch-states (drei-instance)))
          (string (concatenate 'string
                               (search-string (first states))
                               (string char)))
@@ -137,7 +189,7 @@ If found, leaves point before the word. If not, leaves point where it is."
          (forwardp (search-forward-p (first states))))
     (unless (or forwardp (end-of-buffer-p mark))
       (incf (offset mark)))
-    (isearch-from-mark *current-window* mark string forwardp)))
+    (isearch-from-mark (drei-instance) mark string forwardp)))
 
 (define-command (com-isearch-append-char :name t :command-table isearch-drei-table) ()
   (isearch-append-char *current-gesture*))
@@ -146,30 +198,30 @@ If found, leaves point before the word. If not, leaves point where it is."
   (isearch-append-char #\Newline))
 
 (defun isearch-append-text (movement-function)
-  (let* ((states (isearch-states *current-window*))
-	 (start (clone-mark *current-point*))
+  (let* ((states (isearch-states (drei-instance)))
+	 (start (clone-mark (point)))
 	 (mark (clone-mark (search-mark (first states))))
 	 (forwardp (search-forward-p (first states))))
-    (funcall movement-function *current-point*)
+    (funcall movement-function (point))
     (let* ((start-offset (offset start))
-	   (point-offset (offset *current-point*))
+	   (point-offset (offset (point)))
 	   (string (concatenate 'string
 				(search-string (first states))
-				(buffer-substring *current-buffer*
+				(buffer-substring (current-buffer)
 						  start-offset
 						  point-offset))))
       (unless (or forwardp (end-of-buffer-p mark))
 	(incf (offset mark) (- point-offset start-offset)))
-      (isearch-from-mark *current-window* mark string forwardp))))
+      (isearch-from-mark (drei-instance) mark string forwardp))))
 
 (define-command (com-isearch-append-word :name t :command-table isearch-drei-table) ()
-  (isearch-append-text #'(lambda (mark) (forward-word mark *current-syntax*))))
+  (isearch-append-text #'(lambda (mark) (forward-word mark (current-syntax)))))
 
 (define-command (com-isearch-append-line :name t :command-table isearch-drei-table) ()
   (isearch-append-text #'end-of-line))
 
 (define-command (com-isearch-append-kill :name t :command-table isearch-drei-table) ()
-  (let* ((states (isearch-states *current-window*))
+  (let* ((states (isearch-states (drei-instance)))
 	 (yank (handler-case (kill-ring-yank *kill-ring*)
                  (empty-kill-ring ()
                    "")))
@@ -180,50 +232,49 @@ If found, leaves point before the word. If not, leaves point where it is."
 	 (forwardp (search-forward-p (first states))))
     (unless (or forwardp (end-of-buffer-p mark))
       (incf (offset mark) (length yank)))
-    (isearch-from-mark *current-window* mark string forwardp)))
+    (isearch-from-mark (drei-instance) mark string forwardp)))
 
 (define-command (com-isearch-delete-char :name t :command-table isearch-drei-table) ()
-  (let* ((pane *current-window*))
-    (cond ((null (second (isearch-states pane)))
-	   (display-message "Isearch: ")
-           (beep))
-          (t
-           (pop (isearch-states pane))
-           (loop until (endp (rest (isearch-states pane)))
-                 until (search-success-p (first (isearch-states pane)))
-                 do (pop (isearch-states pane)))
-           (let ((state (first (isearch-states pane))))
-             (setf (offset (point pane))
-                   (if (search-forward-p state)
-                       (+ (offset (search-mark state))
-                          (length (search-string state)))
-                       (- (offset (search-mark state))
-                          (length (search-string state)))))
-	     (display-message "Isearch~:[ backward~;~]: ~A"
-			      (search-forward-p state)
-			      (display-string (search-string state))))))))
+  (cond ((null (second (isearch-states (drei-instance))))
+         (display-message "Isearch: ")
+         (beep))
+        (t
+         (pop (isearch-states (drei-instance)))
+         (loop until (endp (rest (isearch-states (drei-instance))))
+            until (search-success-p (first (isearch-states (drei-instance))))
+            do (pop (isearch-states (drei-instance))))
+         (let ((state (first (isearch-states (drei-instance)))))
+           (setf (offset (point (drei-instance)))
+                 (if (search-forward-p state)
+                     (+ (offset (search-mark state))
+                        (length (search-string state)))
+                     (- (offset (search-mark state))
+                        (length (search-string state)))))
+           (display-message "Isearch~:[ backward~;~]: ~A"
+                            (search-forward-p state)
+                            (display-string (search-string state)))))))
 
 (define-command (com-isearch-search-forward :name t :command-table isearch-drei-table) ()
-  (let* ((states (isearch-states *current-window*))
+  (let* ((states (isearch-states (drei-instance)))
          (string (if (null (second states))
-                     (isearch-previous-string *current-window*)
+                     (isearch-previous-string (drei-instance))
                      (search-string (first states))))
-         (mark (clone-mark *current-point*)))
-    (isearch-from-mark *current-window* mark string t)))
+         (mark (clone-mark (point))))
+    (isearch-from-mark (drei-instance) mark string t)))
 
 (define-command (com-isearch-search-backward :name t :command-table isearch-drei-table) ()
-  (let* ((states (isearch-states *current-window*))
+  (let* ((states (isearch-states (drei-instance)))
          (string (if (null (second states))
-                     (isearch-previous-string *current-window*)
+                     (isearch-previous-string (drei-instance))
                      (search-string (first states))))
-         (mark (clone-mark *current-point*)))
-    (isearch-from-mark *current-window* mark string nil)))
+         (mark (clone-mark (point))))
+    (isearch-from-mark (drei-instance) mark string nil)))
 
 (define-command (com-isearch-exit :name t :command-table isearch-drei-table) ()
-  (let* ((states (isearch-states *current-window*))
+  (let* ((states (isearch-states (drei-instance)))
 	 (string (search-string (first states)))
 	 (search-forward-p (search-forward-p (first states))))
-    (setf (isearch-mode *current-window*) nil)
+    (setf (isearch-mode (drei-instance)) nil)
     (when (string= string "")
       (execute-frame-command *application-frame*
 			     (funcall
@@ -258,10 +309,11 @@ If found, leaves point before the word. If not, leaves point where it is."
 (define-command (com-replace-string :name t :command-table search-table)
     ()
   "Replace all occurrences of `string' with `newstring'."
+  (require-minibuffer)
   ;; We have to do it this way if we want to refer to STRING in NEWSTRING
   (let* ((string (accept 'string :prompt "Replace String"))
 	 (newstring (accept'string :prompt (format nil "Replace ~A with" string))))
-    (loop with point = *current-point*
+    (loop with point = (point)
 	  with length = (length string)
 	  with use-region-case = (no-upper-p string)
 	  for occurrences from 0
@@ -280,15 +332,21 @@ If found, leaves point before the word. If not, leaves point where it is."
 
 (defun query-replace-find-next-match (state)
   (with-accessors ((string string1)
-                   (buffers buffers)
-                   (mark mark)) state
-    (let ((offset-before (offset mark)))
+                   (targets targets)) state
+    (let* ((mark (point (view (drei-instance (targets state)))))
+           (offset-before (offset mark)))
       (search-forward mark string :test (case-relevant-test string))
-      (/= (offset mark) offset-before))))
+      (if (= (offset mark) offset-before)
+          (when (subsequent-targets-p targets)
+            (next-target targets)
+            (beginning-of-buffer (point (buffer (drei-instance targets))))
+            (query-replace-find-next-match state))
+        t))))
 
 (define-command (com-query-replace :name t :command-table search-table) ()
-  (let* ((pane *current-window*)
-         (old-state (query-replace-state pane))
+  (require-minibuffer)
+  (let* ((drei (drei-instance))
+         (old-state (query-replace-state drei))
          (old-string1 (when old-state (string1 old-state)))
          (old-string2 (when old-state (string2 old-state)))
          (string1 (handler-case 
@@ -313,93 +371,99 @@ If found, leaves point before the word. If not, leaves point where it is."
                     (error () (progn (beep)
                                      (display-message "Empty string")
                                      (return-from com-query-replace nil))))))
-    (setf (query-replace-state pane) (make-instance 'query-replace-state
+    (setf (query-replace-state drei) (make-instance 'query-replace-state
                                                     :string1 string1
                                                     :string2 string2
-                                                    :mark *current-point*))
-    (when (query-replace-find-next-match (query-replace-state pane))
-      (setf (query-replace-mode pane) t)
-      (display-message "Replace ~A with ~A:"
-                       string1 string2)
-      (simple-command-loop 'query-replace-drei-table
-                           (query-replace-mode pane)
-                           ((setf (query-replace-mode pane) nil)
-                            (display-message "Replaced ~A occurence~:P"
-                                             (occurrences (query-replace-state pane))))
-                           ((setf (query-replace-mode pane) nil)
-                            (signal 'abort-gesture :event *current-gesture*))))))
+                                                    :targets (funcall *default-target-creator* drei)))
+    (activate-target-specification (targets (query-replace-state drei)))
+    (if (query-replace-find-next-match (query-replace-state drei))
+        (progn
+          (setf (query-replace-mode drei) t)
+          (display-message "Replace ~A with ~A:"
+                           string1 string2)
+          (simple-command-loop 'query-replace-drei-table
+                               (query-replace-mode drei)
+                               ((setf (query-replace-mode drei) nil)
+                                (deactivate-target-specification (targets (query-replace-state drei)))
+                                (display-message "Replaced ~A occurence~:P"
+                                                 (occurrences (query-replace-state drei))))
+                               ((setf (query-replace-mode drei) nil)
+                                (signal 'abort-gesture :event *current-gesture*))))
+      (display-message "Replaced 0 occurences"))))
 
 (set-key 'com-query-replace
 	 'search-table
 	 '((#\% :shift :meta)))
 
 (define-command (com-query-replace-replace :name t :command-table query-replace-drei-table) ()
-  (let* ((pane *current-window*)
-         (state (query-replace-state pane)))
+  (let ((state (query-replace-state (drei-instance))))
     (with-accessors ((string1 string1)
                      (string2 string2)
-                     (occurrences occurrences)) state
-      (let ((string1-length (length string1)))
-        (backward-object (mark state) string1-length)
-        (replace-one-string (mark state)
+                     (occurrences occurrences)
+                     (targets targets)) state
+      (let ((string1-length (length string1))
+            (mark (point (view (drei-instance targets)))))
+        (backward-object mark string1-length)
+        (replace-one-string mark
                             string1-length
                             string2
                             (no-upper-p string1))
         (incf occurrences)
-        (if (query-replace-find-next-match (query-replace-state pane))
+        (if (query-replace-find-next-match state)
             (display-message "Replace ~A with ~A:"
                              string1 string2)
-            (setf (query-replace-mode pane) nil))))))
+            (setf (query-replace-mode (drei-instance)) nil))))))
 
 (define-command (com-query-replace-replace-and-quit
 		 :name t
 		 :command-table query-replace-drei-table)
     ()
-  (let* ((pane *current-window*)
-         (state (query-replace-state pane)))
+  (let ((state (query-replace-state (drei-instance))))
     (with-accessors ((string1 string1)
                      (string2 string2)
-                     (occurrences occurrences)) state
-      (let ((string1-length (length string1)))
-        (backward-object (mark state) string1-length)
-        (replace-one-string (mark state)
+                     (occurrences occurrences)
+                     (targets targets)) state
+      (let ((string1-length (length string1))
+            (mark (point (view (drei-instance targets)))))
+        (backward-object mark string1-length)
+        (replace-one-string mark
                             string1-length
                             string2
                             (no-upper-p string1))
         (incf occurrences)
-        (setf (query-replace-mode pane) nil)))))
+        (setf (query-replace-mode (drei-instance)) nil)))))
 
 (define-command (com-query-replace-replace-all
 		 :name t
 		 :command-table query-replace-drei-table)
     ()
-  (let* ((pane *current-window*)
-         (state (query-replace-state pane)))
+  (let ((state (query-replace-state (drei-instance))))
     (with-accessors ((string1 string1)
                      (string2 string2)
-                     (occurrences occurrences)) state
-        (let ((string1-length (length string1)))
-          (loop do (backward-object (mark state) string1-length)
-               (replace-one-string (mark state)
+                     (occurrences occurrences)
+                     (targets targets)) state
+        (let ((string1-length (length string1))
+              (mark (point (view (drei-instance targets)))))
+          (loop do (backward-object mark string1-length)
+               (replace-one-string mark
                                    string1-length
                                    string2
                                    (no-upper-p string1))
                (incf occurrences)
-               while (query-replace-find-next-match (query-replace-state pane))
-               finally (setf (query-replace-mode pane) nil))))))
+               while (query-replace-find-next-match state)
+               finally (setf (query-replace-mode (drei-instance)) nil))))))
 
 (define-command (com-query-replace-skip :name t :command-table query-replace-drei-table) ()
-  (let* ((pane *current-window*)
-         (state (query-replace-state pane)))
+  (let ((state (query-replace-state (drei-instance))))
     (with-accessors ((string1 string1)
                      (string2 string2)) state
       (if (query-replace-find-next-match state)
           (display-message "Replace ~A with ~A:"
                            string1 string2)
-          (setf (query-replace-mode pane) nil)))))
+          (setf (query-replace-mode (drei-instance)) nil)))))
 
 (define-command (com-query-replace-exit :name t :command-table query-replace-drei-table) ()
-  (setf (query-replace-mode *current-window*) nil))
+  (setf (query-replace-mode (drei-instance)) nil))
 
 (defun query-replace-set-key (gesture command)
   (add-command-to-command-table command 'query-replace-drei-table
@@ -431,23 +495,29 @@ If found, leaves point before the word. If not, leaves point where it is."
 	    do (princ char result))))
 
 (define-command (com-regex-search-forward :name t :command-table search-table) ()
+  (require-minibuffer)
   (let ((string (accept 'string :prompt "RE search"
 			:delimiter-gestures nil
 			:activation-gestures
 			'(:newline :return))))
-    (re-search-forward *current-point* (normalise-minibuffer-regex string))))
+    (simple-search-forward (drei-instance)
+                        #'(lambda (mark)
+                            (re-search-forward mark (normalise-minibuffer-regex string))))))
 
 (define-command (com-regex-search-backward :name t :command-table search-table) ()
+  (require-minibuffer)
   (let ((string (accept 'string :prompt "RE search backward"
 			:delimiter-gestures nil
 			:activation-gestures
 			'(:newline :return))))
-    (re-search-backward *current-point* (normalise-minibuffer-regex string))))
+    (simple-search-backward (drei-instance)
+                        #'(lambda (mark)
+                            (re-search-backward mark (normalise-minibuffer-regex string))))))
 
 (define-command (com-how-many :name t :command-table search-table)
     ((regex 'string :prompt "How many matches for"))
   (let* ((re (normalise-minibuffer-regex regex))
-	 (mark (clone-mark *current-point*))
+	 (mark (clone-mark (point)))
 	 (occurrences (loop for count from 0
 			    while (re-search-forward mark re)
 			    finally (return count))))
