@@ -2,6 +2,8 @@
 
 ;;;  (c) copyright 2005 by
 ;;;           Robert Strandh (strandh@labri.fr)
+;;;  (c) copyright 2006-2007 by
+;;;           Troels Henriksen (athas@sigkill.dk)
 
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Library General Public
@@ -26,41 +28,46 @@
 ;;; 
 ;;; Querying ESAs.
 
-(defgeneric buffers (application-frame)
+(defvar *esa-instance* nil
+  "This symbol should be bound to an ESA instance, though any
+object will do, provided the proper methods are defined. It will
+be used as the argument to the various \"query\" functions
+defined by ESA. For the vast majority of ESAs, `*esa-instance*'
+will probably have the same value as `*application-frame*'.")
+
+(defgeneric buffers (esa)
   (:documentation "Return a list of all the buffers of the application."))
 
-(defgeneric frame-current-buffer (application-frame)
-  (:documentation "Return the current buffer of APPLICATION-FRAME.")
-  (:method ((frame application-frame))
-    nil))
+(defgeneric esa-current-buffer (esa)
+  (:documentation "Return the current buffer of APPLICATION-FRAME."))
 
-(defvar *current-buffer* nil
-  "When a command is being executed, the current buffer.")
+(defgeneric (setf esa-current-buffer) (new-buffer esa)
+  (:documentation "Replace the current buffer of
+APPLICATION-FRAME with NEW-BUFFER."))
 
 (defun current-buffer ()
-  "Return the current buffer of `*application-frame*'."
-  (frame-current-buffer *application-frame*))
+  "Return the currently active buffer of the running ESA."
+  (esa-current-buffer *esa-instance*))
 
-(defgeneric windows (application-frame)
-  (:documentation "Return a list of all the windows of the application.")
-  (:method ((application-frame application-frame))
+(defun (setf current-buffer) (new-buffer)
+  "Return the currently active buffer of the running ESA."
+  (setf (esa-current-buffer *esa-instance*) new-buffer))
+
+(defgeneric windows (esa)
+  (:documentation "Return a list of all the windows of the ESA.")
+  (:method ((esa application-frame))
     '()))
 
-(defgeneric frame-current-window (application-frame)
-  (:documentation "Return the current window of APPLICATION-FRAME.")
-  (:method ((frame application-frame))
-    (first (windows frame))))
-
-(defvar *current-window* nil
-  "When a command is being executed, the current window.")
+(defgeneric esa-current-window (esa)
+  (:documentation "Return the current window of ESA."))
 
 (defun current-window ()
-  "Return the current window of `*application-frame*'."  
-  (frame-current-window *application-frame*))
+  "Return the currently active window of the running ESA instance."
+  (esa-current-window *esa-instance*))
 
 (defvar *previous-command* nil
   "When a command is being executed, the command previously
-executed by the current frame.")
+executed by the application.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
@@ -301,7 +308,8 @@ current message was set."))
           (funcall continuation minibuffer))))
 
 (defmethod invoke-with-minibuffer-stream ((minibuffer pointer-documentation-pane) continuation)
-  (funcall continuation minibuffer))
+  (clim-extensions:with-output-to-pointer-documentation (stream (pane-frame minibuffer))
+    (funcall continuation stream)))
 
 (defmethod invoke-with-minibuffer-stream ((minibuffer null) continuation)
   nil)
@@ -346,8 +354,6 @@ on the `format-string' and the `format-args'."
   "While a command is being run, this symbol will be dynamically
 bound to the current command processor.")
 
-(defvar *numeric-argument-p* (list nil))
-
 (defun find-gestures (gestures start-table)
   (loop with table = (find-command-table start-table)
 	for (gesture . rest) on gestures
@@ -453,6 +459,19 @@ fundamental interface to the command processor."))
   (:method ((command-processor command-processor))
     (null (overriding-handler command-processor))))
 
+(defgeneric command-for-unbound-gestures (thing gestures)
+  (:documentation "Called when `gestures' is input by the user
+and there is no associated command in the current command
+table. The function should return either a (possibly incomplete)
+command or NIL. In the latter case (which is handled by a default
+method), the gestures will be treated as actual unbound
+gestures. `Thing' is something that might be interested in
+commands, at the beginning usually a command processor, but it
+can call the function for other objects it knows in order to get
+their opinion. `Gestures' is a list of gestures.")
+  (:method (thing gestures)
+    nil))
+
 (defclass instant-macro-execution-mixin ()
   ()
   (:documentation "Subclasses of this class will immediately
@@ -467,22 +486,8 @@ fundamental interface to the command processor."))
      do (process-gesture drei gesture)
      finally (setf (executingp drei) nil)))
 
-(defclass macrorecord-processed-gestures-mixin ()
-  ()
-  (:documentation "Subclasses of this class will perform gesture
-recording for macro recording when the gesture is being
-processed. This is important when gesture reading does not go
-through `esa-read-gesture', for example when the command
-processor is being in an event-handling context."))
-
-(defmethod process-gesture :before ((command-processor macrorecord-processed-gestures-mixin) gesture)
-  (when (and (recordingp command-processor)
-             (directly-processing-p command-processor))
-    (push gesture (recorded-keys command-processor))))
-
 (defclass asynchronous-command-processor (command-processor
-                                          instant-macro-execution-mixin
-                                          macrorecord-processed-gestures-mixin)
+                                          instant-macro-execution-mixin)
   ()
   (:documentation "Helper class that provides behavior necessary
 for a command processor that expects to receive gestures through
@@ -491,8 +496,9 @@ asynchronous event handling, and not through
 
 (defmethod process-gesture :before ((command-processor asynchronous-command-processor) gesture)
   (when (and (find gesture *abort-gestures*
-                   :test #'gesture-matches-gesture-name-p)
+              :test #'gesture-matches-gesture-name-p)
              (directly-processing-p command-processor))
+    (setf (accumulated-gestures command-processor) nil)
     (signal 'abort-gesture :event gesture)))
 
 (defclass command-loop-command-processor (command-processor)
@@ -629,41 +635,58 @@ of a prefix arg returns 1 (and nil)."
 	  (t (values 1 nil (when first-gesture
                              (cons first-gesture gestures)))))))
 
-(defun substitute-numeric-argument-p (command numargp)
-  (substitute numargp *numeric-argument-p* command :test #'eq))
-
-(defgeneric process-gestures (command-processor))
+(defgeneric process-gestures (command-processor)
+  (:documentation "Process the gestures accumulated in
+`command-processor', returning T if there are no gestures
+accumulated or the accumulated gestures correspond to a
+command. In this case, the command will also be executed and the
+list of accumulated gestures set to NIL. Will return NIL if the
+accumulated gestures do not yet correspond to a command, but
+eventually could, if more gestures are provided. Signals
+`unbound-gesture-sequence' if the accumulated gestures could
+never refer to a command."))
 
 (defmethod process-gestures ((command-processor command-processor))
   (multiple-value-bind (prefix-arg prefix-p gestures)
       (process-gestures-for-numeric-argument
        (accumulated-gestures command-processor))
-    (cond ((null gestures)
-           t)
-          (t
-           (let* ((command-table (command-table command-processor))
-                  (item (find-gestures-with-inheritance gestures command-table)))
-             (cond 
-               ((not item)
-                (setf (accumulated-gestures command-processor) nil)
-                (error 'unbound-gesture-sequence :gestures gestures))
-               ((eq (command-menu-item-type item) :command)
-                (let ((command (command-menu-item-value item))
-                      (*current-gesture* (first (last gestures))))
-                  (unless (consp command)
-                    (setf command (list command)))
-                  (setf command (substitute-numeric-argument-marker command prefix-arg))
-                  (setf command (substitute-numeric-argument-p command prefix-p))
-                  (unwind-protect (when (member *unsupplied-argument-marker* command :test #'eq)
-                                    (setq command
+    (flet ((commandp (object)
+             (or (listp object) (symbolp object))))
+      (cond ((null gestures)
+             t)
+            (t
+             (let* ((command-table (command-table command-processor))
+                    (item (or (find-gestures-with-inheritance gestures command-table)
+                              (command-for-unbound-gestures command-processor gestures))))
+               (cond 
+                 ((not item)
+                  (setf (accumulated-gestures command-processor) nil)
+                  (error 'unbound-gesture-sequence :gestures gestures))
+                 ((or (commandp item) ; c-f-u-g does not return a menu-item.
+                      (eq (command-menu-item-type item) :command))
+                  (let ((command (if (commandp item) item
+                                     (command-menu-item-value item)))
+                        (*current-gesture* (first (last gestures))))
+                    (unless (consp command)
+                      (setf command (list command)))
+                    ;; Call `*partial-command-parser*' to handle numeric
+                    ;; argument.
+                    (unwind-protect (setq command
                                           (funcall
                                            *partial-command-parser*
                                            (command-table command-processor)
-                                           *standard-input* command 0)))
-                   (setf (accumulated-gestures command-processor) nil))
-                  (funcall (command-executor command-processor) command-processor command)
-                  nil))
-               (t t)))))))
+                                           *standard-input* command 0 (when prefix-p
+                                                                        prefix-arg)))
+                      ;; If we are macrorecording, store whatever the user
+                      ;; did to invoke this command.
+                      (when (recordingp command-processor)
+                        (setf (recorded-keys command-processor)
+                              (append (accumulated-gestures command-processor)
+                                      (recorded-keys command-processor))))
+                      (setf (accumulated-gestures command-processor) nil))
+                    (funcall (command-executor command-processor) command-processor command)
+                    nil))
+                 (t t))))))))
 
 (defmethod process-gesture :around ((command-processor command-processor) gesture)
   (with-accessors ((overriding-handler overriding-handler)) command-processor
@@ -686,14 +709,15 @@ of a prefix arg returns 1 (and nil)."
   (loop
      for gesture = (read-gesture :stream stream)
      until (proper-gesture-p gesture)
-     finally (progn (when (recordingp command-processor)
-                      (push gesture (recorded-keys command-processor)))
-                    (return gesture))))
+     finally (return gesture)))
 
 (defun esa-unread-gesture (gesture &key (command-processor *command-processor*)
                            (stream *standard-input*))
   (cond ((recordingp command-processor)
-	 (pop (recorded-keys command-processor))
+         (cond ((equal (first (recorded-keys command-processor)) gesture)
+                (pop (recorded-keys command-processor)))
+               ((equal (first (accumulated-gestures command-processor)) gesture)
+                (pop (accumulated-gestures command-processor))))
 	 (unread-gesture gesture :stream stream))
 	((executingp command-processor)
 	 (push gesture (remaining-keys command-processor)))
@@ -727,14 +751,21 @@ corresponding commands in `command-table' and invoke them using
          (setq command (list command)))       
        (when (member *unsupplied-argument-marker* command :test #'eq)
          (setq command
-               (funcall 
+               (funcall
                 *partial-command-parser*
-                (frame-command-table command-processor)
-                (frame-standard-input command-processor)
-                command 0)))
+                (command-table command-processor)
+                *standard-input* command 0)))
        (funcall (command-executor command-processor)
                 command-processor command)))))
 
+(defmethod process-gestures-or-command :around ((command-processor command-processor))
+  (handler-case (call-next-method)
+    (abort-gesture (c)
+      ;; If the user aborts, we want to forget whatever previous
+      ;; gestures he entered since the last command execution.
+      (setf (accumulated-gestures command-processor) nil)
+      (signal c))))
+
 (defmethod process-gestures-or-command ((command-processor command-processor))
   ;; Build up a list of gestures and repeatedly pass them to
   ;; `process-gestures'. This "clumsy" approach is chosen because we
@@ -743,7 +774,8 @@ corresponding commands in `command-table' and invoke them using
   ;; rescanning of accumulated input data or some yet-unimplemented
   ;; complex state retaining mechanism (such as continuations).
   (loop
-     (setf *current-gesture* (esa-read-gesture :command-processor command-processor))
+     (setf *current-gesture*
+           (esa-read-gesture :command-processor command-processor))
      (unless (process-gesture command-processor *current-gesture*)
        (return))))
 
@@ -754,6 +786,12 @@ corresponding commands in `command-table' and invoke them using
 (defclass esa-frame-mixin (command-processor)
   ((windows :accessor windows)))
 
+(defmethod esa-current-buffer ((esa esa-frame-mixin))
+  (first (buffers esa)))
+
+(defmethod esa-current-window ((esa esa-frame-mixin))
+  (first (windows esa)))
+
 (defmethod command-table ((frame esa-frame-mixin))
   (find-applicable-command-table frame))
 
@@ -783,7 +821,7 @@ corresponding commands in `command-table' and invoke them using
   ;; FIXME: I'm not sure that we want to do this for commands sent
   ;; from other threads; we almost certainly don't want to do it twice
   ;; in such cases...
-  (setf (previous-command (frame-current-window frame)) command))
+  (setf (previous-command (esa-current-window frame)) command))
 
 (defmethod execute-frame-command :around ((frame esa-frame-mixin) command)
   (call-next-method)
@@ -838,16 +876,15 @@ used.")
                     (*partial-command-parser* ,partial-command-parser)
                     (*extended-command-prompt* ,prompt)
                     (*pointer-documentation-output*
-                     (frame-pointer-documentation-output ,frame)))
+                     (frame-pointer-documentation-output ,frame))
+                    (*esa-instance* ,frame))
                 (unless (eq (frame-state ,frame) :enabled)
                   (enable-frame ,frame))
                 (redisplay-frame-panes ,frame :force-p t)
                 (loop
                    do (restart-case
                           (handler-case
-                              (let* ((*current-window* (frame-current-window ,frame))
-                                     (*current-buffer* (frame-current-buffer ,frame))
-                                     (*command-processor* ,frame)
+                              (let* ((*command-processor* ,frame)
                                      (command-table (find-applicable-command-table ,frame))
                                      ,@bindings)
                                 ;; for presentation-to-command-translators,
@@ -1029,15 +1066,29 @@ First ask if modified buffers should be saved. If you decide not to save a modif
 ;;; 
 ;;; Help
 
-(defgeneric help-stream (frame title))
+(defgeneric invoke-with-help-stream (esa title continuation)
+  (:documentation "Invoke `continuation' with a single argument -
+a stream for writing on-line help for `esa' onto. The stream
+should have the title, or name, `title' (a string), but the
+specific meaning of this is left to the respective ESA."))
 
-(defmethod help-stream (frame title)
-  (open-window-stream
-   :label title
-   :input-buffer (#+(or mcclim building-mcclim) climi::frame-event-queue
-                    #-(or mcclim building-mcclim) silica:frame-input-buffer
-                    *application-frame*)
-   :width 400))
+(defmethod invoke-with-help-stream (frame title continuation)
+  (funcall continuation
+           (open-window-stream
+            :label title
+            :input-buffer (#+(or mcclim building-mcclim) climi::frame-event-queue
+                             #-(or mcclim building-mcclim) silica:frame-input-buffer
+                             *application-frame*)
+            :width 400)))
+
+(defmacro with-help-stream ((stream title) &body body)
+  "Evaluate `body' with `stream' bound to a stream suitable for
+writing help information on. `Title' must evaluate to a string,
+and will be used for naming the resulting stream, if that makes
+sense for the ESA."
+  `(invoke-with-help-stream *esa-instance* ,title
+                            #'(lambda (,stream)
+                                ,@body)))
 
 (defun read-gestures-for-help (command-table)
   (with-input-focus (t)
@@ -1049,8 +1100,8 @@ First ask if modified buffers should be saved. If you decide not to save a modif
           when (eq (command-menu-item-type item) :command)
             do (return (values (command-menu-item-value item) gestures)))))
 
-(defun describe-key-briefly (pane)
-  (let ((command-table (command-table pane)))
+(defun describe-key-briefly (frame)
+  (let ((command-table (find-applicable-command-table frame)))
     (multiple-value-bind (command gestures)
 	(read-gestures-for-help command-table)
       (when (consp command)
@@ -1103,8 +1154,10 @@ First ask if modified buffers should be saved. If you decide not to save a modif
                 #'(lambda (menu-name keystroke item)
                     (declare (ignore menu-name))
                     (cond ((and (eq (command-menu-item-type item) :command)
-                                (listp (command-menu-item-value item))
-                                (eq (car (command-menu-item-value item)) command))
+                                (or (and (symbolp (command-menu-item-value item))
+                                         (eq (command-menu-item-value item) command))
+                                    (and (listp (command-menu-item-value item))
+                                         (eq (car (command-menu-item-value item)) command))))
                            (push (cons keystroke prefix) keystrokes))
                           ((eq (command-menu-item-type item) :menu)
                            (helper command (command-menu-item-value item) (cons keystroke prefix)))
@@ -1290,8 +1343,7 @@ First ask if modified buffers should be saved. If you decide not to save a modif
 	       (mapcar #'(lambda (arg)
                            (cond ((eq arg *unsupplied-argument-marker*)
                                   "unsupplied-argument")
-                                 ((or (eq arg *numeric-argument-marker*)
-                                      (eq arg *numeric-argument-p*))
+                                 ((eq arg *numeric-argument-marker*)
                                   "numeric-argument")
                                  (t arg))) command-args)))
       (terpri stream)
@@ -1339,13 +1391,13 @@ First ask if modified buffers should be saved. If you decide not to save a modif
   "Prompt for a key and show the command it invokes."  
   (display-message "Describe key briefly:")
   (redisplay-frame-panes *application-frame*)
-  (describe-key-briefly (car (windows *application-frame*))))
+  (describe-key-briefly *application-frame*))
 
 (set-key 'com-describe-key-briefly 'help-table '((#\h :control) (#\c)))
 
 (define-command (com-where-is :name t :command-table help-table) ()
   "Prompt for a command name and show the key that invokes it."
-  (let* ((command-table (command-table (car (windows *application-frame*))))
+  (let* ((command-table (find-applicable-command-table *application-frame*))
 	 (command
 	  (handler-case
 	      (accept
@@ -1369,14 +1421,14 @@ First ask if modified buffers should be saved. If you decide not to save a modif
     ((sort-by-keystrokes 'boolean :prompt "Sort by keystrokes?"))
   "Show which keys invoke which commands.
 Without a numeric prefix, sorts the list by command name. With a numeric prefix, sorts by key."
-  (let ((stream (help-stream *application-frame* (format nil "Help: Describe Bindings")))
-	 (command-table (find-applicable-command-table *application-frame*)))
-    (describe-bindings stream command-table
-		       (if sort-by-keystrokes
-			   #'sort-by-keystrokes
-			   #'sort-by-name))))
+  (let ((command-table (find-applicable-command-table *application-frame*)))
+    (with-help-stream (stream (format nil "Help: Describe Bindings"))
+      (describe-bindings stream command-table
+                         (if sort-by-keystrokes
+                             #'sort-by-keystrokes
+                             #'sort-by-name)))))
 
-(set-key `(com-describe-bindings ,*numeric-argument-p*) 'help-table '((#\h :control) (#\b)))
+(set-key `(com-describe-bindings ,*numeric-argument-marker*) 'help-table '((#\h :control) (#\b)))
 
 (define-command (com-describe-key :name t :command-table help-table)
     ()
@@ -1392,12 +1444,10 @@ documentation and other details will be displayed in a typeout pane."
       (let ((gesture-name (format nil "~{~A~#[~; ~; ~]~}"
                                   (mapcar #'gesture-name gestures))))
         (if command
-            (let ((out-stream
-		   (help-stream *application-frame*
-				(format nil "~10THelp: Describe Key for ~A" gesture-name))))
+            (with-help-stream (out-stream (format nil "~10THelp: Describe Key for ~A" gesture-name))
               (describe-command-binding-to-stream gesture-name command
-                                                  :command-table command-table
-                                                  :stream out-stream))
+               :command-table command-table
+               :stream out-stream))
             (display-message "Unbound gesture: ~A" gesture-name))))))
 
 (set-key 'com-describe-key
@@ -1407,15 +1457,14 @@ documentation and other details will be displayed in a typeout pane."
 (define-command (com-describe-command :name t :command-table help-table)
     ((command 'command-name :prompt "Describe command"))
   "Display documentation for the given command."
-  (let* ((command-table (find-applicable-command-table *application-frame*))
-	 (out-stream (help-stream *application-frame*
-				  (format nil "~10THelp: Describe Command for ~A"
+  (let ((command-table (find-applicable-command-table *application-frame*)))
+    (with-help-stream (out-stream (format nil "~10THelp: Describe Command for ~A"
 					  (command-line-name-for-command command
 									 command-table
-									 :errorp nil)))))
-    (describe-command-to-stream command
-                                :command-table command-table
-                                :stream out-stream)))
+									 :errorp nil)))
+      (describe-command-to-stream command
+       :command-table command-table
+       :stream out-stream))))
 
 (set-key `(com-describe-command ,*unsupplied-argument-marker*)
          'help-table
@@ -1460,35 +1509,41 @@ Words are comma delimited. When more than two words are given, the documentation
 			    collect (cons function keys))))
       (if (null results)
 	  (display-message "No results for ~{~A~^, ~}" words)
-	  (let ((out-stream (help-stream *application-frame*
-					 (format nil "~10THelp: Apropos ~{~A~^, ~}"
-						 words))))
-	    (loop for (command . keys) in results
-		  for documentation = (or (documentation command 'function)
-					  "Not documented.")
-		  do (with-text-style (out-stream '(:sans-serif :bold nil))
-		       (present command
-				`(command-name :command-table ,command-table)
-				:stream out-stream))
-		     (with-drawing-options (out-stream :ink +dark-blue+
-						       :text-style '(:fix nil nil))
-		       (format out-stream "~30T~:[M-x ... RETURN~;~:*~{~A~^, ~}~]"
-			       (mapcar (lambda (keystrokes)
-					 (format nil "~{~A~^ ~}"
-						 (mapcar #'gesture-name (reverse keystrokes))))
-				       (car keys))))
-		     (with-text-style (out-stream '(:sans-serif nil nil))
-		       (format out-stream "~&~2T~A~%"
-			       (subseq documentation 0 (position #\Newline documentation))))
-		  count command into length
-		  finally (change-space-requirements out-stream
-				 :height (* length (stream-line-height out-stream)))
-			  (scroll-extent out-stream 0 0)))))))
+          (with-help-stream (out-stream (format nil "~10THelp: Apropos ~{~A~^, ~}" words))
+            (loop for (command . keys) in results
+                  for documentation = (or (documentation command 'function)
+                                          "Not documented.")
+                  do (with-text-style (out-stream '(:sans-serif :bold nil))
+                       (present command
+                                `(command-name :command-table ,command-table)
+                                :stream out-stream))
+                  (with-drawing-options (out-stream :ink +dark-blue+
+                                                    :text-style '(:fix nil nil))
+                    (format out-stream "~30T~:[M-x ... RETURN~;~:*~{~A~^, ~}~]"
+                            (mapcar (lambda (keystrokes)
+                                      (format nil "~{~A~^ ~}"
+                                              (mapcar #'gesture-name (reverse keystrokes))))
+                                    (car keys))))
+                  (with-text-style (out-stream '(:sans-serif nil nil))
+                    (format out-stream "~&~2T~A~%"
+                            (subseq documentation 0 (position #\Newline documentation))))
+                  count command into length
+                  finally (change-space-requirements out-stream
+                           :height (* length (stream-line-height out-stream)))
+                  (scroll-extent out-stream 0 0)))))))
 
 (set-key `(com-apropos-command ,*unsupplied-argument-marker*)
 	 'help-table
 	 '((#\h :control) (#\a)))
 
+(define-menu-table help-menu-table (help-table)
+  'com-where-is
+  '(com-describe-bindings nil)
+  '(com-describe-bindings t)
+  'com-describe-key
+  `(com-describe-command ,*unsupplied-argument-marker*)
+  `(com-apropos-command ,*unsupplied-argument-marker*))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
 ;;; Keyboard macros
@@ -1522,7 +1577,7 @@ Use C-x ( to start recording a macro, and C-x e to run it."
 (define-command (com-call-last-kbd-macro
 		 :name t
 		 :command-table keyboard-macro-table)
-    ((count 'integer :prompt "How many times?"))
+    ((count 'integer :prompt "How many times?" :default 1))
   "Run the last keyboard macro that was defined.
 Use C-x ( to start and C-x ) to finish recording a keyboard macro."
   (setf (remaining-keys *command-processor*)
@@ -1532,6 +1587,11 @@ Use C-x ( to start and C-x ) to finish recording a keyboard macro."
 (set-key `(com-call-last-kbd-macro ,*numeric-argument-marker*)
          'keyboard-macro-table '((#\x :control) #\e))
 
+(define-menu-table keyboard-macro-menu-table (keyboard-macro-table)
+  'com-start-kbd-macro
+  'com-end-kbd-macro
+  `(com-call-last-kbd-macro ,*unsupplied-argument-marker*))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
 ;;; example application
