@@ -70,13 +70,21 @@
   ((name :initarg :name :reader command-table-name)
    (inherit-from :initarg :inherit-from
 		 :initform '()
-		 :reader command-table-inherit-from)
+		 :reader command-table-inherit-from
+                 :type list)
    (commands  :accessor commands :initarg :commands
 	     :initform (make-hash-table :test #'eq))
    (command-line-names :accessor command-line-names
 		       :initform (make-hash-table :test #'equal))
    (presentation-translators :reader presentation-translators
 			     :initform (make-instance 'translator-table))
+   (inherit-menu :reader inherit-menu
+                 :initform nil
+                 ;; We interpret :menu to mean "inherit menu items
+                 ;; without keystrokes" and :keystrokes to mean
+                 ;; "inherit menu items with keystrokes".
+                 :type (member nil t :menu :keystrokes)
+                 :initarg :inherit-menu)
    (menu :initarg :menu :initform '())
    (keystroke-accelerators :initform nil)
    (keystroke-items :initform nil)))
@@ -85,6 +93,12 @@
   (print-unreadable-object (table stream :identity t :type t)
     (format stream "~S" (command-table-name table))))
 
+;;; We store command-table designators, but this function should
+;;; return command table objects.
+(defmethod command-table-inherit-from :around
+    ((command-table standard-command-table))
+  (mapcar #'find-command-table (call-next-method)))
+
 ;;; Franz user manual says that this slot is setf-able
 (defgeneric (setf command-table-inherit-from) (inherit-from table))
 
@@ -93,12 +107,40 @@
   (invalidate-translator-caches)
   (setf (slot-value table 'inherit-from) inherit))
 
+(defun inherit-keystrokes (command-table)
+  "Return true if `command-table' (which must be a command table
+designator) inherits keystrokes."
+  (let ((inherit-menu (inherit-menu (find-command-table command-table))))
+    (or (eq inherit-menu t)
+        (eq inherit-menu :keystrokes))))
+
+(defun inherit-menu-items (command-table)
+  "Return true if `command-table' (which must be a command table
+designator) inherits menu items."
+  (let ((inherit-menu (inherit-menu (find-command-table command-table))))
+    (or (inherit-keystrokes command-table)
+        (eq inherit-menu :menu))))
+
 (defparameter *command-tables* (make-hash-table :test #'eq))
 
 (define-condition command-table-error (simple-error)
-  ()
+  ((command-table-name :reader error-command-table-name
+                       :initform nil
+                       :initarg :command-table-name))
   (:default-initargs :format-control "" :format-arguments nil))
 
+(defmethod print-object ((object command-table-error) stream)
+  (print-unreadable-object (object stream :type t :identity t)
+    (when (error-command-table-name object)
+      (princ (error-command-table-name object) stream))))
+
+(defun command-table-designator-as-name (designator)
+  "Return the name of `designator' if it is a command table,
+`designator' otherwise."
+  (if (typep designator 'standard-command-table)
+      (command-table-name designator)
+      designator))
+
 (define-condition command-table-not-found (command-table-error)
   ())
 
@@ -117,7 +159,7 @@
 (defun find-command-table (name &key (errorp t))
   (cond ((command-table-p name) name)
 	((gethash name *command-tables*))
-	(errorp (error 'command-table-not-found))
+	(errorp (error 'command-table-not-found :command-table-name name))
 	(t nil)))
 
 (define-presentation-method present (object (type command-table) stream
@@ -160,13 +202,14 @@
 		     :menu nil))
 
 ; adjusted to allow anonymous command-tables for menu-bars
-(defun make-command-table (name &key inherit-from menu (errorp t))
+(defun make-command-table (name &key inherit-from menu inherit-menu (errorp t))
   (unless inherit-from
     (setq inherit-from '(global-command-table)))
   (if (and name errorp (gethash name *command-tables*))
-      (error 'command-table-already-exists)
+      (error 'command-table-already-exists :command-table-name name)
       (let ((result (make-instance 'standard-command-table :name name
 	                 :inherit-from inherit-from
+                         :inherit-menu inherit-menu
 	                 :menu (menu-items-from-list menu))))
         (when name
           (setf (gethash name *command-tables*) result))
@@ -174,7 +217,7 @@
 
 (make-command-table 'user-command-table)
 
-(defmacro define-command-table (name &key inherit-from menu)
+(defmacro define-command-table (name &key inherit-from menu inherit-menu)
   `(let ((old-table (gethash ',name *command-tables* nil))
 	 (inherit-from-arg (or ',inherit-from '(global-command-table))))
      (if old-table
@@ -184,6 +227,7 @@
 	   old-table)
 	 (make-command-table ',name
 			     :inherit-from inherit-from-arg
+                             :inherit-menu ,inherit-menu
 			     :menu ',menu
 			     :errorp nil))))
 
@@ -194,7 +238,7 @@
 	 (item (gethash command-name (commands table))))
     (if (null item)
 	(when errorp
-	  (error 'command-not-present))
+	  (error 'command-not-present :command-table-name (command-table-name command-table)))
 	(progn 
 	  (when (typep item '%menu-item)
 	    (remove-menu-item-from-command-table table
@@ -243,7 +287,7 @@
 				      :command-line-name name)))
 	     (after (getf menu-options :after)))
 	(when (and errorp (gethash command-name (commands table)))
-	  (error 'command-already-present))
+	  (error 'command-already-present :command-table-name command-table))
 	(remove-command-from-command-table command-name table :errorp nil)
 	(setf (gethash command-name (commands table)) item)
 	(when name
@@ -304,7 +348,7 @@
 	     (values value table)))))
    (find-command-table command-table))
   (if errorp
-      (error 'command-not-accessible)))
+      (error 'command-not-accessible :command-table-name command-table)))
 
 (defun command-line-name-for-command (command-name command-table
 				      &key (errorp t))
@@ -317,16 +361,22 @@
   (cond ((eq errorp :create)
 	 (command-name-from-symbol command-name))
 	(errorp
-	 (error 'command-not-accessible))
+	 (error 'command-not-accessible :command-table-name
+                (command-table-designator-as-name command-table)))
 	(t nil)))
 
 (defun find-menu-item (menu-name command-table &key (errorp t))
   (let* ((table (find-command-table command-table))
 	 (mem (member menu-name (slot-value table 'menu)
-		      :key #'command-menu-item-name :test #'string-equal)))
-    (cond (mem (values (car mem) command-table))
-	  (errorp (error 'command-not-accessible))
-	  (t nil))))
+               :key #'command-menu-item-name :test #'string-equal)))
+    (if mem
+        (values (car mem) command-table)
+        (or (find-if #'(lambda (table)
+                         (find-menu-item menu-name table :errorp nil))
+                     (command-table-inherit-from table))
+            (when errorp
+              (error 'command-not-accessible :command-table-name
+                     (command-table-designator-as-name table)))))))
 
 (defun remove-menu-item-from-command-table (command-table string
 					    &key (errorp t))
@@ -334,7 +384,8 @@
 	(item (find-menu-item string command-table :errorp nil)))
     (with-slots (menu) table
       (if (and errorp (not item))
-	  (error 'command-not-present)
+	  (error 'command-not-present :command-table-name
+                 (command-table-designator-as-name table))
 	  (setf menu (delete string menu
 			     :key #'command-menu-item-name
 			     :test #'string-equal))))))
@@ -364,18 +415,20 @@
 (defun %add-menu-item (command-table item after)
   (with-slots (menu)
       command-table
+    (when (null menu)
+      (setf after :start))
     (case after
       (:start (push item menu))
       ((:end nil) (setf menu (nconc menu (list item))))
       (:sort (setf menu (sort (cons item menu)
-			      #'string-lessp
-			      :key #'command-menu-item-name)))
+                              #'string-lessp
+                              :key #'command-menu-item-name)))
       (t (push item
-	       (cdr (member after menu
-			    :key #'command-menu-item-name
-			    :test #'string-equal))))))
+               (cdr (member after menu
+                     :key #'command-menu-item-name
+                     :test #'string-equal))))))
   (when (and (slot-boundp item 'keystroke)
-	      (slot-value item 'keystroke))
+             (slot-value item 'keystroke))
     (%add-keystroke-item command-table (slot-value item 'keystroke) item nil)))
 
 
@@ -388,7 +441,8 @@
   (let* ((table (find-command-table command-table))
 	 (old-item (find-menu-item string command-table :errorp nil)))
     (cond ((and errorp old-item)
-	   (error 'command-already-present))
+	   (error 'command-already-present :command-table-name
+                  (command-table-designator-as-name table)))
 	  (old-item
 	   (remove-menu-item-from-command-table command-table string))
 	  (t nil))
@@ -397,14 +451,34 @@
 		    after)))
 
 (defun map-over-command-table-menu-items (function command-table)
-  (mapc #'(lambda (item)
- 	    (with-slots (menu-name keystroke) item
- 	      (funcall function
-		       menu-name
-		       (and (slot-boundp item 'keystroke) keystroke)
-		       item)))
-	(slot-value (find-command-table command-table) 'menu))
-  (values))
+  "Applies function to all of the items in `command-table's
+menu. `Command-table' must be a command table or the name of a
+command table. `Function' must be a function of three arguments,
+the menu name, the keystroke accelerator gesture (which will be
+NIL if there is none), and the command menu item; it has dynamic
+extent. The command menu items are mapped over in the order
+specified by `add-menu-item-to-command-table'. `Command-table' is
+a command table designator. Any inherited menu items will be
+mapped over after `command-table's own menu items.
+
+`Map-over-command-table-menu-items' does not descend into
+sub-menus. If the programmer requires this behavior, he should
+examine the type of the command menu item to see if it is
+`:menu'."
+  (let ((table-object (find-command-table command-table)))
+    (flet ((map-table-entries (table)
+             (mapc #'(lambda (item)
+                       (with-slots (menu-name keystroke) item
+                         (funcall function
+                                  menu-name
+                                  (and (slot-boundp item 'keystroke) keystroke)
+                                  item)))
+                   (slot-value table 'menu))))
+      (map-table-entries table-object)
+      (when (inherit-menu-items table-object)
+        (dolist (table (command-table-inherit-from table-object))
+          (map-over-command-table-menu-items function table))))
+    (values)))
 
 ;; At this point we should still see the gesture name as supplied by the
 ;; programmer in 'gesture'
@@ -417,7 +491,8 @@
                         (multiple-value-list (realize-gesture-spec :keyboard gesture))))
            (in-table (position gesture keystroke-accelerators :test #'equal)))
       (when (and in-table errorp)
-        (error 'command-already-present))
+        (error 'command-already-present :command-table-name
+               (command-table-designator-as-name command-table)))
       (if in-table
 	  (setf (nth in-table keystroke-items) item)
 	  (progn
@@ -454,7 +529,8 @@
 		  (setf (cdr accel-tail) (cddr accel-tail))
 		  (setf (cdr items-tail) (cddr items-tail))))
 	    (when errorp
-	      (error 'command-not-present))))))
+	      (error 'command-not-present :command-table-name
+                     (command-table-designator-as-name command-table)))))))
   nil)
 
 (defun map-over-command-table-keystrokes (function command-table)
@@ -478,7 +554,8 @@
 	  if (funcall test gesture keystroke)
 	  do (return-from find-keystroke-item (values item command-table)))
     (if errorp
-	(error 'command-not-present)
+	(error 'command-not-present :command-table-name
+               (command-table-designator-as-name command-table))
 	nil)))
 
 (defun lookup-keystroke-item (gesture command-table
@@ -501,10 +578,11 @@
 		   (values sub-item sub-command-table))))))
        command-table))))
 
-(defun partial-command-from-name (command-name)
+(defun partial-command-from-name (command-name command-table)
   (let ((parser (gethash command-name *command-parser-table*)))
     (if (null parser)
-        (error 'command-not-present)
+        (error 'command-not-present :command-table-name
+               (command-table-designator-as-name command-table))
         (cons command-name
               (mapcar #'(lambda (foo)
                           (declare (ignore foo))
@@ -527,7 +605,7 @@
     (if item
 	(let* ((value (command-menu-item-value item))
 	       (command (case (command-menu-item-type item)
-			 (:command
+                          (:command
 			  value)
 			 (:function
 			  (funcall value gesture numeric-arg))
@@ -536,7 +614,7 @@
 	  (if command
               ; Return a literal command, or create a partial command from a command-name
 	      (substitute-numeric-argument-marker (if (symbolp command)
-                                                      (partial-command-from-name command)
+                                                      (partial-command-from-name command command-table)
                                                       command)
                                                   numeric-arg)
 	      gesture))
@@ -585,7 +663,7 @@
 
 (defun accept-form-for-argument (stream arg)
   (let ((accept-keys '(:default :default-type :display-default
-		       :prompt :documentation)))
+		       :prompt :documentation :insert-default)))
     (destructuring-bind (name ptype &rest key-args
 			 &key (mentioned-default nil mentioned-default-p)
 			 &allow-other-keys)
@@ -614,7 +692,7 @@
 (defun accept-form-for-argument-partial (stream ptype-arg command-arg
 					 original-command-arg )
   (let ((accept-keys '(:default :default-type :display-default
-		       :prompt :documentation)))
+		       :prompt :documentation :insert-default)))
     (destructuring-bind (name ptype &rest key-args)
 	ptype-arg
       (declare (ignore name))
@@ -1149,7 +1227,7 @@
 
 ;;; In order for this to work, the input-editing-stream must implement
 ;;; a method for the nonstandard function
-;;; `input-editing-stream-bounding-rectangle'.
+;;; `input-editing-stream-output-record'.
 (defun command-line-read-remaining-arguments-for-partial-command
     (command-table stream partial-command start-position)
   (declare (ignore start-position))
@@ -1157,8 +1235,7 @@
 						 *command-parser-table*))))
     (if (encapsulating-stream-p stream)
 	(let ((interactor (encapsulating-stream-stream stream)))
-	  (multiple-value-bind (x1 y1 x2 y2)
-	      (input-editing-stream-bounding-rectangle stream)
+	  (with-bounding-rectangle* (x1 y1 x2 y2) (input-editing-stream-output-record stream)
 	    (declare (ignore y1 x2))
 	    ;; Start the dialog below the editor area
 	    (letf (((stream-cursor-position interactor) (values x1 y2)))
