@@ -240,6 +240,8 @@ accept of this query")))
                  (finalize-query-records *accepting-values-stream*)
 		 (setf (last-pass *accepting-values-stream*) t)
                  (redisplay arecord stream)))
+          (dolist (query (queries *accepting-values-stream*))
+            (finalize (editing-stream (record query)) nil))
           (erase-output-record arecord stream)
           (setf (stream-cursor-position stream)
                 (values cx cy)))))))
