@@ -364,7 +364,7 @@ documentation produced by presentations.")
 (defmethod frame-exit ((frame standard-application-frame))
   (if (eq *application-frame* frame)
       (signal 'frame-exit :frame frame)
-    (disown-frame (frame-manager frame) frame)))
+      (disown-frame (frame-manager frame) frame)))
 
 (defmethod frame-exit-frame ((c frame-exit))
   (%frame-exit-frame c))
@@ -395,22 +395,32 @@ documentation produced by presentations.")
   (let ((pane-object (if (typep pane 'pane)
 			 pane
 			 (find-pane-named frame pane))))
-    (multiple-value-bind (redisplayp clearp)
-	(pane-needs-redisplay pane-object)
-      (when force-p
-	(setq redisplayp (or redisplayp t)
-	      clearp t))
-      (when redisplayp
-	(let ((hilited (frame-hilited-presentation frame)))
-	  (when hilited
-	    (highlight-presentation-1 (car hilited) (cdr hilited) :unhighlight)
-	    (setf (frame-hilited-presentation frame) nil)))
-	(with-possible-double-buffering (frame pane-object)
-	  (when clearp
-	    (window-clear pane-object))
-	  (call-next-method))
-	(unless (or (eq redisplayp :command-loop) (eq redisplayp :no-clear))
-	  (setf (pane-needs-redisplay pane-object) nil))))))
+    (restart-case
+        (multiple-value-bind (redisplayp clearp)
+            (pane-needs-redisplay pane-object)
+          (when force-p
+            (setq redisplayp (or redisplayp t)
+                  clearp t))
+          (when redisplayp
+            (let ((hilited (frame-hilited-presentation frame)))
+              (when hilited
+                (highlight-presentation-1 (car hilited) (cdr hilited) :unhighlight)
+                (setf (frame-hilited-presentation frame) nil)))
+            (with-possible-double-buffering (frame pane-object)
+              (when clearp
+                (window-clear pane-object))
+              (call-next-method))
+            (unless (or (eq redisplayp :command-loop) (eq redisplayp :no-clear))
+              (setf (pane-needs-redisplay pane-object) nil))))
+      (clear-pane-try-again ()
+       :report "Clear the output history of the pane and reattempt forceful redisplay"
+       (window-clear pane)
+       (redisplay-frame-pane frame pane :force-p t))
+      (clear-pane ()
+       :report "Clear the output history of the pane, but don't redisplay"
+       (window-clear pane))
+      (skip-redisplay ()
+       :report "Skip this redisplay"))))
 
 (defmethod run-frame-top-level ((frame application-frame)
 				&key &allow-other-keys)
@@ -447,11 +457,11 @@ documentation produced by presentations.")
 			       (call-next-method)))
 		 (frame-layout-changed () nil)))
       (let ((fm (frame-manager frame)))
-	(case original-state
-	  (:disabled
-	   (disable-frame frame))
-	  (:disowned
-	   (disown-frame fm frame)))))))
+        (case original-state
+          (:disabled
+           (disable-frame frame))
+          (:disowned
+           (disown-frame fm frame)))))))
 
 (defparameter +default-prompt-style+ (make-text-style :fix :italic :normal))
 
@@ -521,13 +531,14 @@ documentation produced by presentations.")
       (object)
       (call-next-method)
     (menu-item
-     (let ((command (command-menu-item-value object)))
+     (let ((command (command-menu-item-value object))
+           (table (frame-command-table frame)))
        (unless (listp command)
-	 (setq command (partial-command-from-name command)))
+	 (setq command (partial-command-from-name command table)))
        (if (and (typep stream 'interactor-pane)
 		(partial-command-p command))
 	   (command-line-read-remaining-arguments-for-partial-command
-	    (frame-command-table frame) stream command 0)
+	    table stream command 0)
 	   command)))))
 
 (defmethod read-frame-command ((frame application-frame)
@@ -552,7 +563,13 @@ documentation produced by presentations.")
   ;; frames command loop. Perhaps looking ath the process slot?
   ;; --GB 2005-11-28
   (cond ((eq *application-frame* frame)
-         (apply (command-name command) (command-arguments command)))
+         (restart-case
+             (apply (command-name command) (command-arguments command))
+           (try-again ()
+            :report (lambda (stream)
+                      (format stream "Try executing the command ~A again"
+                              (command-name command)))
+            (execute-frame-command frame command))))
         (t
          (let ((eq (sheet-event-queue (frame-top-level-sheet frame))))
            (event-queue-append eq (make-instance 'execute-command-event
@@ -643,7 +660,10 @@ documentation produced by presentations.")
   (note-frame-enabled (frame-manager frame) frame))
 
 (defmethod disable-frame ((frame application-frame))
-  (setf (sheet-enabled-p (frame-top-level-sheet frame)) nil)
+  (let ((t-l-s (frame-top-level-sheet frame)))
+    (setf (sheet-enabled-p t-l-s) nil)
+    (when (port t-l-s)
+      (port-force-output (port t-l-s))))
   (setf (slot-value frame 'state) :disabled)
   (note-frame-disabled (frame-manager frame) frame))
 
@@ -1111,6 +1131,15 @@ documentation produced by presentations.")
   (declare (ignore input-context stream))
   (equal old-state new-state))
 
+(defun record-on-display (stream record)
+  "Return true if `record' is part of the output history of
+`stream', false otherwise."
+  (labels ((worker (record)
+             (or (eq record (stream-output-history stream))
+                 (and (not (null (output-record-parent record)))
+                      (worker (output-record-parent record))))))
+    (worker record)))
+
 (defgeneric frame-print-pointer-documentation
     (frame input-context stream state event))
 
@@ -1123,71 +1152,82 @@ documentation produced by presentations.")
     (let ((x (device-event-x event))
 	  (y (device-event-y event))
 	  (pstream *pointer-documentation-output*))
-      (loop for (button presentation translator context)
-	    in new-translators
-	    for name = (cadr (assoc button +button-documentation+))
-	    for first-one = t then nil
-	    do (progn
-		 (unless first-one
-		   (write-string "; " pstream))
-		 (unless (zerop current-modifier)
-		   (print-modifiers pstream current-modifier :short)
-		   (write-string "-" pstream))
-		 (format pstream "~A: " name)
-		 (document-presentation-translator translator
-						   presentation
-						   (input-context-type context)
-						   *application-frame*
-						   event
-						   stream
-						   x y
-						   :stream pstream
-						   :documentation-type
-						   :pointer))
-	    finally (when new-translators
-		      (write-char #\. pstream)))
+      (if (null new-translators)
+          (when (and (background-message pstream)
+                     (not (record-on-display pstream (background-message pstream))))
+            (cond ((> (get-universal-time)
+                      (+ (background-message-time pstream)
+                         *background-message-minimum-lifetime*))
+                   (setf (background-message pstream) nil))
+                  (t
+                   (setf (output-record-parent (background-message pstream)) nil)
+                   (stream-add-output-record pstream (background-message pstream))
+                   (replay (background-message pstream) pstream))))
+          (loop for (button presentation translator context)
+                in new-translators
+                for name = (cadr (assoc button +button-documentation+))
+                for first-one = t then nil
+                do (progn
+                     (unless first-one
+                       (write-string "; " pstream))
+                     (unless (zerop current-modifier)
+                       (print-modifiers pstream current-modifier :short)
+                       (write-string "-" pstream))
+                     (format pstream "~A: " name)
+                     (document-presentation-translator translator
+                                                       presentation
+                                                       (input-context-type context)
+                                                       *application-frame*
+                                                       event
+                                                       stream
+                                                       x y
+                                                       :stream pstream
+                                                       :documentation-type
+                                                       :pointer))
+                finally (when new-translators
+                          (write-char #\. pstream))))
       ;; Wasteful to do this after doing
       ;; find-innermost-presentation-context above... look at doing this
       ;; first and then doing the innermost test.
       (let ((all-translators (find-applicable-translators
-			      (stream-output-history stream)
-			      input-context
-			      *application-frame*
-			      stream
-			      x y
-			      :for-menu t))
-	    (other-modifiers nil))
-	(loop for (translator) in all-translators
-	      for gesture = (gesture translator)
-	      unless (eq gesture t)
-	      do (loop for (name type modifier) in gesture
-		       unless (eql modifier current-modifier)
-		       do (pushnew modifier other-modifiers)))
-	(when other-modifiers
-	  (setf other-modifiers (sort other-modifiers #'cmp-modifiers))
-	  (terpri pstream)
-	  (write-string "To see other commands, press "	pstream)
-	  (loop for modifier-tail on other-modifiers
-		for (modifier) = modifier-tail
-		for count from 0
-		do (progn
-		     (if (null (cdr modifier-tail))
-			 (progn
-			   (when (> count 1)
-			     (write-char #\, pstream))
-			   (when (> count 0)
-			     (write-string " or " pstream)))
-			 (when (> count 0)
-			   (write-string ", " pstream)))
-		     (print-modifiers pstream modifier :long)))
-	  (write-char #\. pstream))))))
+                              (stream-output-history stream)
+                              input-context
+                              *application-frame*
+                              stream
+                              x y
+                              :for-menu t))
+            (other-modifiers nil))
+        (loop for (translator) in all-translators
+              for gesture = (gesture translator)
+              unless (eq gesture t)
+              do (loop for (name type modifier) in gesture
+                       unless (eql modifier current-modifier)
+                       do (pushnew modifier other-modifiers)))
+        (when other-modifiers
+          (setf other-modifiers (sort other-modifiers #'cmp-modifiers))
+          (terpri pstream)
+          (write-string "To see other commands, press "	pstream)
+          (loop for modifier-tail on other-modifiers
+                for (modifier) = modifier-tail
+                for count from 0
+                do (progn
+                     (if (null (cdr modifier-tail))
+                         (progn
+                           (when (> count 1)
+                             (write-char #\, pstream))
+                           (when (> count 0)
+                             (write-string " or " pstream)))
+                         (when (> count 0)
+                           (write-string ", " pstream)))
+                     (print-modifiers pstream modifier :long)))
+          (write-char #\. pstream))))))
 
 (defmethod frame-update-pointer-documentation
     ((frame standard-application-frame) input-context stream event)
   (when *pointer-documentation-output*
     (with-accessors ((frame-documentation-state frame-documentation-state)
 		     (documentation-record documentation-record))
-      frame
+        frame
       (setf frame-documentation-state
 	    (frame-compute-pointer-documentation-state frame
 						       input-context
@@ -1202,63 +1242,54 @@ documentation produced by presentations.")
 	    (%event% event))
 	(declare (special %input-context% %stream% %doc-state% %event&))
 	(if (and documentation-record
-		 (output-record-parent documentation-record))
+                 (output-record-parent documentation-record))
 	    (redisplay documentation-record *pointer-documentation-output*)
 	    (progn
-	      (window-clear *pointer-documentation-output*)
 	      (setf documentation-record
-		    (updating-output (*pointer-documentation-output*)
-		      (updating-output (*pointer-documentation-output*
-					:cache-value %doc-state%
-					:cache-test
-					#'equal)
-			(frame-print-pointer-documentation frame
-							   %input-context%
-							   %stream%
-							   %doc-state%
-							   %event%))))))))))
-    
-#-(and)
-(defmethod frame-update-pointer-documentation
-    ((frame standard-application-frame) input-context stream event)
-  (when *pointer-documentation-output*
-    (with-accessors ((frame-documentation-state frame-documentation-state))
-	frame
-      (let ((new-state (frame-compute-pointer-documentation-state frame
-								  input-context
-								  stream
-								  event)))
-	(unless (frame-compare-pointer-documentation-state
-		 frame
-		 input-context
-		 stream
-		 frame-documentation-state
-		 new-state)
-	  (window-clear *pointer-documentation-output*)
-	  (frame-print-pointer-documentation frame
-					     input-context
-					     stream
-					     new-state
-					     event)
-	  (setq frame-documentation-state new-state))))))
+                    (updating-output (*pointer-documentation-output*)
+                      (updating-output (*pointer-documentation-output*
+                                        :cache-value %doc-state%
+                                        :cache-test #'equal)
+                        (frame-print-pointer-documentation frame
+                                                           %input-context%
+                                                           %stream%
+                                                           %doc-state%
+                                                           %event%))))))))))
+
+(defgeneric invoke-with-output-to-pointer-documentation (frame continuation)
+  (:documentation "Invoke `continuation' with a single argument -
+a stream that the continuation can write to, the output of which
+will be used as the background message of the pointer
+documentation pane of `frame'. If the pointer-documentation of
+`frame' is not a `pointer-documentation-pane', `continuation'
+will not be called."))
+
+(defmethod invoke-with-output-to-pointer-documentation
+    ((frame standard-application-frame) continuation)
+  (with-accessors ((pointer-documentation frame-pointer-documentation-output)) frame
+    (when (typep pointer-documentation 'pointer-documentation-pane)
+      (setf (background-message pointer-documentation)
+            (with-output-to-output-record (pointer-documentation)
+              (funcall continuation pointer-documentation))
+            (background-message-time pointer-documentation) (get-universal-time)))))
+
+(defmacro with-output-to-pointer-documentation ((stream frame) &body body)
+  "Bind `stream' to the pointer-documentation pane of `frame' and
+capture the output of `body' on `stream' as the background
+message of the pointer documentation pane. If `frame' does not
+have a `pointer-documentation-pane' as pointer documentation,
+`body' will not be evaluated."
+  `(invoke-with-output-to-pointer-documentation
+    ,frame #'(lambda (,stream)
+               ,@body)))
 
 ;;; A hook for applications to draw random strings in the
 ;;; *pointer-documentation-output* without screwing up the real pointer
 ;;; documentation too badly.
 
-(defgeneric frame-display-pointer-documentation-string
-    (frame documentation-stream string))
-
-(defmethod frame-display-pointer-documentation-string
-    ((frame standard-application-frame) documentation-stream string)
-  (when *pointer-documentation-output*
-    (with-accessors ((frame-documentation-state frame-documentation-state))
-        frame
-      (unless (frame-compare-pointer-documentation-state
-	       frame nil documentation-stream frame-documentation-state string)
-	(window-clear documentation-stream)
-	(write-string string documentation-stream)
-	(setq frame-documentation-state string)))))
+(defun frame-display-pointer-documentation-string (frame string)
+  (with-output-to-pointer-documentation (stream frame)
+    (write-string string stream)))
 
 (defmethod frame-input-context-track-pointer
     ((frame standard-application-frame)
