@@ -89,26 +89,29 @@
         (changed-line-style line-style-p)
         (changed-text-style text-style-p))
     (unwind-protect
-	(progn
+        (progn
           (when (eq ink old-ink) (setf ink nil))
           
-	  (if ink
+	  (when ink
 	      (setf (medium-ink medium) ink))
-	  (if transformation
+	  (when transformation
 	      (setf (medium-transformation medium)
 		(compose-transformations old-transform transformation)))
 
           (when (and clipping-region old-clip
-                     (region-equal clipping-region old-clip))
-            (setf clipping-region nil))                   
+                     (or (eq clipping-region +everywhere+)
+                         (eq clipping-region old-clip)
+                         (region-contains-region-p clipping-region old-clip))
+                     #+NIL (region-equal clipping-region old-clip))
+            (setf clipping-region nil))
 
-          (if clipping-region
-	      (setf (medium-clipping-region medium)
-		(region-intersection (if transformation
-                                         (transform-region transformation old-clip)
-                                       old-clip)
-				     clipping-region)))
-          (if (null line-style)              
+          (when clipping-region
+            (setf (medium-clipping-region medium)
+                  (region-intersection (if transformation
+                                           (transform-region transformation old-clip)
+                                           old-clip)
+                                       clipping-region)))
+          (when (null line-style)
               (setf line-style old-line-style))
 	  (when (or line-unit line-thickness dashes-p line-joint-shape line-cap-shape)
             (setf changed-line-style t)
@@ -128,7 +131,7 @@
 	  (if text-style-p
 	      (setf text-style (merge-text-styles text-style
 						  (medium-merged-text-style medium)))
-	    (setf text-style (medium-merged-text-style medium)))
+              (setf text-style (medium-merged-text-style medium)))
 	  (when (or text-family-p text-face-p text-size-p)
             (setf changed-text-style t)
             (setf text-style (merge-text-styles (make-text-style text-family
@@ -899,6 +902,23 @@ position for the character."
 			       align-x align-y
 			       toward-x toward-y transform-glyphs))
 
+;;; Some image junk...
+
+(defmethod medium-free-image-design ((sheet sheet-with-medium-mixin) design)
+  (medium-free-image-design (sheet-medium sheet) design))
+
+(defmethod medium-draw-image-design* :before (current-medium design x y)
+  (with-slots (medium medium-data) design
+    (unless (eq medium current-medium)
+      (when medium
+	(medium-free-image-design medium design))
+      (setf medium current-medium)
+      (setf medium-data nil))))
+
+(defmethod medium-draw-image-design*
+    ((medium sheet-with-medium-mixin) design x y)
+  (medium-draw-image-design* (sheet-medium medium) design x y))
+
 ;;;;
 ;;;; DRAW-DESIGN
 ;;;;
@@ -992,6 +1012,19 @@ position for the character."
 
 ;;;;
 
+(defmethod draw-design
+    (medium (design rgb-image-design) &rest options
+     &key (x 0) (y 0) &allow-other-keys)
+  (with-medium-options (medium options)
+    (medium-draw-image-design* medium design x y)))
+
+;;;;
+
+(defmethod draw-design (medium (pattern pattern) &key clipping-region transformation
+                        &allow-other-keys)
+  (draw-pattern* medium pattern 0 0
+   :clipping-region clipping-region :transformation transformation))
+
 (defun draw-pattern* (medium pattern x y &key clipping-region transformation)
   ;; Note: I believe the sample implementation in the spec is incorrect.
   ;; --GB
@@ -1093,3 +1126,92 @@ position for the character."
                                   :radius-left :radius-right
                                   :radius-top  :radius-bottom))
      args)))
+
+;;; Bitmap images
+;;;
+;;; Based on CLIM 2.2, with an extension permitting the definition of
+;;; new image formats by the user.
+
+(defvar *bitmap-file-readers* (make-hash-table :test 'equalp)
+  "A hash table mapping keyword symbols naming bitmap image
+formats to a function that can read an image of that format. The
+functions will be called with one argument, the pathname of the
+file to be read. The functions should return two values as per
+`read-bitmap-file'.")
+
+(defmacro define-bitmap-file-reader (bitmap-format (&rest args) &body body)
+  "Define a method for reading bitmap images of format
+BITMAP-FORMAT that will be used by `read-bitmap-file' and
+MAKE-PATTERN-FROM-BITMAP-FILE. BODY should return two values as
+per `read-bitmap-file'."
+  `(setf (gethash ,bitmap-format *bitmap-file-readers*)
+         #'(lambda (,@args)
+             ,@body)))
+
+(defun bitmap-format-supported-p (format)
+  "Return true if FORMAT is supported by `read-bitmap-file'."
+  (not (null (gethash format *bitmap-file-readers*))))
+
+(define-condition unsupported-bitmap-format (error)
+  ((%format :reader bitmap-format
+            :initarg :bitmap-format
+            :initform (error "The bitmap format must be supplied")
+            :documentation "The bitmap format that cannot be loaded"))
+  (:report (lambda (condition stream)
+             (format
+              stream "Cannot read bitmap of unknown format \"~A\""
+              (bitmap-format condition))))
+  (:documentation "This exception is signalled when
+`read-bitmap-file' is called on an bitmap of a type that no reader
+has been defined for."))
+
+(defun unsupported-bitmap-format (format)
+  "Signal an error of type `unsupported-bitmap-format' for the
+bitmap format `format'."
+  (error 'unsupported-bitmap-format :bitmap-format format))
+
+(defun read-bitmap-file (pathname &key (format :bitmap) (port (find-port)))
+  "Read a bitmap file named by `pathname'. `Port' specifies the
+port that the bitmap is to be used on. `Format' is a keyword
+symbol naming any defined bitmap file format defined by
+`clim-extensions:define-bitmap-file-reader'. Two values are
+returned: a two-dimensional array of pixel values and an array of
+either colors or color names. If the second value is non-NIL, the
+pixel values are assumed to be indexes into this
+array. Otherwise, the pixel values are taken to be RGB values
+encoded in 32 bit unsigned integers, with the three most
+significant octets being the values R, G and B, in order."
+  (declare (ignore port)) ; XXX?
+  (funcall (or (gethash format *bitmap-file-readers*)
+               (unsupported-bitmap-format format))
+           pathname))
+
+(defun make-pattern-from-bitmap-file (pathname &key designs
+                                      (format :bitmap) (port (find-port)))
+  "Read a bitmap file named by `pathname'. `Port' specifies the
+port that the bitmap is to be used on. `Format' is a keyword
+symbol naming any defined bitmap file format defined by
+`clim-extensions:define-bitmap-file-reader'. Two values are
+returned: a two-dimensional array of pixel values and an array of
+either colors or color names. If the second value is non-NIL, the
+pixel values are assumed to be indexes into this
+array. Otherwise, the pixel values are taken to be RGB values
+encoded in 32 bit unsigned integers, with the three most
+significant octets being the values R, G and B, in order."
+  (multiple-value-bind (res read-designs)
+      (read-bitmap-file pathname :format format :port port)
+    (if read-designs
+        (make-pattern res (or designs read-designs))
+        (make-instance 'rgb-pattern :image (make-instance 'rgb-image
+                                            :width (array-dimension res 0)
+                                            :height (array-dimension res 1)
+                                            :data res)))))
+
+(define-bitmap-file-reader :xpm (pathname)
+  (xpm-parse-file pathname))
+
+(define-bitmap-file-reader :pixmap (pathname)
+  (read-bitmap-file pathname :format :xpm))
+
+(define-bitmap-file-reader :pixmap-3 (pathname)
+  (read-bitmap-file pathname :format :xpm))
