@@ -2,7 +2,7 @@
 
 ;;;  (c) copyright 2001 by 
 ;;;           Tim Moore (moore@bricoworks.com)
-;;;  (c) copyright 2006 by
+;;;  (c) copyright 2006-2008 by
 ;;;           Troels Henriksen (athas@sigkill.dk)
 
 ;;; This library is free software; you can redistribute it and/or
@@ -31,10 +31,53 @@
   "I true, use the Goatee editing component instead of Drei. The
 Goatee component is faster and more mature than Drei.")
 
-(defvar *activation-gestures* nil)
-(defvar *standard-activation-gestures* '(:newline :return))
+(defvar *activation-gestures* nil
+  "The set of currently active activation gestures. The global
+value of this must be NIL. The exact format of
+`*activation-gestures*' is unspecified. `*activation-gestures*'
+and the elements in it may have dynamic extent.")
 
-(defvar *delimiter-gestures* nil)
+(defvar *standard-activation-gestures* '(:newline :return)
+  "The default set of activation gestures. The exact set of
+standard activation is unspecified, but must include the gesture
+that corresponds to the #\Newline character. ")
+
+(defvar *delimiter-gestures* nil
+  "The set of currently active delimiter gestures. The global
+value of this must be NIL. The exact format of
+`*delimiter-gestures*' is unspecified. `*delimiter-gestures*' and
+the elements in it may have dynamic extent.")
+
+(with-system-redefinition-allowed
+  (when (and (fboundp 'interactive-stream-p)
+             (not (typep (fdefinition 'interactive-stream-p)
+                         'generic-function)))
+    (fmakunbound 'interactive-stream-p))
+  (defgeneric interactive-stream-p (stream)
+    (:method (stream)
+      (cl:interactive-stream-p stream))))
+
+(defclass standard-input-editing-mixin ()
+  ((%typeout-record :accessor typeout-record
+                    :initform nil
+                    :documentation "The output record (if any)
+that is the typeout information for this
+input-editing-stream. `With-input-editor-typeout' manages this
+output record."))
+  (:documentation "A mixin implementing some useful standard
+behavior for input-editing streams."))
+
+(defmethod typeout-record :around ((stream standard-input-editing-mixin))
+  ;; Can't do this in an initform, since we need to proper position...
+  (or (call-next-method)
+      (let ((record
+             (make-instance 'standard-sequence-output-record
+              :x-position 0
+              :y-position (bounding-rectangle-min-y
+                           (input-editing-stream-output-record stream)))))
+        (stream-add-output-record (encapsulating-stream-stream stream)
+                                  record)
+        (setf (typeout-record stream) record))))
 
 ;;; These helper functions take the arguments of ACCEPT so that they
 ;;; can be used directly by ACCEPT.
@@ -63,6 +106,19 @@ Goatee component is faster and more mature than Drei.")
 	(t existing-delimiter-gestures)))
 
 (defmacro with-activation-gestures ((gestures &key override) &body body)
+  "Specifies a list of gestures that terminate input during the
+execution of `body'. `Body' may have zero or more declarations as
+its first forms. `Gestures' must be either a single gesture name
+or a form that evaluates to a list of gesture names.
+
+If the boolean `override' is true, then `gestures' will override
+the current activation gestures. If it is false (the default),
+then gestures will be added to the existing set of activation
+gestures. `with-activation-gestures' must bind
+`*activation-gestures*' to the new set of activation gestures.
+
+See also the `:activation-gestures' and
+`:additional-activation-gestures' options to `accept'."
   ;; XXX Guess this implies that gestures need to be defined at
   ;; compile time.  Sigh.  We permit both CLIM 2.0-style gesture names
   ;; and CLIM 2.2 style characters.
@@ -83,6 +139,21 @@ Goatee component is faster and more mature than Drei.")
        ,@body)))
 
 (defmacro with-delimiter-gestures ((gestures &key override) &body body)
+  "Specifies a list of gestures that terminate an individual
+token, but not the entire input, during the execution of
+`body'. `Body' may have zero or more declarations as its first
+forms. `Gestures' must be either a single gesture name or a form
+that evaluates to a list of gesture names.
+
+If the boolean `override' is true, then `gestures' will override
+the current delimiter gestures. If it is false (the default),
+then gestures will be added to the existing set of delimiter
+gestures. `With-delimiter-gestures' must bind
+`*delimiter-gestures*' to the new set of delimiter
+gestures.
+
+See also the `:delimiter-gestures' and
+`:additional-delimiter-gestures' options to `accept'."
   ;; XXX Guess this implies that gestures need to be defined at
   ;; compile time.  Sigh.  We permit both CLIM 2.0-style gesture names
   ;; and CLIM 2.2 style characters.
@@ -103,22 +174,142 @@ Goatee component is faster and more mature than Drei.")
        ,@body)))
 
 (defun activation-gesture-p (gesture)
+  "Returns true if the gesture object `gesture' is an activation
+gesture, otherwise returns false."
   (loop for gesture-name in *activation-gestures*
 	when (gesture-matches-spec-p gesture gesture-name)
 	do (return t)
 	finally (return nil)))
 
 (defun delimiter-gesture-p (gesture)
+  "Returns true if the gesture object `gesture' is a delimiter
+gesture, otherwise returns false."
   (loop for gesture-name in *delimiter-gestures*
 	when (gesture-matches-spec-p gesture gesture-name)
 	do (return t)
 	finally (return nil)))
 
+(defmacro with-input-editor-typeout ((&optional (stream t) &rest args
+                                                &key erase)
+                                     &body body)
+  "Clear space above the input-editing stream `stream' and
+evaluate `body', capturing output done to `stream'. Place will be
+obtained above the input-editing area and the output put
+there. Nothing will be displayed until `body' finishes. `Stream'
+is not evaluated and must be a symbol. If T (the default),
+`*standard-input*' will be used. `Stream' will be bound to an
+`extended-output-stream' while `body' is being evaluated."
+  (declare (ignore erase))
+  (check-type stream symbol)
+  (let ((stream (if (eq stream t) '*standard-output* stream)))
+    `(invoke-with-input-editor-typeout
+      ,stream
+      #'(lambda (,stream)
+          ,@body)
+      ,@args)))
+
+(defgeneric invoke-with-input-editor-typeout (stream continuation &key erase)
+  (:documentation "Call `continuation' with a single argument, a
+stream to do input-editor-typeout on."))
+
+(defun sheet-move-output-vertically (sheet y delta-y)
+  "Move the output records of `sheet', starting at vertical
+device unit offset `y' or below, down by `delta-y' device units,
+then repaint `sheet'."
+  (unless (zerop delta-y)
+    (with-bounding-rectangle* (sheet-x1 sheet-y1 sheet-x2 sheet-y2) sheet
+      (declare (ignore sheet-x1 sheet-y1))
+      (map-over-output-records-overlapping-region
+       #'(lambda (record)
+           (multiple-value-bind (record-x record-y) (output-record-position record)
+             (when (> (+ record-y (bounding-rectangle-height record)) y)
+               (setf (output-record-position record)
+                     (values record-x (+ record-y delta-y))))))
+       (stream-output-history sheet)
+       (make-bounding-rectangle 0 y sheet-x2 sheet-y2))
+      ;; Only repaint within the visible region...
+      (with-bounding-rectangle* (viewport-x1 viewport-y1 viewport-x2 viewport-y2)
+          (or (pane-viewport-region sheet) sheet)
+        (declare (ignore viewport-y1))
+        (repaint-sheet sheet (make-bounding-rectangle viewport-x1 (- y (abs delta-y))
+                                                      viewport-x2 viewport-y2))))))
+
+(defmethod invoke-with-input-editor-typeout ((editing-stream standard-input-editing-mixin)
+                                             (continuation function) &key erase)
+  (with-accessors ((stream-typeout-record typeout-record)) editing-stream
+    ;; Can't do this in an initform, as we need to set the proper
+    ;; output record position.
+    (let* ((encapsulated-stream (encapsulating-stream-stream editing-stream))
+           (old-min-y (bounding-rectangle-min-y stream-typeout-record))
+           (old-height (bounding-rectangle-height stream-typeout-record))
+           (new-typeout-record (with-output-to-output-record (encapsulated-stream
+                                                              'standard-sequence-output-record
+                                                              record)
+                                 (unless erase
+                                   ;; Steal the children of the old typeout record.
+                                   (map nil #'(lambda (child)
+                                                (setf (output-record-parent child) nil
+                                                      (output-record-position child) (values 0 0))
+                                                (add-output-record child record))
+                                        (output-record-children stream-typeout-record))
+                                   ;; Make sure new output is done
+                                   ;; after the stolen children.
+                                   (stream-increment-cursor-position
+                                    encapsulated-stream 0 old-height))
+                                 (funcall continuation encapsulated-stream))))
+      (with-sheet-medium (medium encapsulated-stream)
+        (setf (output-record-position new-typeout-record) (values 0 old-min-y))
+        ;; Calculate the height difference between the old typeout and the new.
+        (let ((delta-y (- (bounding-rectangle-height new-typeout-record) old-height)))
+          (multiple-value-bind (typeout-x typeout-y)
+              (output-record-position new-typeout-record)
+            (declare (ignore typeout-x))
+            ;; Clear the old typeout...
+            (clear-output-record stream-typeout-record)
+            ;; Move stuff for the new typeout record...
+            (sheet-move-output-vertically encapsulated-stream typeout-y delta-y)
+            ;; Reuse the old stream-typeout-record...
+            (add-output-record new-typeout-record stream-typeout-record)
+            ;; Now, let there be light!
+            (repaint-sheet encapsulated-stream stream-typeout-record)))))))
+
+(defun clear-typeout (&optional (stream t))
+  "Blank out the input-editor typeout displayed on `stream',
+defaulting to T for `*standard-output*'."
+  (with-input-editor-typeout (stream :erase t)
+    (declare (ignore stream))))
+
 (defmacro with-input-editing ((&optional (stream t)
 			       &rest args
 			       &key input-sensitizer (initial-contents "")
 			       (class ''standard-input-editing-stream class-provided-p))
 			      &body body)
+  "Establishes a context in which the user can edit the input
+typed in on the interactive stream `stream'. `Body' is then
+executed in this context, and the values returned by `body' are
+returned as the values of `with-input-editing'. `Body' may have
+zero or more declarations as its first forms.
+
+The stream argument is not evaluated, and must be a symbol that
+is bound to an input stream. If stream is T (the default),
+`*standard-input*' is used. If stream is a stream that is not an
+interactive stream, then `with-input-editing' is equivalent to
+progn.
+
+`input-sensitizer', if supplied, is a function of two arguments,
+a stream and a continuation function; the function has dynamic
+extent. The continuation, supplied by CLIM, is responsible for
+displaying output corresponding to the user's input on the
+stream. The input-sensitizer function will typically call
+`with-output-as-presentation' in order to make the output
+produced by the continuation sensitive.
+
+If `initial-contents' is supplied, it must be either a string or
+a list of two elements, an object and a presentation type. If it
+is a string, the string will be inserted into the input buffer
+using `replace-input'. If it is a list, the printed
+representation of the object will be inserted into the input
+buffer using `presentation-replace-input'."
   (setq stream (stream-designator-symbol stream '*standard-input*))
   (with-keywords-removed (args (:input-sensitizer :initial-contents :class))
     `(invoke-with-input-editing ,stream
@@ -141,25 +332,71 @@ Goatee component is faster and more mature than Drei.")
        ,@body)))
 
 (defun input-editing-rescan-loop (editing-stream continuation)
-  (loop
-   (block rescan
-     (handler-bind ((rescan-condition
-                     #'(lambda (c)
-                         (declare (ignore c))
-                         (reset-scan-pointer editing-stream)
-                         (return-from rescan nil))))
-       (return-from input-editing-rescan-loop
-	 (funcall continuation editing-stream))))))
+  (let ((start-scan-pointer (stream-scan-pointer editing-stream)))
+    (loop (block rescan
+            (handler-bind ((rescan-condition
+                            #'(lambda (c)
+                                (declare (ignore c))
+                                (reset-scan-pointer editing-stream start-scan-pointer)
+                                ;; Input-editing contexts above may be interested...
+                                (return-from rescan nil))))
+              (return-from input-editing-rescan-loop
+                (funcall continuation editing-stream)))))))
+
+(defgeneric finalize (editing-stream input-sensitizer)
+  (:documentation "Do any cleanup on an editing stream that is no
+longer supposed to be used for editing, like turning off the
+cursor, etc."))
+
+(defmethod finalize ((stream input-editing-stream) input-sensitizer)
+  (clear-typeout stream)
+  (redraw-input-buffer stream))
+
+(defgeneric invoke-with-input-editing
+    (stream continuation input-sensitizer initial-contents class)
+  (:documentation "Implements `with-input-editing'. `Class' is
+the class of the input-editing stream to create, if necessary."))
 
 (defmethod invoke-with-input-editing
     (stream continuation input-sensitizer initial-contents class)
   (declare (ignore input-sensitizer initial-contents class))
   (funcall continuation stream))
 
-(defgeneric input-editing-stream-bounding-rectangle (stream)
-  (:documentation "Return the bounding rectangle of `stream' as
-four values. This function does not appear in the spec but is
-used by the command processing code for layout."))
+(defmethod invoke-with-input-editing ((stream input-editing-stream)
+                                      continuation input-sensitizer
+                                      initial-contents class)
+  (unless (stream-rescanning-p stream)
+    (if (stringp initial-contents)
+        (replace-input stream initial-contents)
+        (presentation-replace-input stream
+                                    (first initial-contents)
+                                    (second initial-contents)
+                                    (stream-default-view stream))))
+  (call-next-method))
+
+(defmethod invoke-with-input-editing :around ((stream extended-output-stream)
+					      continuation
+					      input-sensitizer
+					      initial-contents
+					      class)
+  (declare (ignore continuation input-sensitizer initial-contents class))
+  (letf (((cursor-visibility (stream-text-cursor stream)) nil))
+    (call-next-method)))
+
+(defmethod invoke-with-input-editing :around (stream
+					      continuation
+					      input-sensitizer
+					      initial-contents
+					      class)
+  (declare (ignore continuation input-sensitizer initial-contents class))
+  (with-activation-gestures (*standard-activation-gestures*)
+    (call-next-method)))
+
+(defgeneric input-editing-stream-output-record (stream)
+  (:documentation "Return the output record showing the display of the
+input-editing stream `stream' values. This function does not
+appear in the spec but is used by the command processing code for
+layout and to implement a general with-input-editor-typeout."))
 
 (defmethod input-editor-format ((stream t) format-string &rest format-args)
   (unless (and (typep stream '#.*string-input-stream-class*)
@@ -184,6 +421,21 @@ used by the command processing code for layout."))
 		   (pointer-button-press-handler
 		    *pointer-button-press-handler*)
 		   click-only)
+  "Reads characters from the interactive stream `stream' until it
+encounters a delimiter or activation gesture, or a pointer
+gesture. Returns the accumulated string that was delimited by the
+delimiter or activation gesture, leaving the delimiter
+unread.
+
+If the first character of typed input is a quotation mark (#\"),
+then `read-token' will ignore delimiter gestures until another
+quotation mark is seen. When the closing quotation mark is seen,
+`read-token' will proceed as above.
+
+`Click-only' is ignored for now.
+
+`Input-wait-handler' and `pointer-button-press-handler' are as
+for 34stream-read-gesture"
   (declare (ignore click-only))		;XXX For now
   (let ((result (make-array 1
 			    :adjustable t
@@ -222,6 +474,15 @@ used by the command processing code for layout."))
 		    (return (subseq result 0))))))
 
 (defun write-token (token stream &key acceptably)
+  "This function is the opposite of `read-token' given the string
+token, it writes it to the interactive stream stream. If
+`acceptably' is true and there are any characters in the token
+that are delimiter gestures (see the macro
+`with-delimiter-gestures'), then `write-token' will surround the
+token with quotation marks (#\").
+
+Typically, `present' methods will use `write-token' instead of
+`write-string'."
   (let ((put-in-quotes (and acceptably (some #'delimiter-gesture-p token))))
     (when put-in-quotes
       (write-char #\" stream))
@@ -232,9 +493,18 @@ used by the command processing code for layout."))
 ;;; Signalling Errors Inside present (sic)
 
 (define-condition simple-parse-error (simple-condition parse-error)
-  ())
+  ()
+  (:documentation "The error that is signalled by
+`simple-parse-error'. This is a subclass of `parse-error'.
+
+This condition handles two initargs, `:format-string' and
+`:format-arguments', which are used to specify a control string
+and arguments for a call to `format'."))
 
 (defun simple-parse-error (format-string &rest format-args)
+  "Signals a `simple-parse-error' error while parsing an input
+token. Does not return. `Format-string' and `format-args' are as
+for format."
   (error 'simple-parse-error
 	 :format-control format-string :format-arguments format-args))
 
@@ -244,20 +514,50 @@ used by the command processing code for layout."))
   (:report (lambda (condition stream)
 	     (format stream "Input ~S is not of required type ~S"
 		     (not-required-type-string condition)
-		     (not-required-type-type condition)))))
+		     (not-required-type-type condition))))
+  (:documentation "The error that is signalled by
+`input-not-of-required-type'. This is a subclass of
+`parse-error'.
+
+This condition handles two initargs, `:string' and `:type', which
+specify a string to be used in an error message and the expected
+presentation type."))
 
 (defun input-not-of-required-type (object type)
+  "Reports that input does not satisfy the specified type by
+signalling an `input-not-of-required-type' error. `Object' is a
+parsed object or an unparsed token (a string). `Type' is a
+presentation type specifier. Does not return."
   (error 'input-not-of-required-type :string object :type type))
 
 ;;; 24.5 Completion
 
-(defvar *completion-gestures* '(:complete))
-(defvar *help-gestures* '(:help))
-(defvar *possibilities-gestures* '(:possibilities))
+(defvar *completion-gestures* '(:complete)
+  "A list of the gesture names that cause `complete-input' to
+complete the user's input as fully as possible. The exact global
+contents of this list is unspecified, but must include the
+`:complete' gesture name.")
+
+(defvar *help-gestures* '(:help)
+  "A list of the gesture names that cause `accept' and
+`complete-input' to display a (possibly input context-sensitive)
+help message, and for some presentation types a list of
+possibilities as well. The exact global contents of this list is
+unspecified, but must include the `:help' gesture name.")
+
+(defvar *possibilities-gestures* '(:possibilities)
+  "A list of the gesture names that cause `complete-input' to
+display a (possibly input context-sensitive) help message and a
+list of possibilities. The exact global contents of this list is
+unspecified, but must include the `:possibilities' gesture
+name.")
 
 (define-condition simple-completion-error (simple-parse-error)
   ((input-so-far :reader completion-error-input-so-far
-		 :initarg :input-so-far)))
+		 :initarg :input-so-far))
+  (:documentation "The error that is signalled by
+`complete-input' when no completion is found. This is a subclass
+of `simple-parse-error'."))
 
 ;;; wrapper around event-matches-gesture-name-p to match against characters too.
 
@@ -308,11 +608,9 @@ used by the command processing code for layout."))
                          (unread-gesture gesture :stream stream)
                          (return-from complete-input-rescan
                            (values object t input)))
-                       ;; Do we actually want to signal this here? I
-                       ;; don't think we should terminate the
-                       ;; input-editing-session just because the user
-                       ;; tries to complete invalid input.
-		       (error 'simple-completion-error
+                       ;; This used to be an error, but no one thought
+                       ;; that was a really great idea.
+		       (signal 'simple-completion-error
                               :format-control "complete-input: While rescanning,~
                                              can't match ~A~A"
                               :format-arguments (list so-far  gesture)
@@ -325,12 +623,32 @@ used by the command processing code for layout."))
   nil)
 
 (defun possibilities-for-menu (possibilities)
-  (loop for p in possibilities
-	for (display . object) = p
-	if (listp object)
-	  collect `(,display :value ,object)
-	else
-	  collect p))
+  (loop for (display object) in possibilities
+        collect `(,display :value ,object)))
+
+(defun possibility-printer (possibility ptype stream)
+  "A default function for printing a possibility. Suitable for
+used as value of `:possibility-printer' in calls to
+`complete-input'"
+  (with-output-as-presentation (stream possibility ptype)
+    (write-string (first possibility) stream)))
+
+(defun print-possibilities (possibilities possibility-printer stream)
+  "Write `possibitilies' to `stream', using
+`possibility-printer'. `Possibilities' must be a list of
+input-completion possibilities. `Stream' must be an input-editing
+stream. Output will be done to its typeout."
+  (with-input-editor-typeout (stream :erase t)
+    (surrounding-output-with-border (stream :shape :drop-shadow :background +cornsilk1+)
+      (surrounding-output-with-border (stream :shape :rectangle)
+        (let ((ptype `(completion ,possibilities)))
+          (format-items possibilities
+           :stream stream
+           :printer #'(lambda (possibility stream)
+                        (funcall possibility-printer
+                                 possibility
+                                 ptype
+                                 stream))))))))
 
 ;;; Helper returns gesture (or nil if gesture shouldn't be part of the input)
 ;;; and completion mode, if any.
@@ -369,9 +687,9 @@ used by the command processing code for layout."))
 (defparameter *trace-complete-input* nil)
 
 (defun complete-input (stream func &key
-		       partial-completers allow-any-input possibility-printer
+		       partial-completers allow-any-input
+                       (possibility-printer #'possibility-printer)
 		       (help-displays-possibilities t))
-  (declare (ignore possibility-printer))
   (let ((so-far (make-array 1 :element-type 'character :adjustable t
 			    :fill-pointer 0))
 	(*accelerator-gestures* (append *help-gestures*
@@ -415,14 +733,18 @@ used by the command processing code for layout."))
                      (format *trace-output* "nmatches = ~A, mode = ~A~%"
                              nmatches mode))
                    (when (and (> nmatches 0) (eq mode :possibilities))
-                     (multiple-value-bind (menu-object item event)
-                         (menu-choose (possibilities-for-menu possibilities)
-                                      :label "Possibilities"
-                                      :n-columns 1)
-                       (declare (ignore event))
-                       (if item
+                     (print-possibilities possibilities possibility-printer stream)
+                     (redraw-input-buffer stream)
+                     (let ((possibility
+                            (handler-case
+                                (with-input-context (`(completion ,possibilities) :override nil)
+                                    (object type event)
+                                    (prog1 nil (read-gesture :stream stream :peek-p t))
+                                  (t object))
+                              (abort-gesture () nil))))
+                       (if possibility
                            (setf (values input success object nmatches)
-                                 (values (car item) t menu-object 1))
+                                 (values (first possibility) t (second possibility) 1))
                            (setf success nil
                                  nmatches 0))))
                    (unless (and (eq mode :complete) (not success))
@@ -556,7 +878,7 @@ used by the command processing code for layout."))
 			   (length initial-string))
 		       initial-len)
 	       (incf nmatches)
-	       (push (cons str obj) possibilities))))
+	       (push (list str obj) possibilities))))
       (funcall generator initial-string #'suggester)
       (if (and (eql nmatches 1)
 	       (string-equal initial-string (caar possibilities)))
@@ -572,7 +894,7 @@ used by the command processing code for layout."))
 				    (action :complete)
 				    (predicate (constantly t))
 				    (name-key #'car)
-				    (value-key #'cadr))
+				    (value-key #'second))
   (flet ((generator (input-string suggester)
 	   (declare (ignore input-string))
 	   (do-sequence (possibility completions)
@@ -584,11 +906,34 @@ used by the command processing code for layout."))
 			     :predicate predicate)))
 
 (defun suggest (completion object)
+  "Specifies one possibility for
+`completing-from-suggestions'. `Completion' is a string, the
+printed representation of object. `Object' is the internal
+representation.
+
+Calling this function outside of the body of
+`completing-from-suggestions' is an error."
   (declare (ignore completion object))
   (error
    "SUGGEST called outside of lexical scope of COMPLETING-FROM-SUGGESTIONS" ))
 
 (defmacro completing-from-suggestions ((stream &rest args) &body body)
+  "Reads input from the input editing stream `stream', completing
+over a set of possibilities generated by calls to `suggest'
+within `body'. `Body' may have zero or more declarations as its
+first forms.
+
+`Completing-from-suggestions' returns three values, `object',
+`success', and `string'.
+
+The stream argument is not evaluated, and must be a symbol that
+is bound to a stream. If `stream' t is (the default),
+`*standard-input*' is used. `Partial-completers',
+`allow-any-input', and `possibility-printer' are as for
+`complete-input'.
+
+Implementations will probably use `complete-from-generator' to
+implement this."
   (when (eq stream t)
     (setq stream '*standard-input*))
   (let ((generator (gensym "GENERATOR"))
