@@ -392,40 +392,6 @@ otherwise return false."
     (type-key parameters options object type stream view
      &key &allow-other-keys))
 
-(defmacro with-output-as-presentation ((stream object type
-					       &rest key-args
-					       &key modifier single-box
-					       (allow-sensitive-inferiors t)
-					       parent
-					       (record-type
-						''standard-presentation)
-					       &allow-other-keys)
-				       &body body)
-  (declare (ignore parent single-box modifier))
-  (setq stream (stream-designator-symbol stream '*standard-output*))
-  (multiple-value-bind (decls with-body)
-      (get-body-declarations body)
-    (with-gensyms (record-arg continuation)
-      (with-keywords-removed (key-args (:record-type
-					:allow-sensitive-inferiors))
-	`(flet ((,continuation ()
-		  ,@decls
-		  ,@with-body))
-	   (declare (dynamic-extent #'continuation))
-	   (if (and (output-recording-stream-p ,stream)
-		    *allow-sensitive-inferiors*)
-	       (with-new-output-record
-		   (,stream ,record-type ,record-arg
-			    :object ,object
-			    :type (expand-presentation-type-abbreviation
-				   ,type)
-			    ,@key-args)
-		 (let ((*allow-sensitive-inferiors*
-			,allow-sensitive-inferiors))
-		   (,continuation)))
-	       (,continuation)))))))
-
-
 (defun present (object &optional (type (presentation-type-of object))
 		&key
 		(stream *standard-output*)
@@ -719,122 +685,6 @@ history will be unchanged."
     (unless (and top-ptype (eql object top-object) (equal ptype top-ptype))
       (presentation-history-insert history object ptype))))
 
-;;; Context-dependent input
-;;; An input context is a cons of a presentation type and a continuation to
-;;; call to return a presentation to that input context.
-
-(defvar *input-context* nil)
-
-(defun input-context-type (context-entry)
-  (car context-entry))
-
-;;; Many presentation functions, internal and external, take an input
-;;; context as an argument, but they really only need to look at one
-;;; presentation type.
-(defun make-fake-input-context (ptype)
-  (list (cons (expand-presentation-type-abbreviation ptype)
-	      #'(lambda (object type event options)
-		  (declare (ignore event options))
-		  (error "Fake input context called with object ~S type ~S. ~
-                          This shouldn't happen!"
-			 object type)))))
-
-(defun input-context-wait-test (stream)
-  (let* ((queue (stream-input-buffer stream))
-	 (event (event-queue-peek queue)))
-    (when event
-      (let ((sheet (event-sheet event)))
-	(when (and (output-recording-stream-p sheet)
-		   (or (typep event 'pointer-event)
-		       (typep event 'keyboard-event))
-		   (not (gadgetp sheet)))
-	  (return-from input-context-wait-test t))))
-    nil))
-
-(defun highlight-applicable-presentation (frame stream input-context
-					  &optional (prefer-pointer-window t))
-  (let* ((queue (stream-input-buffer stream))
-	 (event (event-queue-peek queue)))
-    (when (and event
-	       (or (and (typep event 'pointer-event)
-			(or prefer-pointer-window 
-			    (eq stream (event-sheet event))))
-		   (typep event 'keyboard-event)))
-      ;; Stream only needs to see button press events.
-      ;; XXX Need to think about this more.  Should any pointer events be
-      ;; passed through?  If there's no presentation, maybe?
-      (unless (typep event 'keyboard-event)
-	(event-queue-read queue))
-      (progn
-	(frame-input-context-track-pointer frame
-					   input-context
-					   (event-sheet event)
-					   event)
-	(when (typep event 'pointer-button-press-event)
-	  (funcall *pointer-button-press-handler* stream event)))
-      #+nil
-      (if (and (typep event 'pointer-motion-event)
-	       (pointer-event-button event))
-	  (frame-drag frame input-context (event-sheet event) event)
-	  ))))
-
-(defun input-context-event-handler (stream)
-  (highlight-applicable-presentation *application-frame*
-				     stream
-				     *input-context*))
-
-(defun input-context-button-press-handler (stream button-event)
-  (declare (ignore stream))
-  (frame-input-context-button-press-handler *application-frame*
-					    (event-sheet button-event)
-					    button-event))
-
-(defun highlight-current-presentation (frame input-context)
-  (let ((event (synthesize-pointer-motion-event (port-pointer
-						 (port
-						  *application-frame*)))))
-    (when event
-      (frame-input-context-track-pointer *application-frame*
-				       *input-context*
-				       (event-sheet event)
-				       event))))
-
-(defmacro with-input-context ((type &key override)
-			      (&optional (object-var (gensym))
-					 (type-var (gensym))
-					 event-var
-					 options-var)
-			      form
-			      &body pointer-cases)
-  (let ((vars `(,object-var
-		,type-var
-		,@(and event-var `(,event-var))
-		,@(and options-var `(,options-var))))
-	(return-block (gensym "RETURN-BLOCK"))
-	(context-block (gensym "CONTEXT-BLOCK")))
-    `(block ,return-block
-       (multiple-value-bind ,vars
-	   (block ,context-block
-	     (let ((*input-context*
-		    (cons (cons (expand-presentation-type-abbreviation ,type)
-                                #'(lambda (object type event options)
-				    (return-from ,context-block
-				      (values object type event options))))
-			  ,(if override nil '*input-context*)))
-		   (*pointer-button-press-handler*
-		    #'input-context-button-press-handler)
-		   (*input-wait-test* #'input-context-wait-test)
-		   (*input-wait-handler* #'input-context-event-handler))
-	       (return-from ,return-block ,form )))
-         (declare (ignorable ,@vars))
-	 (highlight-current-presentation *application-frame* *input-context*)
-	 (cond ,@(mapcar #'(lambda (pointer-case)
-			     (destructuring-bind (case-type &body case-body)
-				 pointer-case
-			       `((presentation-subtypep ,type-var ',case-type)
-				 ,@case-body)))
-			 pointer-cases))))))
-
 (define-presentation-generic-function %accept accept
     (type-key parameters options type stream view &key))
 
@@ -1114,6 +964,103 @@ history will be unchanged."
   (declare (ignore type view other-args))
   nil)
 
+;;; For ACCEPT-FROM-STRING, use this barebones input-editing-stream.
+(defclass string-input-editing-stream (input-editing-stream fundamental-character-input-stream)
+  ((input-buffer :accessor stream-input-buffer)
+   (insertion-pointer :accessor stream-insertion-pointer
+                      :initform 0
+                      :documentation "This is not used for anything at any point.")
+   (scan-pointer :accessor stream-scan-pointer
+                 :initform 0
+                 :documentation "This is not used for anything at any point."))
+  (:documentation "An implementation of the input-editing stream
+protocol retrieving gestures from a provided string."))
+
+(defmethod initialize-instance :after ((stream string-input-editing-stream)
+                                       &key (string (error "A string must be provided"))
+                                       (start 0) (end (length string))
+                                       &allow-other-keys)
+  (setf (stream-input-buffer stream)
+        (replace (make-array (- end start) :fill-pointer (- end start))
+                 string :start1 start :end2 end)))
+
+(defmethod stream-element-type ((stream string-input-editing-stream))
+  'character)
+
+(defmethod close ((stream string-input-editing-stream) &key abort)
+  (declare (ignore abort)))
+
+(defmethod stream-peek-char ((stream string-input-editing-stream))
+  (let ((char (read-char-no-hang stream nil nil)))
+    (when char
+      (unread-char char stream))
+    (or char :eof)))
+
+(defmethod stream-read-char-no-hang ((stream string-input-editing-stream))
+  (if (> (stream-scan-pointer stream) (length (stream-input-buffer stream)))
+   :eof
+   (stream-read-gesture stream)))
+
+(defmethod stream-read-char ((stream string-input-editing-stream))
+  (stream-read-gesture stream))
+
+(defmethod stream-listen ((stream string-input-editing-stream))
+  (< (stream-scan-pointer stream) (length (stream-input-buffer stream))))
+
+(defmethod stream-unread-char ((stream string-input-editing-stream) char)
+  (stream-unread-gesture stream char))
+
+(defmethod invoke-with-input-editor-typeout ((stream string-input-editing-stream) continuation
+                                             &key erase)
+  (declare (ignore erase)))
+
+(defmethod input-editor-format ((stream string-input-editing-stream) format-string
+                                &rest args)
+  (declare (ignore args)))
+
+(defmethod stream-rescanning-p ((stream string-input-editing-stream))
+  t)
+
+(defmethod reset-scan-pointer ((stream string-input-editing-stream)
+                               &optional scan-pointer)
+  (declare (ignore scan-pointer)))
+
+(defmethod immediate-rescan ((stream string-input-editing-stream)))
+
+(defmethod queue-rescan ((stream string-input-editing-stream)))
+
+(defmethod rescan-if-necessary ((stream string-input-editing-stream)
+                                &optional inhibit-activation)
+  (declare (ignore inhibit-activation)))
+
+(defmethod erase-input-buffer ((stream string-input-editing-stream)
+                                &optional start-position)
+  (declare (ignore start-position)))
+
+(defmethod redraw-input-buffer ((stream string-input-editing-stream)
+                                &optional start-position)
+  (declare (ignore start-position)))
+
+(defmethod stream-process-gesture ((stream string-input-editing-stream) gesture type)
+  (when (characterp gesture)
+    (values gesture type)))
+
+(defmethod stream-read-gesture ((stream string-input-editing-stream)
+                                &key peek-p &allow-other-keys)
+  (unless (> (stream-scan-pointer stream) (length (stream-input-buffer stream)))
+    (prog1 (if (= (stream-scan-pointer stream) (length (stream-input-buffer stream)))
+               (second (first (gethash (first *activation-gestures*)
+                                       climi::*gesture-names*))) ; XXX - will always be non-NIL?
+               (aref (stream-input-buffer stream) (stream-scan-pointer stream)))
+      (unless peek-p
+        (incf (stream-scan-pointer stream))))))
+
+(defmethod stream-unread-gesture ((stream string-input-editing-stream) gesture)
+  (decf (stream-scan-pointer stream)))
+
+(defmethod stream-accept ((stream string-input-editing-stream) type &rest args)
+  (apply #'accept-1 stream type args))
+
 ;;; XXX This needs work! It needs to do everything that accept does for
 ;;; expanding ptypes and setting up recursive call procesusing
 (defun accept-from-string (type string
@@ -1121,18 +1068,18 @@ history will be unchanged."
 			   &key view
 			   (default nil defaultp)
 			   (default-type nil default-type-p)
-			   activation-gestures
+			   (activation-gestures nil activationsp)
 			   (additional-activation-gestures
 			    nil
 			    additional-activations-p)
-			   delimiter-gestures
+			   (delimiter-gestures nil delimitersp)
 			   (additional-delimiter-gestures
 			    nil
 			    additional-delimiters-p)
 			   (start 0)
 			   (end (length string)))
   (declare (ignore view))
-  ;;; XXX work in progress here.
+  ;; XXX work in progress here.
   (with-activation-gestures ((if additional-activations-p
 				 additional-activation-gestures
 				 activation-gestures)
@@ -1141,8 +1088,7 @@ history will be unchanged."
 				  additional-delimiter-gestures
 				  delimiter-gestures)
 			      :override delimitersp)))
-  (when (or (zerop (- end start))
-	    (let ((maybe-end))))
+  (when (zerop (- end start))
     (if defaultp
 	(return-from accept-from-string (values default
 						(if default-type-p
@@ -1150,13 +1096,12 @@ history will be unchanged."
 						    type)
 						0))
 	(simple-parse-error "Empty string")))
-  (let ((index 0))
+  (let ((stream (make-instance 'string-input-editing-stream
+                 :string string :start start :end end)))
     (multiple-value-bind (val ptype)
-	(with-input-from-string (stream string :start start :end end
-				 :index index)
-	  (with-keywords-removed (args (:start :end))
-	    (apply #'stream-accept stream type :view +textual-view+ args)))
-      (values val ptype index))))
+        (with-keywords-removed (args (:start :end))
+          (apply #'stream-accept stream type :history nil :view +textual-view+ args))
+      (values val ptype (+ (stream-scan-pointer stream) start)))))
 
 (define-presentation-generic-function %presentation-refined-position-test
     presentation-refined-position-test
