@@ -52,6 +52,39 @@
       (when *print-presentation-verbose*
 	(format stream " ~S" (presentation-object self))))))
 
+(defmacro with-output-as-presentation ((stream object type
+					       &rest key-args
+					       &key modifier single-box
+					       (allow-sensitive-inferiors t)
+					       parent
+					       (record-type
+						''standard-presentation)
+					       &allow-other-keys)
+				       &body body)
+  (declare (ignore parent single-box modifier))
+  (setq stream (stream-designator-symbol stream '*standard-output*))
+  (multiple-value-bind (decls with-body)
+      (get-body-declarations body)
+    (with-gensyms (record-arg continuation)
+      (with-keywords-removed (key-args (:record-type
+					:allow-sensitive-inferiors))
+	`(flet ((,continuation ()
+		  ,@decls
+		  ,@with-body))
+	   (declare (dynamic-extent #'continuation))
+	   (if (and (output-recording-stream-p ,stream)
+		    *allow-sensitive-inferiors*)
+	       (with-new-output-record
+		   (,stream ,record-type ,record-arg
+			    :object ,object
+			    :type (expand-presentation-type-abbreviation
+				   ,type)
+			    ,@key-args)
+		 (let ((*allow-sensitive-inferiors*
+			,allow-sensitive-inferiors))
+		   (,continuation)))
+	       (,continuation)))))))
+
 (defgeneric ptype-specializer (type)
   (:documentation "The specializer to use for this type in a presentation
 method lambda list"))
@@ -240,9 +273,7 @@ that has no default is supplied with '*"
 		       (let ((var (car lambda-var)))
 			 (do-arg (if (and (eq state '&key) (consp var))
 				     (car var)
-				     var))))
-		      (t (list (fake-params-args lambda-var))))))))
-  
+				     var)))))))))
 
 ;;; Yet another variation on a theme...
 
@@ -1156,15 +1187,25 @@ function lambda list"))
   (let ((gf (gethash name *presentation-gf-table*)))
     (unless gf
       (error "~S is not a presentation generic function" name))
-    (let* ((rebound-args (mapcar (lambda (arg)
-				   `(,(gensym "ARG") ,arg))
-				 args))
-	   (gf-name (generic-function-name gf))
-	   (type-spec-var (car (nth (1- (type-arg-position gf)) rebound-args))))
+    (let* ((rebound-args (loop for arg in args
+                            unless (symbolp arg)
+                            collect (list (gensym "ARG") arg)))
+           (gf-name (generic-function-name gf))
+           (type-spec-var (nth (1- (type-arg-position gf)) args)))
       `(let ,rebound-args
-	 (,gf-name (prototype-or-error (presentation-type-name
-					,type-spec-var))
-		   ,@(mapcar #'car rebound-args))))))
+         (,gf-name (prototype-or-error (presentation-type-name
+                                        ,(or (first (find type-spec-var rebound-args :key #'second))
+                                             type-spec-var)))
+                   ,@(mapcar #'(lambda (arg)
+                                 ;; Order of evaluation doesn't matter
+                                 ;; for symbols, and this shuts up
+                                 ;; warnings about arguments in a
+                                 ;; keyword position not being
+                                 ;; constant. By the way, why do we
+                                 ;; care about order of evaluation
+                                 ;; here? -trh
+                                 (or (first (find arg rebound-args :key #'second))
+                                     arg)) args))))))
 
 (defmacro apply-presentation-generic-function (name &rest args)
   (let ((gf (gethash name *presentation-gf-table*)))
@@ -1909,12 +1950,13 @@ a presentation"
                        :label label
                        :associated-window window
                        :printer #'(lambda (item stream)
-                                    (document-presentation-translator
-                                     (presentation-translator-menu-item-translator item)
-                                     (presentation-translator-menu-item-presentation item)
-                                     (presentation-translator-menu-item-context item)
-                                     frame nil window x y
-                                     :stream stream))
+                                    (let ((object (first item)))
+                                     (document-presentation-translator
+                                      (presentation-translator-menu-item-translator object)
+                                      (presentation-translator-menu-item-presentation object)
+                                      (presentation-translator-menu-item-context object)
+                                      frame nil window x y
+                                      :stream stream)))
                        :label label
                        :pointer-documentation *pointer-documentation-output*)
         (declare (ignore object))
@@ -1936,4 +1978,120 @@ a presentation"
   (map-over-output-records
    (lambda (child)
      (highlight-output-record child stream state))
-   record))
\ No newline at end of file
+   record))
+
+;;; Context-dependent input
+;;; An input context is a cons of a presentation type and a continuation to
+;;; call to return a presentation to that input context.
+
+(defvar *input-context* nil)
+
+(defun input-context-type (context-entry)
+  (car context-entry))
+
+;;; Many presentation functions, internal and external, take an input
+;;; context as an argument, but they really only need to look at one
+;;; presentation type.
+(defun make-fake-input-context (ptype)
+  (list (cons (expand-presentation-type-abbreviation ptype)
+	      #'(lambda (object type event options)
+		  (declare (ignore event options))
+		  (error "Fake input context called with object ~S type ~S. ~
+                          This shouldn't happen!"
+			 object type)))))
+
+(defun input-context-wait-test (stream)
+  (let* ((queue (stream-input-buffer stream))
+	 (event (event-queue-peek queue)))
+    (when event
+      (let ((sheet (event-sheet event)))
+	(when (and (output-recording-stream-p sheet)
+		   (or (typep event 'pointer-event)
+		       (typep event 'keyboard-event))
+		   (not (gadgetp sheet)))
+	  (return-from input-context-wait-test t))))
+    nil))
+
+(defun highlight-applicable-presentation (frame stream input-context
+					  &optional (prefer-pointer-window t))
+  (let* ((queue (stream-input-buffer stream))
+	 (event (event-queue-peek queue)))
+    (when (and event
+	       (or (and (typep event 'pointer-event)
+			(or prefer-pointer-window 
+			    (eq stream (event-sheet event))))
+		   (typep event 'keyboard-event)))
+      ;; Stream only needs to see button press events.
+      ;; XXX Need to think about this more.  Should any pointer events be
+      ;; passed through?  If there's no presentation, maybe?
+      (unless (typep event 'keyboard-event)
+	(event-queue-read queue))
+      (progn
+	(frame-input-context-track-pointer frame
+					   input-context
+					   (event-sheet event)
+					   event)
+	(when (typep event 'pointer-button-press-event)
+	  (funcall *pointer-button-press-handler* stream event)))
+      #+nil
+      (if (and (typep event 'pointer-motion-event)
+	       (pointer-event-button event))
+	  (frame-drag frame input-context (event-sheet event) event)
+	  ))))
+
+(defun input-context-event-handler (stream)
+  (highlight-applicable-presentation *application-frame*
+				     stream
+				     *input-context*))
+
+(defun input-context-button-press-handler (stream button-event)
+  (declare (ignore stream))
+  (frame-input-context-button-press-handler *application-frame*
+					    (event-sheet button-event)
+					    button-event))
+
+(defun highlight-current-presentation (frame input-context)
+  (let ((event (synthesize-pointer-motion-event (port-pointer
+						 (port
+						  *application-frame*)))))
+    (when event
+      (frame-input-context-track-pointer frame
+                                         input-context
+                                         (event-sheet event)
+                                         event))))
+
+(defmacro with-input-context ((type &key override)
+			      (&optional (object-var (gensym))
+					 (type-var (gensym))
+					 event-var
+					 options-var)
+			      form
+			      &body pointer-cases)
+  (let ((vars `(,object-var
+		,type-var
+		,@(and event-var `(,event-var))
+		,@(and options-var `(,options-var))))
+	(return-block (gensym "RETURN-BLOCK"))
+	(context-block (gensym "CONTEXT-BLOCK")))
+    `(block ,return-block
+       (multiple-value-bind ,vars
+	   (block ,context-block
+	     (let ((*input-context*
+		    (cons (cons (expand-presentation-type-abbreviation ,type)
+                                #'(lambda (object type event options)
+				    (return-from ,context-block
+				      (values object type event options))))
+			  ,(if override nil '*input-context*)))
+		   (*pointer-button-press-handler*
+		    #'input-context-button-press-handler)
+		   (*input-wait-test* #'input-context-wait-test)
+		   (*input-wait-handler* #'input-context-event-handler))
+	       (return-from ,return-block ,form )))
+         (declare (ignorable ,@vars))
+	 (highlight-current-presentation *application-frame* *input-context*)
+	 (cond ,@(mapcar #'(lambda (pointer-case)
+			     (destructuring-bind (case-type &body case-body)
+				 pointer-case
+			       `((presentation-subtypep ,type-var ',case-type)
+				 ,@case-body)))
+			 pointer-cases))))))
