@@ -39,8 +39,6 @@
 (define-condition abort-gesture (condition)
   ((event :reader %abort-gesture-event :initarg :event)))
 
-(defgeneric abort-gesture-event (condition))
-
 (defmethod abort-gesture-event ((condition abort-gesture))
   (%abort-gesture-event condition))
 
@@ -50,8 +48,6 @@
 		     :initarg :numeric-argument
 		     :initform 1)))
 
-(defgeneric accelerator-gesture-event (condition))
-
 (defmethod accelerator-gesture-event ((condition accelerator-gesture))
   (%accelerator-gesture-event condition))
 
@@ -126,13 +122,9 @@
 	do (handle-event (event-sheet event) event))
   nil)
 
-;;; XXX The should be moved to protocol-classes.lisp and the
-;;; standard-sheet-input-mixin superclass should be removed.
-(define-protocol-class extended-input-stream (fundamental-character-input-stream ;Gray stream
-					      standard-sheet-input-mixin)
-  ())
-
-(defclass standard-extended-input-stream (extended-input-stream)
+(defclass standard-extended-input-stream (extended-input-stream
+                                          ;; FIXME: is this still needed?
+                                          standard-sheet-input-mixin)
   ((pointer)
    (cursor :initarg :text-cursor)
    (last-gesture :accessor last-gesture :initform nil
@@ -144,19 +136,14 @@
 (defvar *input-wait-handler* nil)
 (defvar *pointer-button-press-handler* nil)
 
-(defgeneric stream-set-input-focus (stream))
-
 (defmacro with-input-focus ((stream) &body body)
   (when (eq stream t)
     (setq stream '*standard-input*))
   (let ((old-stream (gensym "OLD-STREAM")))
     `(let ((,old-stream (stream-set-input-focus ,stream)))
-       (unwind-protect (locally
-			 ,@body)
-	 (if ,old-stream
-	     (stream-set-input-focus ,old-stream)
-	     (setf (port-keyboard-input-focus (port ,stream)) nil))))))
-
+       (unwind-protect (locally ,@body)
+         (when ,old-stream
+           (stream-set-input-focus ,old-stream))))))
 
 (defun read-gesture (&key
 		     (stream *standard-input*)
@@ -174,18 +161,14 @@
 		       :pointer-button-press-handler
 		       pointer-button-press-handler))
 
-(defgeneric stream-read-gesture (stream
-				 &key timeout peek-p
-				 input-wait-test
-				 input-wait-handler
-				 pointer-button-press-handler))
 
 ;;; Do streams care about any other events?
 (defun handle-non-stream-event (buffer)
   (let* ((event (event-queue-peek buffer))
 	 (sheet (event-sheet event)))
     (if (and event
-	     (or (gadgetp sheet)
+	     (or (and (gadgetp sheet)
+                      (gadget-active-p sheet))
 		 (not (and (typep sheet 'clim-stream-pane)
 			   (or (typep event 'key-press-event)
 			       (typep event 'pointer-button-press-event))))))
@@ -279,9 +262,9 @@
 	   ;; the problem. -- moore
 	   (cond ((null gesture)
 		  (go wait-for-char))
-		 ((and pointer-button-press-handler
-		       (typep gesture 'pointer-button-press-event))
-		  (funcall pointer-button-press-handler stream gesture))
+                 ((and pointer-button-press-handler
+                       (typep gesture 'pointer-button-press-event))
+                  (funcall pointer-button-press-handler stream gesture))
 		 ((loop for gesture-name in *abort-gestures*
 			thereis (event-matches-gesture-name-p gesture
 							      gesture-name))
@@ -295,8 +278,6 @@
 	 (go wait-for-char)))))
 
 
-(defgeneric stream-input-wait (stream &key timeout input-wait-test))
-
 (defmethod stream-input-wait ((stream standard-extended-input-stream)
 			      &key timeout input-wait-test)
   (block exit
@@ -328,8 +309,6 @@
 (defun unread-gesture (gesture &key (stream *standard-input*))
   (stream-unread-gesture stream gesture))
 
-(defgeneric stream-unread-gesture (stream gesture))
-
 (defmethod stream-unread-gesture ((stream standard-extended-input-stream)
 				  gesture)
   (declare (ignore gesture))
@@ -449,6 +428,11 @@
   `(add-gesture-name ',name ',type ',gesture-spec ,@(and unique
 							 `(:unique ',unique))))
 
+(defun delete-gesture-name (name)
+  "Delete the gesture named by the symbol `name' from the list of
+known gestures."
+  (remhash name *gesture-names*))
+
 ;;; XXX perhaps this should be in the backend somewhere?
 (defconstant +name-to-char+ '((:newline . #\newline)
 			      (:linefeed . #\linefeed)
@@ -657,9 +641,6 @@
 ;;;
 ;;; Implemented by the back end.  Sort of.
 
-(define-protocol-class pointer ()
-  ((port :reader port :initarg :port)))
-
 ;;; FIXME: I think the standard-pointer should absorb some of the
 ;;; common methods that are currently entirely provided by the
 ;;; backends.
@@ -734,9 +715,6 @@
 	  (logandc2 (slot-value pointer 'button-state)
 		    (pointer-event-button event)))))
 
-(defmethod pointer-butt)
-(defgeneric stream-pointer-position (stream &key pointer))
-
 (defmethod stream-pointer-position ((stream standard-extended-input-stream)
 				    &key (pointer
 					  (port-pointer (port stream))))
