@@ -42,11 +42,35 @@
 (defclass pointer-documentation-view (textual-view)
   ())
 
-;;; Views described in the Franz User manual...
+;;; Views described in the Franz User manual (CLIM 2.2)...
+
+(defclass toggle-button-view (gadget-view)
+  ())
+
+(defclass push-button-view (gadget-view)
+  ())
+
+(defclass radio-box-view (gadget-view)
+  ())
+
+(defclass check-box-view (gadget-view)
+  ())
+
+(defclass slider-view (gadget-view)
+  ())
 
 (defclass text-field-view (gadget-dialog-view)
   ((width :accessor width :initarg :width :initform nil)))
 
+(defclass text-editor-view (gadget-view)
+  ())
+
+(defclass list-pane-view (gadget-view)
+  ())
+
+(defclass option-pane-view (gadget-view)
+  ())
+
 (defparameter +textual-view+ (make-instance 'textual-view))
 
 (defparameter +textual-menu-view+ (make-instance 'textual-menu-view))
@@ -59,11 +83,24 @@
 
 (defparameter +gadget-dialog-view+ (make-instance 'gadget-dialog-view))
 
-(defparameter +pointer-documentation-view+
-  (make-instance 'pointer-documentation-view))
+(defparameter +pointer-documentation-view+ (make-instance 'pointer-documentation-view))
+
+(defparameter +toggle-button-view+ (make-instance 'toggle-button-view))
+
+(defparameter +push-button-view+ (make-instance 'push-button-view))
+
+(defparameter +radio-box-view+ (make-instance 'radio-box-view))
+
+(defparameter +slider-view+ (make-instance 'slider-view))
 
 (defparameter +text-field-view+ (make-instance 'text-field-view))
 
+(defparameter +text-editor-view+ (make-instance 'text-editor-view))
+
+(defparameter +list-pane-view+ (make-instance 'list-pane-view))
+
+(defparameter +option-pane-view+ (make-instance 'option-pane-view))
+
 (defmethod stream-default-view (stream)
   (declare (ignore stream))
   +textual-view+)
